package cn.ezhizao.project.business.BizInspectionStandards.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.BizInspectionStandards.domain.BizInspectionStandards;
import cn.ezhizao.project.business.BizInspectionStandards.service.IBizInspectionStandardsService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * 检查标准Controller
 *
 * @author ezhizao
 * @date 2024-08-09
 */
@RestController
@RequestMapping("/business/standards")
public class BizInspectionStandardsController extends BaseController
{
    @Resource
    private IBizInspectionStandardsService bizInspectionStandardsService;

    /**
     * 查询检查标准列表
     */
    @GetMapping("/list")
    @Log(title = "检查标准列表", businessType = BusinessType.SELECT)
    public TableDataInfo list(BizInspectionStandards bizInspectionStandards) throws NoSuchFieldException, IllegalAccessException
    {
        List<BizInspectionStandards> list = bizInspectionStandardsService.getList(bizInspectionStandards);
        return getDataTable(list);
    }

    /**
     * 导出检查标准列表
     */
    @PreAuthorize("@ss.hasPermi('business:standards:export')")
    @Log(title = "检查标准导出", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizInspectionStandards bizInspectionStandards) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizInspectionStandards);
        List<BizInspectionStandards> list = bizInspectionStandardsService.getList(bizInspectionStandards);
        ExcelUtil<BizInspectionStandards> util = new ExcelUtil<BizInspectionStandards>(BizInspectionStandards.class);
        util.exportExcel(response, list, "检查标准数据");
    }

    /**
     * 获取检查标准详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:standards:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "检查标准信息", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizInspectionStandardsService.getById(id));
    }

    /**
     * 新增检查标准
     */
    @PreAuthorize("@ss.hasPermi('business:standards:add')")
    @Log(title = "新增检查标准", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizInspectionStandards bizInspectionStandards) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizInspectionStandards);
        return toAjax(bizInspectionStandardsService.save(bizInspectionStandards));
    }

    /**
     * 修改检查标准
     */
    @PreAuthorize("@ss.hasPermi('business:standards:edit')")
    @Log(title = "修改检查标准", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizInspectionStandards bizInspectionStandards) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizInspectionStandards);
        return toAjax(bizInspectionStandardsService.updateById(bizInspectionStandards));
    }

    /**
     * 删除检查标准
     */
    @PreAuthorize("@ss.hasPermi('business:standards:remove')")
    @Log(title = "删除检查标准", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizInspectionStandardsService.removeBatchByIds(ids));
    }
}
