package cn.ezhizao.project.business.BizInspectionStandardsCheck.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.BizInspectionStandardsCheck.domain.BizInspectionStandardsCheck;
import cn.ezhizao.project.business.BizInspectionStandardsCheck.service.IBizInspectionStandardsCheckService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * 检查类别标准Controller
 *
 * @author ezhizao
 * @date 2024-08-09
 */
@RestController
@RequestMapping("/business/check")
public class BizInspectionStandardsCheckController extends BaseController
{
    @Resource
    private IBizInspectionStandardsCheckService bizInspectionStandardsCheckService;

    /**
     * 查询检查类别标准列表
     */
    @PreAuthorize("@ss.hasPermi('business:check:list')")
    @Log(title = "查询检查类别标准列表", businessType = BusinessType.SELECT)
    @GetMapping("/list")
    public TableDataInfo list(BizInspectionStandardsCheck bizInspectionStandardsCheck) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizInspectionStandardsCheck);
        startPage();
        List<BizInspectionStandardsCheck> list = bizInspectionStandardsCheckService.getList(bizInspectionStandardsCheck);
        return getDataTable(list);
    }

    /**
     * 导出检查类别标准列表
     */
    @PreAuthorize("@ss.hasPermi('business:check:export')")
    @Log(title = "检查类别标准", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizInspectionStandardsCheck bizInspectionStandardsCheck) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizInspectionStandardsCheck);
        List<BizInspectionStandardsCheck> list = bizInspectionStandardsCheckService.getList(bizInspectionStandardsCheck);
        ExcelUtil<BizInspectionStandardsCheck> util = new ExcelUtil<BizInspectionStandardsCheck>(BizInspectionStandardsCheck.class);
        util.exportExcel(response, list, "检查类别标准数据");
    }

    /**
     * 获取检查类别标准详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:check:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "查询检查类别标准信息", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizInspectionStandardsCheckService.getById(id));
    }

    /**
     * 新增检查类别标准
     */
    @PreAuthorize("@ss.hasPermi('business:check:add')")
    @Log(title = "新增检查类别标准", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizInspectionStandardsCheck bizInspectionStandardsCheck) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizInspectionStandardsCheck);
        return toAjax(bizInspectionStandardsCheckService.save(bizInspectionStandardsCheck));
    }

    /**
     * 修改检查类别标准
     */
    @PreAuthorize("@ss.hasPermi('business:check:edit')")
    @Log(title = "编辑检查类别标准", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizInspectionStandardsCheck bizInspectionStandardsCheck) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizInspectionStandardsCheck);
        return toAjax(bizInspectionStandardsCheckService.updateById(bizInspectionStandardsCheck));
    }

    /**
     * 删除检查类别标准
     */
    @PreAuthorize("@ss.hasPermi('business:check:remove')")
    @Log(title = "删除检查类别标准", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizInspectionStandardsCheckService.removeBatchByIds(ids));
    }
}
