package cn.ezhizao.project.business.equipment.controller;

import java.util.List;
import java.util.Map;
import javax.annotation.Resource;

import cn.ezhizao.common.utils.SecurityUtils;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.equipment.domain.BizEquipmentDetail;
import cn.ezhizao.project.business.equipment.domain.BizUserEquipment;
import cn.ezhizao.project.business.equipment.domain.BizUserEquipmentGroup;
import cn.ezhizao.project.business.equipment.service.IBizEquipmentDetailService;
import cn.ezhizao.project.business.equipment.service.IBizUserEquipmentGroupService;
import cn.ezhizao.project.system.domain.SysUser;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;


/**
 * 设备档案Controller
 *
 * @author ezhizao
 * @date 2023-11-14
 */
@RestController
@RequestMapping("/business/equipmentDetail")
public class BizEquipmentDetailController extends BaseController {
    @Resource
    private IBizEquipmentDetailService bizEquipmentDetailService;
    @Resource
    private IBizUserEquipmentGroupService bizUserEquipmentGroupService;

    /**
     * 查询设备档案列表
     */
//    @PreAuthorize("@ss.hasPermi('business:equipmentDetail:list')")
    @PostMapping("/list")
    @Log(title = "查询设备列表", businessType = BusinessType.SELECT)
    public TableDataInfo list(@RequestBody Map<String, Object> data) throws NoSuchFieldException, IllegalAccessException {
        QueryWrapper<BizEquipmentDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("deleted", 0);

        final String param1 = "equipmentId";
        final String param2 = "equipmentCode";
        long equipmentId = 0L;
        if (data.get(param1) != null) {
            equipmentId = Long.parseLong(data.get(param1).toString());
        }
        if (data.get(param2) != null && !"".equals(data.get(param2).toString())) {
            queryWrapper.like("equipment_code", data.get(param2).toString());
        }
        queryWrapper.eq("equipment_id", equipmentId);
        startPage();
        List<BizEquipmentDetail> list = bizEquipmentDetailService.list(queryWrapper);
        return getDataTable(list);
    }

    /**
     * 获取设备档案详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:equipmentDetail:query')")
    @GetMapping(value = "/getInfo/{id}")
    @Log(title = "查询设备信息", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bizEquipmentDetailService.getById(id));
    }

    /**
     * 新增设备档案
     */
//    @PreAuthorize("@ss.hasPermi('business:equipmentDetail:add, business:equipmentDetail:add')")
    @Log(title = "保存设备档案", businessType = BusinessType.INSERT)
    @PostMapping(value = "/save")
    public AjaxResult save(@RequestBody BizEquipmentDetail bizEquipmentDetail) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizEquipmentDetail);
        return toAjax(bizEquipmentDetailService.saveOrUpdate(bizEquipmentDetail));
    }

    /**
     * 删除设备档案
     */
//    @PreAuthorize("@ss.hasPermi('business:equipmentDetail:remove')")
    @Log(title = "删除设备档案", businessType = BusinessType.DELETE)
    @DeleteMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids) {
        return toAjax(bizEquipmentDetailService.removeBatchByIds(ids));
    }

    @GetMapping("/getDeptId")
    public AjaxResult getUsageDeptIdByEquipmentDetailsId(Long equipmentDetailsId) {
        Long deptEquipmentList = bizEquipmentDetailService.getUsageDeptIdByEquipmentDetailsId(equipmentDetailsId);
        SysUser user = SecurityUtils.getLoginUser().getUser();
        BizUserEquipmentGroup group = new BizUserEquipmentGroup();
        group.setUserId(user.getUserId());

        group = bizUserEquipmentGroupService.getList(group).get(0);
        if (group!=null){
            if (group.getUserEquipmentList() != null && group.getUserEquipmentList().size() > 0){
                return success();
            }
        }

        return AjaxResult.success();
    }
}