package cn.ezhizao.project.business.equipment.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.SecurityUtils;
import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.equipment.domain.BizUserEquipment;
import cn.ezhizao.project.business.equipment.domain.BizUserEquipmentGroup;
import cn.ezhizao.project.business.equipment.service.IBizUserEquipmentGroupService;
import cn.ezhizao.project.business.equipment.service.IBizUserEquipmentService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 员工绑定设备组Controller
 *
 * @author ezhizao
 * @date 2024-01-15
 */
@RestController
@RequestMapping("/business/equipmentGroup")
public class BizUserEquipmentGroupController extends BaseController
{
    @Resource
    private IBizUserEquipmentGroupService bizUserEquipmentGroupService;

    @Resource
    private IBizUserEquipmentService bizUserEquipmentService;

    /**
     * 查询员工绑定设备组列表
     */
    @GetMapping("/list")
    @Log(title = "查询员工绑定设备组列表", businessType = BusinessType.SELECT)
    public TableDataInfo list(BizUserEquipmentGroup bizUserEquipmentGroup) throws NoSuchFieldException, IllegalAccessException
    {
        bizUserEquipmentGroup.setUserId(SecurityUtils.getLoginUser().getUser().getUserId());
        List<BizUserEquipmentGroup> list = bizUserEquipmentGroupService.getList(bizUserEquipmentGroup);
        if (list.size() > 0){
            List<BizUserEquipment> userEquipmentList = list.get(0).getUserEquipmentList();
            return getDataTable(userEquipmentList);
        }
        return null;
    }

    /**
     * 导出员工绑定设备组列表
     */
    @Log(title = "员工绑定设备组", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizUserEquipmentGroup bizUserEquipmentGroup) throws NoSuchFieldException, IllegalAccessException
    {

        List<BizUserEquipmentGroup> list = bizUserEquipmentGroupService.getList(bizUserEquipmentGroup);
        ExcelUtil<BizUserEquipmentGroup> util = new ExcelUtil<BizUserEquipmentGroup>(BizUserEquipmentGroup.class);
        util.exportExcel(response, list, "员工绑定设备组数据");
    }

    /**
     * 获取员工绑定设备组详细信息
     */
    @GetMapping(value = "/{id}")
    @Log(title = "查询员工绑定设备组信息", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizUserEquipmentGroupService.getById(id));
    }

    /**
     * 新增员工绑定设备组
     */
    @Log(title = "新增员工绑定设备组", businessType = BusinessType.INSERT)
    @PostMapping
    @Transactional
    public AjaxResult add(@RequestBody BizUserEquipmentGroup bizUserEquipmentGroup) throws NoSuchFieldException, IllegalAccessException
    {
        //只保存不修改
        bizUserEquipmentGroup.setId(null);
        bizUserEquipmentGroupService.save(bizUserEquipmentGroup);
        for (BizUserEquipment item:bizUserEquipmentGroup.getUserEquipmentList()
             ) {
            item.setGroupId(bizUserEquipmentGroup.getId());
        }

        return toAjax(bizUserEquipmentService.saveBatch(bizUserEquipmentGroup.getUserEquipmentList()));
    }

    /**
     * 修改员工绑定设备组
     */
    @Log(title = "修改员工绑定设备组", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizUserEquipmentGroup bizUserEquipmentGroup) throws NoSuchFieldException, IllegalAccessException
    {

        return toAjax(bizUserEquipmentGroupService.updateById(bizUserEquipmentGroup));
    }

    /**
     * 删除员工绑定设备组
     */
    @Log(title = "删除员工绑定设备组", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizUserEquipmentService.removeBatchByIds(ids));
    }
}
