package cn.ezhizao.project.business.process.controller;

import java.util.ArrayList;
import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.SecurityUtils;
import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.equipment.domain.BizUserEquipment;
import cn.ezhizao.project.business.equipment.service.IBizUserEquipmentService;
import cn.ezhizao.project.business.process.domain.BizProcess;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.ezhizao.project.business.process.service.IBizProcessService;

/**
 * 标准工序Controller
 *
 * @author ezhizao
 * @date 2023-11-13
 */
@RestController
@RequestMapping("/business/process")
public class BizProcessController extends BaseController {
    @Resource
    private IBizProcessService bizProcessService;

    @Resource
    private IBizUserEquipmentService bizUserEquipmentService;
    /**
     * 查询标准工序列表
     */
//    @PreAuthorize("@ss.hasPermi('business:process:list')")
    @GetMapping("/list")
    @Log(title = "查询标准工序列表", businessType = BusinessType.SELECT)
    public TableDataInfo list(BizProcess bizProcess) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProcess);
        startPage();
        List<BizProcess> list = bizProcessService.getList(bizProcess);
        return getDataTable(list);
    }

    /**
     * 查询标准工序列表
     */
//    @PreAuthorize("@ss.hasPermi('business:process:list')")
    @GetMapping("/getUserProcess")
    @Log(title = "查询员工绑定设备对应的工序列表", businessType = BusinessType.SELECT)
    public AjaxResult getUserProcess() throws NoSuchFieldException, IllegalAccessException {

        BizUserEquipment userEquipment = new BizUserEquipment();
        userEquipment.setUserId(SecurityUtils.getLoginUser().getUser().getUserId());

        List<BizUserEquipment> userEquipments = bizUserEquipmentService.getList(userEquipment);
        List<Long> ids = new ArrayList<>();
        if(userEquipments.size() > 0){
            for (BizUserEquipment item:userEquipments
            ) {
                ids.add(item.getEquipmentDetailId());
            }
            List<BizProcess> list = bizProcessService.getUserProcess(ids);
            return success(list);
        }

        return error("当前未绑定设备");
    }

    /**
     * 获取标准工序详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:process:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "查询工序信息", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bizProcessService.getById(id));
    }

    /**
     * 新增标准工序
     */
    @PreAuthorize("@ss.hasPermi('business:process:add')")
    @Log(title = "保存标准工序", businessType = BusinessType.INSERT)
    @PostMapping(value = "/save")
    public AjaxResult save(@RequestBody BizProcess bizProcess) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProcess);
        return toAjax(bizProcessService.saveOrUpdate(bizProcess));
    }

    /**
     * 删除标准工序
     */
    @PreAuthorize("@ss.hasPermi('business:process:remove')")
    @Log(title = "删除标准工序", businessType = BusinessType.DELETE)
    @DeleteMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids) {
        return toAjax(bizProcessService.removeBatchByIds(ids));
    }
}
