package cn.ezhizao.project.business.product.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import cn.ezhizao.project.business.product.domain.BizDayworkCarrier;
import cn.ezhizao.project.business.product.service.IBizDayworkCarrierService;

/**
 * 报工载具中间表Controller
 *
 * @author ezhizao
 * @date 2023-12-04
 */
@RestController
@RequestMapping("/business/dayworkCarrier")
public class BizDayworkCarrierController extends BaseController
{
    @Resource
    private IBizDayworkCarrierService bizDayworkCarrierService;
    @Resource
    HttpServletRequest request;

    /**
     * 查询报工载具中间表列表
     */
//    @PreAuthorize("@ss.hasPermi('business:dayworkCarrier:list')")
    @GetMapping("/list")
    @Log(title = "查询报工载具列表", businessType = BusinessType.SELECT)
    public TableDataInfo list(BizDayworkCarrier bizDayworkCarrier) throws NoSuchFieldException, IllegalAccessException
    {
//        String tenantId = request.getHeader("tenantId");
//        bizDayworkCarrier.setTenantId(Long.valueOf(tenantId));
//        startPage();
        List<BizDayworkCarrier> list = bizDayworkCarrierService.getList(bizDayworkCarrier);
        return getDataTable(list);
    }
    @PostMapping("/getDayworkCarrierByCarrierCode")
    @Log(title = "根据载具编号查询载具对应报工列表", businessType = BusinessType.SELECT)
    public AjaxResult getDayworkCarrierByCarrierCode(@RequestBody  BizDayworkCarrier bizDayworkCarrier) throws NoSuchFieldException, IllegalAccessException
    {
        return success(bizDayworkCarrierService.getDayworkCarrierByCarrierCode(bizDayworkCarrier));
    }
    /**
     * 导出报工载具中间表列表
     */
//    @PreAuthorize("@ss.hasPermi('business:dayworkCarrier:export')")
    @Log(title = "导出报工载具中间表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizDayworkCarrier bizDayworkCarrier) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkCarrier);
        List<BizDayworkCarrier> list = bizDayworkCarrierService.getList(bizDayworkCarrier);
        ExcelUtil<BizDayworkCarrier> util = new ExcelUtil<BizDayworkCarrier>(BizDayworkCarrier.class);
        util.exportExcel(response, list, "报工载具中间表数据");
    }

    /**
     * 获取报工载具中间表详细信息
     */
//    @PreAuthorize("@ss.hasPermi('business:dayworkCarrier:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "查询报工载具信息", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizDayworkCarrierService.getById(id));
    }

    /**
     * 新增报工载具中间表
     */
//    @PreAuthorize("@ss.hasPermi('business:dayworkCarrier:add')")
    @Log(title = "新增报工载具中间表", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizDayworkCarrier bizDayworkCarrier) throws NoSuchFieldException, IllegalAccessException
    {
        String tenantId = request.getHeader("tenantId");
        bizDayworkCarrier.setTenantId(Long.valueOf(tenantId));
        return toAjax(bizDayworkCarrierService.save(bizDayworkCarrier));
    }

    /**
     * 修改报工载具中间表
     */
//    @PreAuthorize("@ss.hasPermi('business:dayworkCarrier:edit')")
    @Log(title = "修改报工载具中间表", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizDayworkCarrier bizDayworkCarrier) throws NoSuchFieldException, IllegalAccessException
    {
        String tenantId = request.getHeader("tenantId");
        bizDayworkCarrier.setTenantId(Long.valueOf(tenantId));
        return toAjax(bizDayworkCarrierService.updateById(bizDayworkCarrier));
    }

    /**
     * 删除报工载具中间表
     */
//    @PreAuthorize("@ss.hasPermi('business:dayworkCarrier:remove')")
    @Log(title = "删除报工载具中间表", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizDayworkCarrierService.removeBatchByIds(ids));
    }
}
