package cn.ezhizao.project.business.product.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.product.domain.BizDayworkItemCollaboration;
import cn.ezhizao.project.business.product.service.IBizDayworkItemCollaborationService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 报工记录协作者Controller
 *
 * @author ezhizao
 * @date 2024-01-18
 */
@RestController
@RequestMapping("/business/collaboration")
public class BizDayworkItemCollaborationController extends BaseController
{
    @Resource
    private IBizDayworkItemCollaborationService bizDayworkItemCollaborationService;

    /**
     * 查询报工记录协作者列表
     */
    @PreAuthorize("@ss.hasPermi('business:collaboration:list')")
    @GetMapping("/list")
    @Log(title = "查询报工记录协作列表", businessType = BusinessType.SELECT)
    public TableDataInfo list(BizDayworkItemCollaboration bizDayworkItemCollaboration) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkItemCollaboration);
        startPage();
        List<BizDayworkItemCollaboration> list = bizDayworkItemCollaborationService.getList(bizDayworkItemCollaboration);
        return getDataTable(list);
    }

    /**
     * 导出报工记录协作者列表
     */
    @PreAuthorize("@ss.hasPermi('business:collaboration:export')")
    @Log(title = "导出报工记录协作者列表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizDayworkItemCollaboration bizDayworkItemCollaboration) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkItemCollaboration);
        List<BizDayworkItemCollaboration> list = bizDayworkItemCollaborationService.getList(bizDayworkItemCollaboration);
        ExcelUtil<BizDayworkItemCollaboration> util = new ExcelUtil<BizDayworkItemCollaboration>(BizDayworkItemCollaboration.class);
        util.exportExcel(response, list, "报工记录协作者数据");
    }

    /**
     * 获取报工记录协作者详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:collaboration:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "获取报工协作记录详情", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizDayworkItemCollaborationService.getById(id));
    }

    /**
     * 新增报工记录协作者
     */
    @PreAuthorize("@ss.hasPermi('business:collaboration:add')")
    @Log(title = "新增报工记录协作者", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizDayworkItemCollaboration bizDayworkItemCollaboration) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkItemCollaboration);
        return toAjax(bizDayworkItemCollaborationService.save(bizDayworkItemCollaboration));
    }

    /**
     * 修改报工记录协作者
     */
    @PreAuthorize("@ss.hasPermi('business:collaboration:edit')")
    @Log(title = "修改报工记录协作者", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizDayworkItemCollaboration bizDayworkItemCollaboration) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkItemCollaboration);
        return toAjax(bizDayworkItemCollaborationService.updateById(bizDayworkItemCollaboration));
    }

    /**
     * 删除报工记录协作者
     */
    @PreAuthorize("@ss.hasPermi('business:collaboration:remove')")
    @Log(title = "删除报工记录协作者", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizDayworkItemCollaborationService.removeBatchByIds(ids));
    }
}
