package cn.ezhizao.project.business.product.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import cn.ezhizao.project.business.product.domain.BizProduct;
import cn.ezhizao.project.business.product.service.IBizProductService;

/**
 * 产品管理Controller
 *
 * @author ezhizao
 * @date 2023-11-13
 */
@RestController
@RequestMapping("/base/product")
public class BizProductController extends BaseController
{
    @Resource
    private IBizProductService bizProductService;

    /**
     * 查询产品管理列表
     */
//    @PreAuthorize("@ss.hasPermi('base:product:list')")
    @GetMapping("/list")
    @Log(title = "查询产品列表", businessType = BusinessType.SELECT)
    public TableDataInfo list(BizProduct bizProduct) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProduct);
        startPage();
        List<BizProduct> list = bizProductService.getList(bizProduct);
        return getDataTable(list);
    }

    /**
     * 导出产品管理列表
     */
//    @PreAuthorize("@ss.hasPermi('base:product:export')")
    @Log(title = "导出产品列表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizProduct bizProduct) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProduct);
        List<BizProduct> list = bizProductService.getList(bizProduct);
        ExcelUtil<BizProduct> util = new ExcelUtil<BizProduct>(BizProduct.class);
        util.exportExcel(response, list, "产品管理数据");
    }

    /**
     * 获取产品管理详细信息
     */
//    @PreAuthorize("@ss.hasPermi('base:product:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "获取产品信息", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizProductService.getById(id));
    }

    /**
     * 新增产品管理
     */
//    @PreAuthorize("@ss.hasPermi('base:product:add')")
    @Log(title = "新增产品", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizProduct bizProduct) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProduct);
        return toAjax(bizProductService.save(bizProduct));
    }

    /**
     * 修改产品管理
     */
//    @PreAuthorize("@ss.hasPermi('base:product:edit')")
    @Log(title = "修改产品", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizProduct bizProduct) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProduct);
        return toAjax(bizProductService.updateById(bizProduct));
    }

    /**
     * 删除产品管理
     */
//    @PreAuthorize("@ss.hasPermi('base:product:remove')")
    @Log(title = "删除产品", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizProductService.removeBatchByIds(ids));
    }
}
