package cn.ezhizao.project.business.product.controller;

import cn.ezhizao.common.utils.SecurityUtils;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.carrierReject.domain.BizCarrierReject;
import cn.ezhizao.project.business.carrierReject.service.IBizCarrierRejectService;
import cn.ezhizao.project.business.outsource.domain.BizOutsourcedOrder;
import cn.ezhizao.project.business.outsource.domain.BizOutsourcedOrderDetail;
import cn.ezhizao.project.business.outsource.service.IBizOutsourcedOrderDetailService;
import cn.ezhizao.project.business.outsource.service.IBizOutsourcedOrderService;
import cn.ezhizao.project.business.product.domain.*;
import cn.ezhizao.project.business.product.service.*;
import cn.ezhizao.project.business.resourceGroup.domain.BizProductionResourceGroup;
import cn.ezhizao.project.business.resourceGroup.domain.BizProductionResourceGroupDetail;
import cn.ezhizao.project.business.resourceGroup.domain.BizProductionResourceGroupSubPlan;
import cn.ezhizao.project.business.resourceGroup.service.IBizProductionResourceGroupDetailService;
import cn.ezhizao.project.business.resourceGroup.service.IBizProductionResourceGroupService;
import cn.ezhizao.project.business.resourceGroup.service.IBizProductionResourceGroupSubPlanService;
import cn.ezhizao.project.system.domain.SysDept;
import cn.ezhizao.project.system.service.ISysDeptService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 生产计划明细Controller
 *
 * @author ruoyi
 * @date 2023-11-15
 */
@RestController
@RequestMapping("/business/productionPlanDetail")
public class BizProductionPlanDetailController extends BaseController {
    @Resource
    private IBizProductionPlanDetailService bizProductionPlanDetailService;

    @Resource
    private IBizProductionResourceGroupSubPlanService bizProductionResourceGroupSubPlanService;

    @Resource
    private IBizProductionResourceGroupService bizProductionResourceGroupService;

    @Resource
    private IBizProductionResourceGroupDetailService bizProductionResourceGroupDetailService;

    @Resource
    private IBizDayworkService dayworkService;
    @Resource
    private IBizLotService bizLotService;
    @Resource
    private IBizCarrierService bizCarrierService;
    @Resource
    private IBizCarrierRejectService bizCarrierRejectService;
    @Resource
    private IBizDayworkCarrierService bizDayworkCarrierService;
    @Resource
    private IBizDayworkService bizDayworkService;
    @Resource
    private IBizOutsourcedOrderDetailService bizOutsourcedOrderDetailService;
    @Resource
    private IBizReturnReceiptDetailService bizReturnReceiptDetailService;
    @Resource
    private IBizOutsourcedOrderService bizOutsourcedOrderService;
    @Resource
    private ISysDeptService sysDeptService;



    /**
     * 查询生产计划明细列表
     */
//  @PreAuthorize("@ss.hasPermi('business:productionPlanDetail:list')")
    @Transactional
    @PostMapping("/list")
    @Log(title = "查询生产计划明细列表", businessType = BusinessType.SELECT)
    /*
    public AjaxResult list(@RequestBody BizProductionPlanDetail bizProductionPlanDetail) throws NoSuchFieldException, IllegalAccessException {
        // 根据id获取资源组明细
        List<BizProductionResourceGroupDetail> detailsList = bizProductionResourceGroupDetailService.query().eq("common_type", "person").eq("common_id", SecurityUtils.getLoginUser().getUser().getUserId()).list();
        List<Long> ids = new ArrayList<>();
        for (BizProductionResourceGroupDetail detail : detailsList) {
            ids.add(detail.getProductionResourceGroupId());
        }
        if(ids.isEmpty()){
            return null;
        }

        //获取资源组
        List<BizProductionResourceGroup> groupList = bizProductionResourceGroupService.query().in("id", ids).eq("dept_id",bizProductionPlanDetail.getDeptId()).list();

        //根据资源组获取分配的生产计划
        List<Long> groupIds = new ArrayList<>();
        for (BizProductionResourceGroup group : groupList) {
            groupIds.add(group.getId());
        }
        if (groupIds.isEmpty()){
            return null;
        }
        List<BizProductionResourceGroupSubPlan> groupSubPlans = bizProductionResourceGroupSubPlanService.query().in("resource_group_id", groupIds).eq("deleted",0).list();

        //根据分配的计划，查询计划单
        List<Long> planIds = new ArrayList<>();
        for (BizProductionResourceGroupSubPlan groupSubPlan : groupSubPlans) {
            planIds.add(groupSubPlan.getProductionPlanDetailId());
        }
        if (planIds.isEmpty()){
            return null;
        }
//        QueryWrapper<BizProductionPlanDetail> queryWrapper = new QueryWrapper<>();
//        queryWrapper.in("id", planIds);
//        if(bizProductionPlanDetail.getKeywords()!=null) {
//            queryWrapper.and(item -> item
//                    .like("product_description", bizProductionPlanDetail.getKeywords())
//                    .or()
//                    .like("production_plan_no", bizProductionPlanDetail.getKeywords()));
//        }
        bizProductionPlanDetail.setPlanIds(planIds);
        List<BizProductionPlanDetail> list = bizProductionPlanDetailService.getInfo(bizProductionPlanDetail);
        for (BizProductionPlanDetail item: list) {
            BizDaywork daywork = new BizDaywork();
            daywork.setProductionPlanDetailId(item.getId());
            item.setDayWorkList(dayworkService.getList(daywork));
            for (BizProductionResourceGroupSubPlan subPlan:groupSubPlans) {
                if (subPlan.getProductionPlanDetailId().equals(item.getId())){
                    item.setResourceGroupId(subPlan.getResourceGroupId());
                }
            }
        }
        return success(list);
    }
    */

    public AjaxResult list(@RequestBody BizProductionPlanDetail bizProductionPlanDetail) throws NoSuchFieldException, IllegalAccessException {
        // 根据当前登录者的id，以及部门的id，获取资源组明细
        Long currentUserId = SecurityUtils.getLoginUser().getUser().getUserId();
        List<BizProductionResourceGroupDetail> detailsList = bizProductionResourceGroupDetailService
            .query()
            .eq("common_type", "person")
            .eq("common_id", currentUserId)
            .eq("dept_id",bizProductionPlanDetail.getDeptId())
            .list();
        List<BizProductionResourceGroup> bizProductionResourceGroups = bizProductionResourceGroupService.query().in("id", detailsList.isEmpty() ? Collections.singletonList(0L) : detailsList.stream().map(BizProductionResourceGroupDetail::getProductionResourceGroupId).collect(Collectors.toList())).list();
        detailsList = detailsList.stream().filter(v -> bizProductionResourceGroups.stream().anyMatch(e -> e.getId().equals(v.getProductionResourceGroupId()))).collect(Collectors.toList());
        /*
         * 将集合中的productionResourceGroupId取出来，存入一个List集合resourceGroupIds中
         * 后面会根据resourceGroupIds，获得到资源组分配的生产计划（生产计划与资源组的中间表）的集合
         */
        List<Long> resourceGroupIds = detailsList.stream()
            .map(BizProductionResourceGroupDetail::getProductionResourceGroupId)
            .collect(Collectors.toList());
        if(resourceGroupIds.isEmpty()){
            return null;
        }

        // 根据资源组id集合，获取到所有的资源组分配的生产计划（生产计划与资源组的中间表）的集合
        List<BizProductionResourceGroupSubPlan> groupSubPlans = bizProductionResourceGroupSubPlanService.query()
            .in("resource_group_id", resourceGroupIds)
            .eq("deleted",0)
            .list();

        /*
         * 将上面的中间表的集合中的生产计划id取出来，存入一个List集合planDetailIds中
         * 后面会根据planDetailIds，
         */
        List<Long> planDetailIds = groupSubPlans.stream()
            .map(BizProductionResourceGroupSubPlan::getProductionPlanDetailId)
            .collect(Collectors.toList());
        if (planDetailIds.isEmpty()){
            return null;
        }
        //去掉已完成的计划单
        List<Long> detailIds = new ArrayList<>();
        List<BizLot> bizLotList = bizLotService.query().in("production_plan_detail_id", planDetailIds).eq("deleted", 0).list();
        List<BizDaywork> dayworkList = dayworkService.query().in("production_plan_detail_id", planDetailIds).eq("status", 2).list();
        planDetailIds.forEach(id -> {
            long lotCount = bizLotList.stream().filter(item -> item.getProductionPlanDetailId().equals(id)).count();
            long dayworkCount = dayworkList.stream().filter(item -> item.getProductionPlanDetailId().equals(id)).count();
            if(lotCount > dayworkCount) {
                detailIds.add(id);
            }
        });
        BizProductionPlanDetail detail = new BizProductionPlanDetail();
        detail.setPlanDetailIds(detailIds);
        detail.setKeywords(bizProductionPlanDetail.getKeywords());
        List<BizProductionPlanDetail> list = bizProductionPlanDetailService.selectNotComplatedList(detail);
        // 将获取到的ProductionPlanDetail集合的id提取到集合中
        List<Long> tempPlanDetailIds = list.stream()
                .map(BizProductionPlanDetail::getId)
                .collect(Collectors.toList());

        List<BizDaywork> allDayworks = dayworkService.query()
                .in("production_plan_detail_id", tempPlanDetailIds.isEmpty()? Collections.singletonList(0L) :  list.stream()
                        .map(BizProductionPlanDetail::getId)
                        .collect(Collectors.toList()))
                .eq("deleted",0)
                .list();
        for (BizProductionPlanDetail item: list) {
            List<BizDaywork> dayworks = allDayworks.stream()
                    .filter(d -> d.getProductionPlanDetailId().equals(item.getId()))
                    .collect(Collectors.toList());
            item.setDayWorkList(dayworks);
        }

        return success(list);
    }
    @Transactional
    @PostMapping("/innerList")
    @Log(title = "查询生产计划明细列表", businessType = BusinessType.SELECT)
    public AjaxResult innerList(@RequestBody BizProductionPlanDetail bizProductionPlanDetail) throws NoSuchFieldException, IllegalAccessException {

        // 根据资源组id集合，获取到所有的资源组分配的生产计划（生产计划与资源组的中间表）的集合
        List<BizProductionResourceGroupSubPlan> groupSubPlans = bizProductionResourceGroupSubPlanService.query()
                .in("dept_id", bizProductionPlanDetail.getDeptId())
                .eq("deleted",0)
                .list();
        if(groupSubPlans.isEmpty()){
            return success();
        }
        /*
         * 将上面的中间表的集合中的生产计划id取出来，存入一个List集合planDetailIds中
         * 后面会根据planDetailIds，
         */
        List<Long> planDetailIds = groupSubPlans.stream()
                .map(BizProductionResourceGroupSubPlan::getProductionPlanDetailId)
                .collect(Collectors.toList());
        if (planDetailIds.isEmpty()){
            return null;
        }
        //去掉已完成的计划单
        List<Long> detailIds = new ArrayList<>();
        List<BizLot> bizLotList = bizLotService.query().in("production_plan_detail_id", planDetailIds).eq("deleted", 0).list();
        List<BizDaywork> dayworkList = dayworkService.query().in("production_plan_detail_id", planDetailIds).eq("status", 2).list();
        planDetailIds.forEach(id -> {
            long lotCount = bizLotList.stream().filter(item -> item.getProductionPlanDetailId().equals(id)).count();
            long dayworkCount = dayworkList.stream().filter(item -> item.getProductionPlanDetailId().equals(id)).count();
            if(lotCount > dayworkCount) {
                detailIds.add(id);
            }
        });
        BizProductionPlanDetail detail = new BizProductionPlanDetail();
        detail.setPlanDetailIds(detailIds);
        detail.setKeywords(bizProductionPlanDetail.getKeywords());
        List<BizProductionPlanDetail> list = bizProductionPlanDetailService.selectNotComplatedList(detail);

        // 将获取到的ProductionPlanDetail集合的id提取到集合中
        List<Long> tempPlanDetailIds = list.stream()
                .map(BizProductionPlanDetail::getId)
                .collect(Collectors.toList());

        List<BizDaywork> allDayworks = dayworkService.query()
                .in("production_plan_detail_id", tempPlanDetailIds.isEmpty()? Collections.singletonList(0L) :  list.stream()
                        .map(BizProductionPlanDetail::getId)
                        .collect(Collectors.toList()))
                .eq("deleted",0)
                .list();
        //内部外协已经收回的批次，则不展示收回单
            SysDept sysDept = sysDeptService.selectDeptById(bizProductionPlanDetail.getDeptId());
        List<BizOutsourcedOrder> outsourceOrderList = bizOutsourcedOrderService.query().eq("outsource_dept_id", sysDept.getDeptId()).list();
        List<BizOutsourcedOrderDetail> outsourcedOrderDetailList = bizOutsourcedOrderDetailService.query()
                        .in("production_plan_detail_id", tempPlanDetailIds.isEmpty() ? Collections.singletonList(0L):tempPlanDetailIds)
                        .eq("supplier_id", sysDept.getSupplierId())
                .in("master_id", outsourceOrderList.isEmpty() ? Collections.singletonList(0L): outsourceOrderList.stream().map(BizOutsourcedOrder::getId).collect(Collectors.toList()))
                    .list();
                List<BizReturnReceiptDetail> returnReceiptDetail = bizReturnReceiptDetailService.query()
                        .in("outsource_detail_id", outsourcedOrderDetailList.isEmpty() ? Collections.singletonList(0L) : outsourcedOrderDetailList.stream().map(BizOutsourcedOrderDetail::getId).collect(Collectors.toList()))
                        .list();
                List<BizProductionPlanDetail> newList = new ArrayList<>();
           for (BizProductionPlanDetail item: list){
               List<BizOutsourcedOrderDetail> collect = outsourcedOrderDetailList.stream().filter(v -> v.getProductionPlanDetailId().equals(item.getId())).collect(Collectors.toList());
               if(!collect.isEmpty()){
                   //该计划单有发出单
                  for(BizOutsourcedOrderDetail v:collect){
                      if(returnReceiptDetail.stream().filter(l-> l.getOutsourceDetailId().equals(v.getId())).collect(Collectors.toList()).isEmpty()){
                          if(!newList.contains(item)) {
                              newList.add(item);
                          }
                      }
                  }
               }
            }

        for (BizProductionPlanDetail item: newList) {
            List<BizDaywork> dayworks = allDayworks.stream()
                    .filter(d -> d.getProductionPlanDetailId().equals(item.getId()))
                    .collect(Collectors.toList());
            item.setDayWorkList(dayworks);
        }

        return success(newList);
    }
    @GetMapping("/selectByCarrierCode")
    @Log(title = "查询箱码对应报工", businessType = BusinessType.SELECT)
    public AjaxResult selectByCarrierCode(BizDaywork daywork) {
        BizCarrier carrier = bizCarrierService.query().eq("code", daywork.getCarrierCode()).one();
        if(carrier == null) {
            return error("未找到该箱号");
        }
        //判断是否是外协箱
        if(carrier.getIsInspection() == 1){
            return error("不能扫描检测载具");
        }
        //判断该箱是否废弃
        List<BizCarrierReject> carrierRejectList = bizCarrierRejectService.query().eq("carrier_id", carrier.getId()).list();
        if(!carrierRejectList.isEmpty()){
            if(carrierRejectList.stream().sorted(Comparator.comparing(BizCarrierReject::getCreateTime).reversed()).collect(Collectors.toList()).get(0).getIsAbandoned() == 1) {
                return error("该箱号已废弃");
            }
        }
        //判断是否绑定批次
        List<BizDayworkCarrier> dayworkCarrierList = bizDayworkCarrierService.query().eq("carrier_code", carrier.getCode()).eq("is_changed", 0).list();
        if(dayworkCarrierList.isEmpty()) {
            return error("该载具未绑定批次");
        }
        List<Long> dayworkIds = dayworkCarrierList.stream().map(BizDayworkCarrier::getDayworkId).collect(Collectors.toList());
        List<BizDaywork> dayworkList = bizDayworkService.query().in("id", dayworkIds).list();
        //当前工段，当前员工是否有这个计划单
        Long currentUserId = SecurityUtils.getLoginUser().getUser().getUserId();
        List<BizProductionResourceGroupDetail> productionResourceGroupDetailList =  bizProductionResourceGroupDetailService
                .query()
                .eq("common_type", "person")
                .eq("common_id", currentUserId)
                .eq("dept_id",daywork.getDeptId())
                .list();
        if(productionResourceGroupDetailList.isEmpty()){
            return error("该计划单不在加工范围内");
        }
        List<Long> productionResourceGroupIds = productionResourceGroupDetailList.stream().map(BizProductionResourceGroupDetail::getProductionResourceGroupId).collect(Collectors.toList());
        //员工对应资源组下分配的计划
        List<BizProductionResourceGroupSubPlan> productionResourceGroupSubPlanList = bizProductionResourceGroupSubPlanService.query().in("resource_group_id", productionResourceGroupIds).list();
        List<BizDaywork> matchedDayworkList = dayworkList.stream()
                .filter(d -> productionResourceGroupSubPlanList.stream()
                        .anyMatch(subPlan -> subPlan.getProductionPlanDetailId().equals(d.getProductionPlanDetailId())))
                .collect(Collectors.toList());
        if (matchedDayworkList.isEmpty()){
            return error("该计划单不在加工范围内");
        }
        return success(matchedDayworkList);
    }

    @GetMapping("/selectInfoByLotCode")
    @Log(title = "查询批次信息", businessType = BusinessType.SELECT)
    public AjaxResult selectInfoByLotCode(BizLot bizLot) {
        BizProductionPlanDetail bizProductionPlanDetail = bizProductionPlanDetailService.selectInfoByLotCode(bizLot);
        List<BizDaywork> dayworkList = bizDayworkService.query().eq("production_plan_detail_id", bizProductionPlanDetail.getId()).list();
        bizProductionPlanDetail.setDayWorkList(dayworkList);
        //判断该批次是否是特殊批次
        BizLot lot = bizLotService.query().eq("lot_code", bizLot.getLotCode()).one();
        bizProductionPlanDetail.setIsAmend(lot.getIsAmend());
        bizProductionPlanDetail.setIsWasteRecycling(lot.getIsWasteRecycling());
        return success(bizProductionPlanDetail);
    }
    /**
     * 获取生产计划明细详细信息
     */
//  @PreAuthorize("@ss.hasPermi('business:productionPlanDetail:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "查询生产计划信息", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bizProductionPlanDetailService.getById(id));
    }

    /**
     * 新增生产计划明细
     */
//  @PreAuthorize("@ss.hasPermi('business:productionPlanDetail:add')")
    @Log(title = "新增生产计划明细", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizProductionPlanDetail bizProductionPlanDetail) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProductionPlanDetail);
        return toAjax(bizProductionPlanDetailService.save(bizProductionPlanDetail));
    }

    /**
     * 修改生产计划明细
     */
//  @PreAuthorize("@ss.hasPermi('business:productionPlanDetail:edit')")
    @Log(title = "修改生产计划明细", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizProductionPlanDetail bizProductionPlanDetail) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProductionPlanDetail);
        return toAjax(bizProductionPlanDetailService.updateById(bizProductionPlanDetail));
    }

    /**
     * 删除生产计划明细
     */
//  @PreAuthorize("@ss.hasPermi('business:productionPlanDetail:remove')")
    @Log(title = "删除生产计划明细", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids) {
        return toAjax(bizProductionPlanDetailService.removeBatchByIds(ids));
    }
}
