package cn.ezhizao.project.business.product.controller;

import cn.ezhizao.common.utils.SecurityUtils;
import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.equipment.domain.BizUserEquipment;
import cn.ezhizao.project.business.equipment.service.IBizUserEquipmentService;
import cn.ezhizao.project.business.product.domain.BizProductionPlanDetailSubDetail;
import cn.ezhizao.project.business.product.service.IBizProductionPlanDetailSubDetailService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 生产子计划明细Controller
 *
 * @author ezhizao
 * @date 2023-11-15
 */
@RestController
@RequestMapping("/business/planDetailSubDetail")
public class BizProductionPlanDetailSubDetailController extends BaseController {
    @Resource
    private IBizProductionPlanDetailSubDetailService bizProductionPlanDetailSubDetailService;

    @Resource
    HttpServletRequest request;

    @Resource
    IBizUserEquipmentService bizUserEquipmentService;

    /**getListByUserEquipment
     * 查询生产子计划明细列表
     */
//    @PreAuthorize("@ss.hasPermi('business:planDetailSubDetail:list')")
    @Log(title = "查询生产子计划明细", businessType = BusinessType.SELECT)
    @GetMapping("/list")
    public TableDataInfo list(BizProductionPlanDetailSubDetail bizProductionPlanDetailSubDetail) throws NoSuchFieldException, IllegalAccessException {
//        startPage();
        List<BizProductionPlanDetailSubDetail> list = bizProductionPlanDetailSubDetailService.getList(bizProductionPlanDetailSubDetail);
        return getDataTable(list);
    }


    /**
     * 查询生产子计划明细列表
     */
//    @PreAuthorize("@ss.hasPermi('business:planDetailSubDetail:list')")
    @GetMapping("/getListByUserEquipment")
    @Log(title = "根据设备查询生产计划列表", businessType = BusinessType.SELECT)
    @Transactional
    public AjaxResult getListByUserEquipment(BizProductionPlanDetailSubDetail bizProductionPlanDetailSubDetail){
        BizUserEquipment userEquipment = new BizUserEquipment();
        userEquipment.setUserId(SecurityUtils.getLoginUser().getUser().getUserId());

        List<BizUserEquipment> userEquipments = bizUserEquipmentService.getList(userEquipment);
        List<Long> ids = new ArrayList<>();
        if (userEquipments.size()>0){
            for (BizUserEquipment item:userEquipments
            ) {
                ids.add(item.getEquipmentDetailId());
            }
            Map<String, Object> data = new HashMap<>();
            data.put("ids", ids);
            data.put("tenantId",bizProductionPlanDetailSubDetail.getTenantId());
            data.put("productDescription",bizProductionPlanDetailSubDetail.getKeywords());
            data.put("productionPlanNo",bizProductionPlanDetailSubDetail.getKeywords());
            List<BizProductionPlanDetailSubDetail> list = bizProductionPlanDetailSubDetailService.getListByUserEquipment(data);
            return success(list);
        }
       return error("绑定设备下,没有生产计划单");
    }

    /**
     * 导出生产子计划明细列表
     */
//    @PreAuthorize("@ss.hasPermi('business:planDetailSubDetail:export')")
    @Log(title = "导出生产子计划明细", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizProductionPlanDetailSubDetail bizProductionPlanDetailSubDetail) throws NoSuchFieldException, IllegalAccessException {
        List<BizProductionPlanDetailSubDetail> list = bizProductionPlanDetailSubDetailService.getList(bizProductionPlanDetailSubDetail);
        ExcelUtil<BizProductionPlanDetailSubDetail> util = new ExcelUtil<BizProductionPlanDetailSubDetail>(BizProductionPlanDetailSubDetail.class);
        util.exportExcel(response, list, "生产子计划明细数据");
    }

    /**
     * 获取生产子计划明细详细信息
     */
//    @PreAuthorize("@ss.hasPermi('business:planDetailSubDetail:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "查询生产计划明细", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        BizProductionPlanDetailSubDetail bizProductionPlanDetailSubDetail = new BizProductionPlanDetailSubDetail();
        bizProductionPlanDetailSubDetail.setId(id);

        List<BizProductionPlanDetailSubDetail> list = bizProductionPlanDetailSubDetailService.getList(bizProductionPlanDetailSubDetail);
        if (list.size() > 1) {
            return error();
        }
        return success(list.get(0));
    }

    /**
     * 新增生产子计划明细
     */
//    @PreAuthorize("@ss.hasPermi('business:planDetailSubDetail:add')")
    @Log(title = "新增生产子计划明细", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizProductionPlanDetailSubDetail bizProductionPlanDetailSubDetail) throws NoSuchFieldException, IllegalAccessException {
        return toAjax(bizProductionPlanDetailSubDetailService.saveOrUpdate(bizProductionPlanDetailSubDetail));
    }

    /**
     * 修改生产子计划明细
     */
//    @PreAuthorize("@ss.hasPermi('business:planDetailSubDetail:edit')")
    @Log(title = "编辑生产子计划明细", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizProductionPlanDetailSubDetail bizProductionPlanDetailSubDetail) throws NoSuchFieldException, IllegalAccessException {
        return toAjax(bizProductionPlanDetailSubDetailService.updateById(bizProductionPlanDetailSubDetail));
    }

    /**
     * 删除生产子计划明细
     */
//    @PreAuthorize("@ss.hasPermi('business:planDetailSubDetail:remove')")
    @Log(title = "删除生产子计划明细", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids) {
        return toAjax(bizProductionPlanDetailSubDetailService.removeBatchByIds(ids));
    }
}
