package cn.ezhizao.project.business.product.controller;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.product.domain.BizProductionPlanDetailSubDetailEquipment;
import cn.ezhizao.project.business.product.service.IBizProductionPlanDetailSubDetailEquipmentService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 子计划对应的工序指定的使用设备 Controller
 *
 * @author ezhizao
 * @date 2023-11-15
 */
@RestController
@RequestMapping("/business/equipment")
public class BizProductionPlanDetailSubDetailEquipmentController extends BaseController {
    @Resource
    private IBizProductionPlanDetailSubDetailEquipmentService bizProductionPlanDetailSubDetailEquipmentService;

    /**
     * 子计划对应的工序指定的使用设备 列表
     */
//    @PreAuthorize("@ss.hasPermi('business:equipment:list')")
    @GetMapping("/list")
    @Log(title = "子计划对应工序指定使用设备列表", businessType = BusinessType.SELECT)
    public TableDataInfo list(BizProductionPlanDetailSubDetailEquipment bizProductionPlanDetailSubDetailEquipment) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProductionPlanDetailSubDetailEquipment);
        startPage();
        List<BizProductionPlanDetailSubDetailEquipment> list = bizProductionPlanDetailSubDetailEquipmentService.getList(bizProductionPlanDetailSubDetailEquipment);
        return getDataTable(list);
    }

    /**
     * 导出子计划对应的工序指定的使用设备 列表
     */
//    @PreAuthorize("@ss.hasPermi('business:equipment:export')")
    @Log(title = "导出子计划对应的工序指定的使用设备", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizProductionPlanDetailSubDetailEquipment bizProductionPlanDetailSubDetailEquipment) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProductionPlanDetailSubDetailEquipment);
        Map<String, Object> data = new HashMap<>();
        data.put("bizProductionPlanDetailSubDetailEquipment", bizProductionPlanDetailSubDetailEquipment);
        List<BizProductionPlanDetailSubDetailEquipment> list = bizProductionPlanDetailSubDetailEquipmentService.getList(data);
        ExcelUtil<BizProductionPlanDetailSubDetailEquipment> util = new ExcelUtil<BizProductionPlanDetailSubDetailEquipment>(BizProductionPlanDetailSubDetailEquipment.class);
        util.exportExcel(response, list, "生产计划明细数据");
    }

    /**
     * 获取子计划对应的工序指定的使用设备 详细信息
     */
//    @PreAuthorize("@ss.hasPermi('business:equipment:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "子计划工序指定使用设备详细信息", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizProductionPlanDetailSubDetailEquipmentService.getById(id));
    }

    /**
     * 新增子计划对应的工序指定的使用设备
     */
//    @PreAuthorize("@ss.hasPermi('business:equipment:add')")
    @Log(title = "新增子计划对应工序指定使用社保", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizProductionPlanDetailSubDetailEquipment bizProductionPlanDetailSubDetailEquipment) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProductionPlanDetailSubDetailEquipment);
        return toAjax(bizProductionPlanDetailSubDetailEquipmentService.save(bizProductionPlanDetailSubDetailEquipment));
    }
    /**
     * 新增子计划对应的工序指定的使用设备
     */
//    @PreAuthorize("@ss.hasPermi('business:equipment:add')")
    @Log(title = "生产计划明细", businessType = BusinessType.INSERT)
    @PostMapping("/saveBatch")
    public AjaxResult saveBatch(@RequestBody List<BizProductionPlanDetailSubDetailEquipment> bizProductionPlanDetailSubDetailEquipment) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProductionPlanDetailSubDetailEquipment);
        //批量修改插入操作：先查出原数组，再根据新老数据比对进行插入
        BizProductionPlanDetailSubDetailEquipment equipment = new BizProductionPlanDetailSubDetailEquipment();
        equipment.setProductionPlanDetailSubDetailId(bizProductionPlanDetailSubDetailEquipment.get(0).getProductionPlanDetailSubDetailId());
        equipment.setProcessId(bizProductionPlanDetailSubDetailEquipment.get(0).getProcessId());

        List oldList = bizProductionPlanDetailSubDetailEquipmentService.getList(equipment);

        return toAjax(saveOrUpdateBatch(bizProductionPlanDetailSubDetailEquipmentService,bizProductionPlanDetailSubDetailEquipment,oldList));
    }
    /**
     * 修改子计划对应的工序指定的使用设备
     */
//    @PreAuthorize("@ss.hasPermi('business:equipment:edit')")
    @Log(title = "生产计划明细", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizProductionPlanDetailSubDetailEquipment bizProductionPlanDetailSubDetailEquipment) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProductionPlanDetailSubDetailEquipment);
        return toAjax(bizProductionPlanDetailSubDetailEquipmentService.updateById(bizProductionPlanDetailSubDetailEquipment));
    }

    /**
     * 删除子计划对应的工序指定的使用设备
     */
//    @PreAuthorize("@ss.hasPermi('business:equipment:remove')")
    @Log(title = "生产计划明细", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizProductionPlanDetailSubDetailEquipmentService.removeBatchByIds(ids));
    }
}
