package cn.ezhizao.project.business.product.service.impl;


import cn.ezhizao.framework.logGing.SomeService;
import cn.ezhizao.project.business.product.domain.BizDaywork;
import cn.ezhizao.project.business.product.domain.BizDayworkCarrier;
import cn.ezhizao.project.business.product.domain.BizDayworkItem;
import cn.ezhizao.project.business.product.mapper.BizDayworkCarrierMapper;
import cn.ezhizao.project.business.product.mapper.BizDayworkItemMapper;
import cn.ezhizao.project.business.product.service.IBizDayworkItemService;
import cn.ezhizao.project.business.technologicalProcessDetailDrawing.domain.BizTechnologicalProcessDetailDrawing;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 报工记录Service业务层处理
 *
 * @author ezhizao
 * @date 2023-12-04
 */
@Service
public class BizDayworkItemServiceImpl  extends ServiceImpl<BizDayworkItemMapper, BizDayworkItem> implements IBizDayworkItemService
{
    @Resource
    private BizDayworkItemMapper bizDayworkItemMapper;

    @Resource
    private BizDayworkCarrierMapper bizDayworkCarrierMapper;

    /**
     * 查询报工记录列表
     *
     * @param bizDayworkItem 报工记录
     * @return 报工记录
     */
    @Override
    public List<BizDayworkItem> getList(BizDayworkItem bizDayworkItem)
    {
        return bizDayworkItemMapper.getList(bizDayworkItem);
    }

    @Override
    public List<BizDaywork> getInnerListByDeptId(BizDayworkItem bizDayworkItem) {
        return bizDayworkItemMapper.getInnerListByDeptId(bizDayworkItem);
    }

    @Override
    public List<BizDaywork> getOutListByDeptId(BizDayworkItem bizDayworkItem) {
        return bizDayworkItemMapper.getOutListByDeptId(bizDayworkItem);
    }

    /**
     * 物理删除
     * @param bizDayworkItem
     * @return 删除结果
     */
    @Override
    public int physicalDelete(BizDayworkItem bizDayworkItem){ return bizDayworkItemMapper.physicalDelete(bizDayworkItem); }

    @Override
    public List<BizDayworkItem> getListByDayworkId(BizDayworkItem bizDayworkItem) {
        return bizDayworkItemMapper.getListByDayworkId(bizDayworkItem);
    }

    @Override
    public List<BizDayworkItem> getSortListByDayworkId(BizDayworkItem bizDayworkItem) {
        return bizDayworkItemMapper.getSortListByDayworkId(bizDayworkItem);
    }

    @Override
    public List<BizDayworkItem> getDayworkItemByCarrier(BizDayworkItem bizDayworkItem) {
        return bizDayworkItemMapper.getDayworkItemByCarrier(bizDayworkItem);
    }

    @Override
    public List<BizDayworkItem> getDayworkItemByCarrierCode(BizDayworkItem bizDayworkItem) {
        return bizDayworkItemMapper.getDayworkItemByCarrierCode(bizDayworkItem);
    }

    @Override
    public List<BizDayworkItem> getUndoneItem(BizDayworkItem bizDayworkItem) {
        return bizDayworkItemMapper.getUndoneItem(bizDayworkItem);
    }

    @Override
    public List<BizDayworkItem> getDayWorkItemHistory(String startTime,String endTime,String userName) {
        return bizDayworkItemMapper.getDayWorkItemHistory(startTime,endTime,userName);
    }

    @Override
    public List<BizDayworkItem> getItemScreen(Long dayWorkId) {
        return bizDayworkItemMapper.getItemScreen(dayWorkId);
    }

    /**
     * 获取该用户未完成的快速报工
     * @param userId 用户id
     * @return 返回结果
     */
    @Override
    public List<BizDayworkItem> getUnfinishedDayworkItemByUser(Long userId) {
        return bizDayworkItemMapper.getUnfinishedDayworkItemByUser(userId);
    }

    @Override
    public List<BizDayworkItem> getByDayworkId(Long dayworkId) {
        return bizDayworkItemMapper.getByDayworkId(dayworkId);
    }

    @Override
    public BizDayworkItem selById(Long dayworkItemId) {
        return bizDayworkItemMapper.selById(dayworkItemId);
    }

    @Override
    public List<BizDayworkItem> getDayworkItemByDayworkIds(List<Long> dayworkIds) {
        return bizDayworkItemMapper.getDayworkItemByDayworkIds(dayworkIds);
    }

    @Override
    public List<BizDayworkItem> getByDaywork(BizDaywork daywork) {
        return bizDayworkItemMapper.getByDaywork(daywork);
    }

    @Override
    public List<BizDayworkItem> getTurnoverList(BizDaywork daywork) {
        return bizDayworkItemMapper.getTurnoverList(daywork);
    }

    /**
     * 保存工序结束时当前工序箱数
     * @param dayworkItems
     */
    @Override
    public void saveCarrierNum(List<BizDayworkItem> dayworkItems) {
        /**
         * 1.因为只在工序结束时保存箱数所以在传入的dayworkItem集合中筛选出status=3的数据
         * 2.根据筛选出的数据中的dayworkId查询出当前绑定的箱数，只计算普通蓝箱子和铁箱子，不计算外协箱子，检测箱子
         * 3.将对应的箱数保存到dayworkItem中
         */



        List<BizDayworkItem> dayworkItemList=dayworkItems.stream().filter(item -> "3".equals(item.getStatus())).collect(Collectors.toList());




        if(!dayworkItemList.isEmpty()){
            //创建log数据
            List<BizDayworkItem> logList = new ArrayList<>();
            // 获取当前时间
            LocalDateTime now = LocalDateTime.now();

            // 定义时间格式
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

            // 将当前时间格式化为字符串
            String formattedTime = now.format(formatter);
            StringBuffer message=new StringBuffer();
            message.append(formattedTime).append(" 开始:\n");

            //将工序已完成status='3'的数据筛选出来
            message.append("报工数据:    ");
            // 报工数据
            message.append("dayworkId: ").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(item -> String.valueOf(item.getDayworkId())).toArray(String[]::new))).append("  ").append("批次号：").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(BizDayworkItem::getLotCode).toArray(String[]::new))).append("  ").append("LotId: ").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(item -> String.valueOf(item.getLotId())).toArray(String[]::new))).append("  ").append("员工编号： ").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(BizDayworkItem::getUserName).toArray(String[]::new))).append("  ").append("员工姓名： ").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(BizDayworkItem::getNickName).toArray(String[]::new))).append("  ").append("dayworkItemId: ").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(item -> String.valueOf(item.getId())).toArray(String[]::new))).append("  ").append("创建时间：").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(item -> String.valueOf(item.getCreateTime())).toArray(String[]::new))).append("  ").append("修改时间：").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(item -> String.valueOf(item.getUpdateTime())).toArray(String[]::new))).append("  ").append("箱数：").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(item -> String.valueOf(item.getCarrierNum())).toArray(String[]::new))).append("\n");


            // 提取dayworkId到一个新的集合中
            List<Long> dayworkIds = dayworkItemList.stream()
                    .map(BizDayworkItem::getDayworkId) // 提取dayworkId
                    .collect(Collectors.toList()); // 收集到新的列表中
            //根据dayworkId查询绑定信息只查询蓝箱子和铁箱子
            List<BizDayworkCarrier> dayworkCarriers=bizDayworkCarrierMapper.getListBayDayworkId(dayworkIds);
            message.append("载具信息：    ").append(String.join(System.lineSeparator(), dayworkCarriers.stream()
                    .map(BizDayworkCarrier::toString)
                    .toArray(String[]::new))).append("\n");;

            Map<Long, Integer> countsByDayworkId = dayworkCarriers.stream().collect(Collectors.groupingBy(BizDayworkCarrier::getDayworkId,Collectors.summingInt(carrier -> 1)));

            //将对应的箱数保存到dayworkItem中
            for (BizDayworkItem dayworkItem:dayworkItemList){
                dayworkItem.setCarrierNum(countsByDayworkId.get(dayworkItem.getDayworkId()));
            }

            super.saveOrUpdateBatch(dayworkItemList);
            //报工数据
            message.append("最终数据：    ");
            message.append("dayworkId: ").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(item -> String.valueOf(item.getDayworkId())).toArray(String[]::new))).append("  ").append("批次号：").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(BizDayworkItem::getLotCode).toArray(String[]::new))).append("  ").append("LotId: ").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(item -> String.valueOf(item.getLotId())).toArray(String[]::new))).append("  ").append("员工编号： ").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(BizDayworkItem::getUserName).toArray(String[]::new))).append("  ").append("员工姓名： ").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(BizDayworkItem::getNickName).toArray(String[]::new))).append("  ").append("dayworkItemId: ").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(item -> String.valueOf(item.getId())).toArray(String[]::new))).append("  ").append("创建时间：").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(item -> String.valueOf(item.getCreateTime())).toArray(String[]::new))).append("  ").append("修改时间：").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(item -> String.valueOf(item.getUpdateTime())).toArray(String[]::new))).append("  ").append("箱数：").append(String.join(System.lineSeparator(), dayworkItems.stream()
                    .map(item -> String.valueOf(item.getCarrierNum())).toArray(String[]::new))).append("结束----------------------------------------------------------").append("\n");
            SomeService service=new SomeService();
            service.someMethod(message.toString());
            service.onDestroy();

        }

    }


}




