package cn.ezhizao.project.business.technologicalProcessDetailDrawing.controller;

import cn.ezhizao.common.utils.SecurityUtils;
import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.deptProcess.domain.BizDeptProcess;
import cn.ezhizao.project.business.deptProcess.service.IBizDeptProcessService;
import cn.ezhizao.project.business.product.domain.BizLotTechnologicalProcessDetail;
import cn.ezhizao.project.business.product.service.IBizLotTechnologicalProcessDetailService;
import cn.ezhizao.project.business.technologicalProcessDetail.domain.BizTechnologicalProcessDetail;
import cn.ezhizao.project.business.technologicalProcessDetail.service.IBizTechnologicalProcessDetailService;
import cn.ezhizao.project.business.technologicalProcessDetailDrawing.domain.BizTechnologicalProcessDetailDrawing;
import cn.ezhizao.project.business.technologicalProcessDetailDrawing.service.IBizTechnologicalProcessDetailDrawingService;
import cn.ezhizao.project.business.technologicalProcessDetailDrawingRecords.domain.BizTechnologicalProcessDetailDrawingRecords;
import cn.ezhizao.project.business.technologicalProcessDetailDrawingRecords.service.IBizTechnologicalProcessDetailDrawingRecordsService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 电子图纸Controller
 *
 * @author ezhizao
 * @date 2024-05-30
 */
@RestController
@RequestMapping("/business/drawing")
public class BizTechnologicalProcessDetailDrawingController extends BaseController
{
    @Resource
    private IBizTechnologicalProcessDetailDrawingService bizTechnologicalProcessDetailDrawingService;
    @Resource
    private IBizTechnologicalProcessDetailDrawingRecordsService bizTechnologicalProcessDetailDrawingRecordsService;
    @Resource
    private IBizDeptProcessService bizDeptProcessService;
    @Resource
    private IBizTechnologicalProcessDetailService iBizTechnologicalProcessDetailService;
    @Resource
    private IBizLotTechnologicalProcessDetailService bizLotTechnologicalProcessDetailService;

    /**
     * 查询电子图纸列表
     */
//    @PreAuthorize("@ss.hasPermi('business:drawing:list')")
    @GetMapping("/list")
    @Log(title = "获取电子图纸列表", businessType = BusinessType.SELECT)
    public AjaxResult list(BizTechnologicalProcessDetailDrawing bizTechnologicalProcessDetailDrawing) throws NoSuchFieldException, IllegalAccessException
    {
//        setTenantId(bizTechnologicalProcessDetailDrawing);
//        startPage();
        List<BizTechnologicalProcessDetailDrawing> list = bizTechnologicalProcessDetailDrawingService.getList(bizTechnologicalProcessDetailDrawing);
        return success(list);
    }
    @GetMapping("/listProcess")
    @Log(title = "获取基础电子图纸列表", businessType = BusinessType.SELECT)
    public AjaxResult listProcess(BizTechnologicalProcessDetailDrawing bizTechnologicalProcessDetailDrawing) throws NoSuchFieldException, IllegalAccessException
    {
//        setTenantId(bizTechnologicalProcessDetailDrawing);
//        startPage();
        List<BizDeptProcess> bizDeptProcesses = bizDeptProcessService.query().eq("dept_id", bizTechnologicalProcessDetailDrawing.getDeptId()).list();
        List<BizTechnologicalProcessDetail> technologicalProcessDetails = iBizTechnologicalProcessDetailService.query()
                .eq("product_id", bizTechnologicalProcessDetailDrawing.getProductId())
                .eq("technology_version", bizTechnologicalProcessDetailDrawing.getProductVersion())
                .eq("technological_process_id", bizTechnologicalProcessDetailDrawing.getTechnologicalProcessId()).list();
//        bizTechnologicalProcessDetailDrawing.setProcessCodes(bizDeptProcesses.stream().map(BizDeptProcess::getProcessCode).collect(Collectors.toList()));
//        List<BizTechnologicalProcessDetailDrawing> list = bizTechnologicalProcessDetailDrawingService.getList(bizTechnologicalProcessDetailDrawing);

        return success(technologicalProcessDetails.stream().filter(v -> bizDeptProcesses.stream().anyMatch(t -> t.getProcessId().equals(v.getProcessId()))).collect(Collectors.toList()));
    }
    @GetMapping("/listLotProcess")
    @Log(title = "获取单批单改电子图纸列表", businessType = BusinessType.SELECT)
    public AjaxResult listLotProcess(BizTechnologicalProcessDetailDrawing bizTechnologicalProcessDetailDrawing) throws NoSuchFieldException, IllegalAccessException
    {
//        setTenantId(bizTechnologicalProcessDetailDrawing);
//        startPage();
        List<BizDeptProcess> bizDeptProcesses = bizDeptProcessService.query().eq("dept_id", bizTechnologicalProcessDetailDrawing.getDeptId()).list();
        List<BizLotTechnologicalProcessDetail> technologicalProcessDetails = bizLotTechnologicalProcessDetailService.query().eq("product_id", bizTechnologicalProcessDetailDrawing.getProductId()).eq("technology_version", bizTechnologicalProcessDetailDrawing.getProductVersion()).eq("lot_technological_process_id", bizTechnologicalProcessDetailDrawing.getTechnologicalProcessId()).list();
//        bizTechnologicalProcessDetailDrawing.setProcessCodes(bizDeptProcesses.stream().map(BizDeptProcess::getProcessCode).collect(Collectors.toList()));
//        List<BizTechnologicalProcessDetailDrawing> list = bizTechnologicalProcessDetailDrawingService.getList(bizTechnologicalProcessDetailDrawing);

        return success(technologicalProcessDetails.stream().filter(v -> bizDeptProcesses.stream().anyMatch(t -> t.getProcessId().equals(v.getProcessId()))).distinct().collect(Collectors.toList()));
    }
    /**
     * 查询电子图纸列表
     */
//    @PreAuthorize("@ss.hasPermi('business:drawing:list')")
    @GetMapping("/drawingList")
    @Log(title = "获取电子图纸列表", businessType = BusinessType.SELECT)
    public TableDataInfo drawingList(BizTechnologicalProcessDetailDrawing bizTechnologicalProcessDetailDrawing) throws NoSuchFieldException, IllegalAccessException
    {
//        setTenantId(bizTechnologicalProcessDetailDrawing);
        startPage();
        List<BizTechnologicalProcessDetailDrawing> list = bizTechnologicalProcessDetailDrawingService.getDrawingList(bizTechnologicalProcessDetailDrawing);
        return getDataTable(list);
    }

    /**
     * 查询电子图纸列表
     */
    @GetMapping("/byName")
    @Log(title = "根据名称获取电子图纸列表", businessType = BusinessType.SELECT)
    public AjaxResult byName(BizTechnologicalProcessDetailDrawing bizTechnologicalProcessDetailDrawing) throws NoSuchFieldException, IllegalAccessException
    {

        List<BizTechnologicalProcessDetailDrawing> drawingName = bizTechnologicalProcessDetailDrawingService
                .query()
                .eq("drawing_name", bizTechnologicalProcessDetailDrawing.getDrawingName())
                .eq("technological_process_detail_id",bizTechnologicalProcessDetailDrawing.getTechnologicalProcessDetailId())
                .list();

        return success(drawingName);
    }
    /**
     * 根据文件名查询列表
     */
    @GetMapping("/getListByName")
    @Log(title = "根据文件名查询列表", businessType = BusinessType.SELECT)
    public AjaxResult getListByName(BizTechnologicalProcessDetailDrawing bizTechnologicalProcessDetailDrawing) throws NoSuchFieldException, IllegalAccessException
    {

        List<BizTechnologicalProcessDetailDrawing> drawingName = bizTechnologicalProcessDetailDrawingService
                .query()
                .eq("drawing_name", bizTechnologicalProcessDetailDrawing.getDrawingName())
                .eq("technological_process_detail_id",bizTechnologicalProcessDetailDrawing.getTechnologicalProcessDetailId())
                .eq("status",0)
                .list();

        return success(drawingName);
    }
    /**
     * 导出电子图纸列表
     */
    @PreAuthorize("@ss.hasPermi('business:drawing:export')")
    @Log(title = "导出电子图纸", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizTechnologicalProcessDetailDrawing bizTechnologicalProcessDetailDrawing) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizTechnologicalProcessDetailDrawing);
        List<BizTechnologicalProcessDetailDrawing> list = bizTechnologicalProcessDetailDrawingService.getList(bizTechnologicalProcessDetailDrawing);
        ExcelUtil<BizTechnologicalProcessDetailDrawing> util = new ExcelUtil<BizTechnologicalProcessDetailDrawing>(BizTechnologicalProcessDetailDrawing.class);
        util.exportExcel(response, list, "电子图纸数据");
    }

    /**
     * 获取电子图纸详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:drawing:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "获取电子图纸详细信息", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizTechnologicalProcessDetailDrawingService.getById(id));
    }

    /**
     * 新增电子图纸
     */
    @PreAuthorize("@ss.hasPermi('business:drawing:add')")
    @Log(title = "新增电子图纸", businessType = BusinessType.INSERT)
    @PostMapping
    @Transactional
    public AjaxResult add(@RequestBody BizTechnologicalProcessDetailDrawing bizTechnologicalProcessDetailDrawing) throws NoSuchFieldException, IllegalAccessException
    {
        //判断覆盖的文件序号最高排到第几个，然后递增
        List<BizTechnologicalProcessDetailDrawing> drawingName = bizTechnologicalProcessDetailDrawingService
                .query()
                .like("drawing_name", bizTechnologicalProcessDetailDrawing.getDrawingName().substring(0, bizTechnologicalProcessDetailDrawing.getDrawingName().length() - 4)+"%")
                .eq("technological_process_detail_id",bizTechnologicalProcessDetailDrawing.getTechnologicalProcessDetailId())
                .list();
        List<Integer> numList = new ArrayList<>();
        if (drawingName.size()>0){
            for (BizTechnologicalProcessDetailDrawing item: drawingName
            ) {
                //判断倒数第五第六字符是否是数字,不是数字或者只有一个数字的话都会返回空值
                String lastFifthCharDigit = isLastFifthCharDigit(item.getDrawingName());
                if (!lastFifthCharDigit.equals("")){

                    numList.add(Integer.parseInt(lastFifthCharDigit));
                }

            }
            //获取最大值
            if (numList.size()>0){
                Integer max = Collections.max(numList);
                String str = "";
                if (max<10){
                    str = "0"+ (max +1);
                }else {
                    str = ""+ (max +1);
                }
                bizTechnologicalProcessDetailDrawing.setDrawingName(
                        bizTechnologicalProcessDetailDrawing.getDrawingName().substring(0, bizTechnologicalProcessDetailDrawing.getDrawingName().length() - 4)
                        + "_"
                        + str
                        + bizTechnologicalProcessDetailDrawing.getDrawingName().substring(bizTechnologicalProcessDetailDrawing.getDrawingName().length() - 4)
                );
            }else {
                bizTechnologicalProcessDetailDrawing.setDrawingName(
                        bizTechnologicalProcessDetailDrawing.getDrawingName().substring(0, bizTechnologicalProcessDetailDrawing.getDrawingName().length() - 4)
                                + "_01"
                                + bizTechnologicalProcessDetailDrawing.getDrawingName().substring(bizTechnologicalProcessDetailDrawing.getDrawingName().length() - 4)
                );
            }

        }

        bizTechnologicalProcessDetailDrawingService.save(bizTechnologicalProcessDetailDrawing);
        //新增同时增加操作记录,覆盖不增加
        if (!bizTechnologicalProcessDetailDrawing.getType()){
            BizTechnologicalProcessDetailDrawingRecords drawingRecords = new BizTechnologicalProcessDetailDrawingRecords();
            drawingRecords.setDrawingName(bizTechnologicalProcessDetailDrawing.getDrawingName());
            drawingRecords.setTechnologicalProcessDetailDrawingId(bizTechnologicalProcessDetailDrawing.getId());
            drawingRecords.setTechnologicalProcessDetailId(bizTechnologicalProcessDetailDrawing.getTechnologicalProcessDetailId());
            drawingRecords.setUserId(SecurityUtils.getUserId());
            drawingRecords.setType("新增");
            bizTechnologicalProcessDetailDrawingRecordsService.save(drawingRecords);
        }
        return success();
    }


    /**
     * 新增电子图纸会签
     */
    @PreAuthorize("@ss.hasPermi('business:drawing:add')")
    @Log(title = "保存电子图纸", businessType = BusinessType.INSERT)
    @PostMapping("/save")
    @Transactional
    public AjaxResult save(@RequestBody List<BizTechnologicalProcessDetailDrawing> bizTechnologicalProcessDetailDrawing) throws NoSuchFieldException, IllegalAccessException
    {
        for(BizTechnologicalProcessDetailDrawing item: bizTechnologicalProcessDetailDrawing){
            if (item.getStatus() == null){
                // 默认情况下执行的代码
                item.setProducerId(SecurityUtils.getLoginUser().getUser().getUserId());
                item.setProducer(SecurityUtils.getLoginUser().getUser().getNickName());
                item.setStatus(0);
                item.setPattern("新产品");
            }else {
                switch (item.getStatus()) {
                    //0：待审核，1待批准
                    case 0:
                        item.setReviewerId(SecurityUtils.getLoginUser().getUser().getUserId());
                        item.setReviewer(SecurityUtils.getLoginUser().getUser().getNickName());
                        item.setStatus(1);
                        break;
                    case 1:
                        item.setIssuerId(SecurityUtils.getLoginUser().getUser().getUserId());
                        item.setIssuer(SecurityUtils.getLoginUser().getUser().getNickName());
                        item.setStatus(2);
                        break;
//                    case 2:
//                        item.setVoidedId(SecurityUtils.getLoginUser().getUser().getUserId());
//                        item.setVoided(SecurityUtils.getLoginUser().getUser().getNickName());
//                        item.setStatus(3);
//                        break;
                }
            }

        }


        //新增同时增加操作记录,覆盖不增加
//        if (!bizTechnologicalProcessDetailDrawing.getType()){
//            BizTechnologicalProcessDetailDrawingRecords drawingRecords = new BizTechnologicalProcessDetailDrawingRecords();
//            drawingRecords.setDrawingName(bizTechnologicalProcessDetailDrawing.getDrawingName());
//            drawingRecords.setTechnologicalProcessDetailDrawingId(bizTechnologicalProcessDetailDrawing.getId());
//            drawingRecords.setTechnologicalProcessDetailId(bizTechnologicalProcessDetailDrawing.getTechnologicalProcessDetailId());
//            drawingRecords.setUserId(SecurityUtils.getUserId());
//            drawingRecords.setType("新增");
//            bizTechnologicalProcessDetailDrawingRecordsService.save(drawingRecords);
//        }

        return success(bizTechnologicalProcessDetailDrawingService.saveOrUpdateBatch(bizTechnologicalProcessDetailDrawing));
    }

    /**
     * 电子图纸作废
     */
    @PreAuthorize("@ss.hasPermi('business:drawing:remove')")
    @Log(title = "电子图纸作废", businessType = BusinessType.UPDATE)
    @PutMapping("/drawingNG")
    @Transactional
    public AjaxResult drawingNG(@RequestBody List<BizTechnologicalProcessDetailDrawing> processDetailDrawing) throws NoSuchFieldException, IllegalAccessException
    {
        for(BizTechnologicalProcessDetailDrawing  item: processDetailDrawing){
            item.setStatus(3);
            item.setVoided(SecurityUtils.getLoginUser().getUser().getNickName());
            item.setVoidedId(SecurityUtils.getLoginUser().getUser().getUserId());
            bizTechnologicalProcessDetailDrawingService.updateById(item);
        }

        return success();
    }

    /**
     * 电子图纸替换
     */
    @PreAuthorize("@ss.hasPermi('business:drawing:edit')")
    @Log(title = "电子图纸替换", businessType = BusinessType.UPDATE)
    @PutMapping("/replace")
    @Transactional
    public AjaxResult replace(@RequestBody BizTechnologicalProcessDetailDrawing processDetailDrawing) throws NoSuchFieldException, IllegalAccessException
    {
        //废弃旧图纸，生成新图纸
        processDetailDrawing.setStatus(3);
        processDetailDrawing.setVoided(SecurityUtils.getLoginUser().getUser().getNickName());
        processDetailDrawing.setVoidedId(SecurityUtils.getLoginUser().getUser().getUserId());
        bizTechnologicalProcessDetailDrawingService.updateById(processDetailDrawing);
        processDetailDrawing.setParentId(processDetailDrawing.getId());
        processDetailDrawing.setId(null);
        processDetailDrawing.setStatus(0);
        processDetailDrawing.setPattern("替换");
        processDetailDrawing.setIssuer(null);
        processDetailDrawing.setReviewer(null);
        processDetailDrawing.setIssuerId(null);
        processDetailDrawing.setReviewerId(null);
        processDetailDrawing.setProducerId(SecurityUtils.getLoginUser().getUser().getUserId());
        processDetailDrawing.setProducer(SecurityUtils.getLoginUser().getUser().getNickName());
        return toAjax(bizTechnologicalProcessDetailDrawingService.save(processDetailDrawing));
    }

    /**
     * 覆盖电子图纸
     */
    @PreAuthorize("@ss.hasPermi('business:drawing:edit')")
    @Log(title = "覆盖电子图纸", businessType = BusinessType.UPDATE)
    @PutMapping
    @Transactional
    public AjaxResult edit(@RequestBody BizTechnologicalProcessDetailDrawing processDetailDrawing) throws NoSuchFieldException, IllegalAccessException
    {
        BizTechnologicalProcessDetailDrawing bizTechnologicalProcessDetailDrawing = bizTechnologicalProcessDetailDrawingService.query().eq("id", processDetailDrawing.getId()).one();
        //进行保存操作记录
        BizTechnologicalProcessDetailDrawingRecords drawingRecords = new BizTechnologicalProcessDetailDrawingRecords();
        drawingRecords.setTechnologicalProcessDetailDrawingId(bizTechnologicalProcessDetailDrawing.getId());
        drawingRecords.setTechnologicalProcessDetailId(bizTechnologicalProcessDetailDrawing.getTechnologicalProcessDetailId());
        drawingRecords.setUserId(SecurityUtils.getUserId());
        drawingRecords.setType("覆盖");
        drawingRecords.setDrawingName(bizTechnologicalProcessDetailDrawing.getDrawingName());
        bizTechnologicalProcessDetailDrawingRecordsService.save(drawingRecords);
        //覆盖操作
        BizTechnologicalProcessDetailDrawing item = new BizTechnologicalProcessDetailDrawing();
        item.setUrl(processDetailDrawing.getUrl());
        item.setTechnologicalProcessDetailId(bizTechnologicalProcessDetailDrawing.getTechnologicalProcessDetailId());
        item.setDrawingName(bizTechnologicalProcessDetailDrawing.getDrawingName());
        bizTechnologicalProcessDetailDrawingService.removeById(bizTechnologicalProcessDetailDrawing.getId());


        return toAjax(bizTechnologicalProcessDetailDrawingService.save(item));
    }


    /**
     * 计算重名文件后缀
     * @param s
     * @return
     */
    public static String isLastFifthCharDigit(String s ) {
        int length = s.length();
        if (s == null || length<7){
            return "";
        }
        // 获取倒数第五个到第七字符，查询是不是“_00”格式
        char fifthCharFromEnd = s.charAt(s.length() - 5);
        char sixthCharFromEnd = s.charAt(s.length() - 6);
        char seventhCharFromEnd = s.charAt(s.length() - 7);
        String item = "";
        // 判断字符是否是数字
        if (seventhCharFromEnd == '_'){
            if (Character.isDigit(fifthCharFromEnd)){
                if (Character.isDigit(sixthCharFromEnd)){

                    return  Character.getNumericValue(sixthCharFromEnd) + "" + Character.getNumericValue(fifthCharFromEnd);
                }else{
                    return "0"+ Character.getNumericValue(fifthCharFromEnd);
                }
            }else {
                return "";
            }
        }else {
            return "";
        }


    }

//    public static String isLastFifthCharDigit(String s ) {
//        int length = s.length();
//        if (s == null || length<6){
//            return "";
//        }
//        // 获取倒数第五个字符的位置
//        char fifthCharFromEnd = s.charAt(s.length() - 5);
//        char sixthCharFromEnd = s.charAt(s.length() - 6);
//        String item = "";
//        // 判断字符是否是数字
//        if (Character.isDigit(fifthCharFromEnd)){
//            if (Character.isDigit(sixthCharFromEnd)){
//
//                return  Character.getNumericValue(sixthCharFromEnd) + "" + Character.getNumericValue(fifthCharFromEnd);
//            }else{
//                return Character.getNumericValue(fifthCharFromEnd) +"";
//            }
//        }else {
//            return "";
//        }
//
//    }

    /**
     * 删除电子图纸
     */
    @PreAuthorize("@ss.hasPermi('business:drawing:remove')")
    @Log(title = "删除电子图纸", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    @Transactional
    public AjaxResult remove(@PathVariable Long ids)
    {
        BizTechnologicalProcessDetailDrawing bizTechnologicalProcessDetailDrawing = bizTechnologicalProcessDetailDrawingService.query().eq("id", ids).one();
        BizTechnologicalProcessDetailDrawingRecords drawingRecords = new BizTechnologicalProcessDetailDrawingRecords();
        drawingRecords.setDrawingName(bizTechnologicalProcessDetailDrawing.getDrawingName());
        drawingRecords.setTechnologicalProcessDetailDrawingId(bizTechnologicalProcessDetailDrawing.getId());
        drawingRecords.setTechnologicalProcessDetailId(bizTechnologicalProcessDetailDrawing.getTechnologicalProcessDetailId());
        drawingRecords.setUserId(SecurityUtils.getUserId());
        drawingRecords.setType("删除");
        bizTechnologicalProcessDetailDrawingRecordsService.save(drawingRecords);
        return toAjax(bizTechnologicalProcessDetailDrawingService.removeById(ids));
    }
}
