package cn.ezhizao.project.business.turnover.controller;

import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.project.business.turnover.domain.BizTurnover;
import cn.ezhizao.project.business.turnover.service.IBizTurnoverService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;


/**
 * 周转区Controller
 *
 * @author ezhizao
 * @date 2023-11-14
 */
@RestController
@RequestMapping("/business/turnover")
public class BizTurnoverController extends BaseController {
    @Resource
    private IBizTurnoverService iBizTurnoverService;
    @Resource
    HttpServletRequest request;

    /**
     * 查询周转区列表
     */
//    @PreAuthorize("@ss.hasPermi('business:turnover:list')")
    @Log(title = "周转区列表获取", businessType = BusinessType.SELECT)
    @PostMapping("/list")
    public AjaxResult list(@RequestBody Map<String, Object> data) throws NoSuchFieldException, IllegalAccessException {
        Integer zero = 0;
        String tenantId = request.getHeader("tenantId");
        QueryWrapper<BizTurnover> queryWrapper = new QueryWrapper<>();
        final String param1 = "deptId";
        long deptId = 0L;
        if (data.get(param1) != null) {
            deptId = Long.parseLong(data.get(param1).toString());
        }

        queryWrapper.eq("dept_id", deptId);
        queryWrapper.eq("status",0);
        if (tenantId != null && !zero.toString().equals(tenantId)) {
            queryWrapper.eq("tenant_id", Long.valueOf(tenantId));
        }
        List<BizTurnover> list = iBizTurnoverService.list(queryWrapper);
        return success(list);
    }

    /**
     * 获取周转区详细信息
     */
//    @PreAuthorize("@ss.hasPermi('business:turnover:query')")
    @Log(title = "单一周转区信息获取", businessType = BusinessType.SELECT)
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(iBizTurnoverService.getById(id));
    }

    /**
     * 新增周转区
     */
//    @PreAuthorize("@ss.hasPermi('business:turnover:add, business:turnover:edit')")
    @Log(title = "周转区保存", businessType = BusinessType.INSERT)
    @PostMapping(value = "/save")
    public AjaxResult save(@RequestBody BizTurnover bizTurnover) throws NoSuchFieldException, IllegalAccessException {
        String tenantId = request.getHeader("tenantId");
        QueryWrapper<BizTurnover> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("code", bizTurnover.getCode());
        queryWrapper.eq("dept_id", bizTurnover.getDeptId());
        queryWrapper.eq("status", 0);
        queryWrapper.eq("tenant_id", Long.valueOf(tenantId));
        long currentTurnoverId = 0L;
        if (bizTurnover.getId() != null) {
            currentTurnoverId = bizTurnover.getId();
        }
        queryWrapper.ne("id", currentTurnoverId);
        List<BizTurnover> carriers = iBizTurnoverService.list(queryWrapper);
        if (carriers.size() > 0) {
            return AjaxResult.warn("该周转区编码已存在");
        }
        bizTurnover.setTenantId(Long.valueOf(tenantId));
        return toAjax(iBizTurnoverService.saveOrUpdate(bizTurnover));
    }

    /**
     * 删除周转区
     */
//    @PreAuthorize("@ss.hasPermi('business:turnover:remove')")
    @Log(title = "周转区删除", businessType = BusinessType.DELETE)
    @DeleteMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids) {
        return toAjax(iBizTurnoverService.removeBatchByIds(ids));
    }
}
