package cn.ezhizao.project.system.domain;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import cn.ezhizao.common.xss.Xss;
import cn.ezhizao.framework.aspectj.lang.annotation.Excel;
import cn.ezhizao.framework.aspectj.lang.annotation.Excel.ColumnType;
import cn.ezhizao.framework.aspectj.lang.annotation.Excel.Type;
import cn.ezhizao.framework.web.domain.BaseEntity;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

import javax.validation.constraints.Email;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;
import java.security.Permission;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * 用户对象 sys_user
 *
 * @author ruoyi
 */
public class SysUser extends BaseEntity {
    private static final long serialVersionUID = 1L;

    /**
     * 用户ID
     */
    @Excel(name = "用户序号", cellType = ColumnType.NUMERIC, prompt = "用户编号")
    @TableId
    private Long userId;

    private String orgName;

    /**
     * 用户账号
     */
    @Excel(name = "登录名称")
    private String userName;

    /**
     * 用户昵称
     */
    @Excel(name = "用户名称")
    private String nickName;

    /**
     * 用户邮箱
     */
    @Excel(name = "用户邮箱")
    private String email;

    /**
     * 手机号码
     */
    @Excel(name = "手机号码")
    private String phoneNumber;

    /**
     * 用户性别
     */
    @Excel(name = "用户性别", readConverterExp = "0=男,1=女,2=未知")
    private String sex;

    /**
     * 用户头像
     */
    private String avatar;

    /**
     * 密码
     */
    private String password;


    private String userType;


    /**
     * 帐号状态（0正常 1停用）
     */
    @Excel(name = "帐号状态", readConverterExp = "0=正常,1=停用")
    private String status;

    /**
     * 删除标志（0代表存在 2代表删除）
     */
    private String delFlag;

    /**
     * 最后登录IP
     */
    @Excel(name = "最后登录IP", type = Type.EXPORT)
    private String loginIp;

    /**
     * 最后登录时间
     */
    @Excel(name = "最后登录时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss", type = Type.EXPORT)
    private Date loginDate;

    /**
     * 使用厂别
     */
    private Long usageCompanyId;

    /**
     * 合同厂别
     */
    private Long contractCompanyId;

    private List<Long> ids = new ArrayList<>();

    public List<Long> getIds() {
        return ids;
    }

    public void setIds(List<Long> ids) {
        this.ids = ids;
    }

    /**
     * 角色对象
     */
    private List<SysRole> roles;
    // 权限集合
    private Set<String> permissions;

    /**
     * 角色组
     */
    private Long[] roleIds;

    /**
     * 岗位组
     */
    private Long[] postIds;

    private Integer isLeader;
    private Integer isPartTime;
    private Integer isDispatcher;
    private Integer isFormman;
    private Integer isYardman;
    private Integer isEdit;

    public Long getUsageCompanyId(){return usageCompanyId;}

    public void setUsageCompanyId(Long usageCompanyId){this.usageCompanyId=usageCompanyId;}

    public Long getContractCompanyId(){return contractCompanyId;}

    public void setContractCompanyId(Long contractCompanyId){this.contractCompanyId=contractCompanyId;}


    public Integer getIsLeader() {
        return isLeader;
    }
    public void setIsLeader(Integer leader) {
        isLeader = leader;
    }
    public Integer getIsPartTime() {
        return isPartTime;
    }
    public void setIsPartTime(Integer partTime) {
        isPartTime = partTime;
    }
    public Integer getIsDispatcher() {
        return isDispatcher;
    }
    public void setIsDispatcher(Integer dispatcher) {
        isDispatcher = dispatcher;
    }
    public Integer getIsFormman() {
        return isFormman;
    }

    public void setIsFormman(Integer formman) {
        isFormman = formman;
    }

    public Integer getIsYardman() {
        return isYardman;
    }

    public void setIsYardman(Integer yardman) {
        isYardman = yardman;
    }

    public Integer getIsEdit() {
        return isEdit;
    }

    public void setIsEdit(Integer edit) {
        isEdit = edit;
    }

    public String getOrgName() {
        return orgName;
    }

    public void setOrgName(String orgName) {
        this.orgName = orgName;
    }

    public String getUserType() {
        return userType;
    }

    public void setUserType(String userType) {
        this.userType = userType;
    }

    public SysUser() {

    }

    public SysUser(Long userId) {
        this.userId = userId;
    }

    public Long getUserId() {
        return userId;
    }

    public void setUserId(Long userId) {
        this.userId = userId;
    }

    public boolean isAdmin() {
        return isAdmin(this.userId);
    }

    public static boolean isAdmin(Long userId) {
        return userId != null && 1L == userId;
    }

    @Xss(message = "用户昵称不能包含脚本字符")
    @Size(min = 0, max = 30, message = "用户昵称长度不能超过30个字符")
    public String getNickName() {
        return nickName;
    }

    public void setNickName(String nickName) {
        this.nickName = nickName;
    }

    @Xss(message = "用户账号不能包含脚本字符")
    @NotBlank(message = "用户账号不能为空")
    @Size(min = 0, max = 30, message = "用户账号长度不能超过30个字符")
    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    @Email(message = "邮箱格式不正确")
    @Size(min = 0, max = 50, message = "邮箱长度不能超过50个字符")
    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    @Size(min = 0, max = 11, message = "手机号码长度不能超过11个字符")
    public String getPhoneNumber() {
        return phoneNumber;
    }

    public void setPhoneNumber(String phoneNumber) {
        this.phoneNumber = phoneNumber;
    }

    public String getSex() {
        return sex;
    }

    public void setSex(String sex) {
        this.sex = sex;
    }

    public String getAvatar() {
        return avatar;
    }

    public void setAvatar(String avatar) {
        this.avatar = avatar;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getDelFlag() {
        return delFlag;
    }

    public void setDelFlag(String delFlag) {
        this.delFlag = delFlag;
    }

    public String getLoginIp() {
        return loginIp;
    }

    public void setLoginIp(String loginIp) {
        this.loginIp = loginIp;
    }

    public Date getLoginDate() {
        return loginDate;
    }

    public void setLoginDate(Date loginDate) {
        this.loginDate = loginDate;
    }

    public List<SysRole> getRoles() {
        return roles;
    }

    public void setRoles(List<SysRole> roles) {
        this.roles = roles;
    }

    public Long[] getRoleIds() {
        return roleIds;
    }

    public void setRoleIds(Long[] roleIds) {
        this.roleIds = roleIds;
    }

    public Long[] getPostIds() {
        return postIds;
    }

    public void setPostIds(Long[] postIds) {
        this.postIds = postIds;
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this, ToStringStyle.MULTI_LINE_STYLE)
                       .append("userId", getUserId())
                       .append("userName", getUserName())
                       .append("nickName", getNickName())
                       .append("email", getEmail())
                       .append("phoneNumber", getPhoneNumber())
                       .append("sex", getSex())
                       .append("avatar", getAvatar())
                       .append("password", getPassword())
                       .append("status", getStatus())
                       .append("delFlag", getDelFlag())
                       .append("loginIp", getLoginIp())
                       .append("loginDate", getLoginDate())
                       .append("createBy", getCreateBy())
                       .append("createTime", getCreateTime())
                       .append("updateBy", getUpdateBy())
                       .append("updateTime", getUpdateTime())
                       .append("remark", getRemark())
                       .toString();
    }

    public Set<String> getPermissions() {
        return permissions;
    }

    public void setPermissions(Set<String> permissions) {
        this.permissions = permissions;
    }
    @TableField(exist = false)
    private Boolean isTakeStock;

    public Boolean getTakeStock() {
        return isTakeStock;
    }

    public void setTakeStock(Boolean takeStock) {
        isTakeStock = takeStock;
    }
}
