package cn.ezhizao.project.business.product.controller;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.SecurityUtils;
import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.redis.RedisCache;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.BizInspectionStandardsCheck.domain.BizInspectionStandardsCheck;
import cn.ezhizao.project.business.BizInspectionStandardsCheck.service.IBizInspectionStandardsCheckService;
import cn.ezhizao.project.business.carrierReject.domain.BizCarrierReject;
import cn.ezhizao.project.business.carrierReject.service.IBizCarrierRejectService;
import cn.ezhizao.project.business.inspection.domain.BizProcessInspectionPicture;
import cn.ezhizao.project.business.inspection.service.IBizProcessInspectionPictureService;
import cn.ezhizao.project.business.outsource.domain.BizOutsourcedOrderDetail;
import cn.ezhizao.project.business.outsource.domain.BizOutsourcedOrderDetailProcess;
import cn.ezhizao.project.business.outsource.service.IBizOutsourcedOrderDetailProcessService;
import cn.ezhizao.project.business.outsource.service.IBizOutsourcedOrderDetailService;

import cn.ezhizao.project.business.process.domain.BizProcess;
import cn.ezhizao.project.business.process.service.IBizProcessService;
import cn.ezhizao.project.business.product.domain.*;
import cn.ezhizao.project.business.product.service.*;
import cn.ezhizao.project.business.resourceGroup.domain.BizProductionResourceGroupDetail;
import cn.ezhizao.project.business.resourceGroup.service.IBizProductionResourceGroupDetailService;
import cn.ezhizao.project.business.resourceGroup.service.IBizProductionResourceGroupService;
import cn.ezhizao.project.business.technologicalProcessDetail.domain.BizTechnologicalProcessDetail;
import cn.ezhizao.project.business.technologicalProcessDetail.service.IBizTechnologicalProcessDetailService;
import cn.ezhizao.project.business.technologicalProcessDetailDrawing.domain.BizTechnologicalProcessDetailDrawing;
import cn.ezhizao.project.business.technologicalProcessDetailDrawing.service.IBizTechnologicalProcessDetailDrawingService;
import cn.ezhizao.project.system.domain.SysUser;
import cn.ezhizao.project.system.service.ISysUserService;
import cn.hutool.core.date.DateTime;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

/**
 * 序检主Controller
 *
 * @author ezhizao
 * @date 2024-04-24
 */
@RestController
@RequestMapping("/business/inspecion")
public class BizProcessInspecionController extends BaseController {
    @Resource
    private IBizProcessInspecionService bizProcessInspecionService;

    @Resource
    private ISysUserService sysUserService;

    @Resource
    private IBizDayworkCarrierService bizDayworkCarrierService;
    @Resource
    private IBizLotTechnologicalProcessDetailService bizLotTechnologicalProcessDetailService;

    @Resource
    private IBizTechnologicalProcessDetailService bizTechnologicalProcessDetailService;

    @Resource
    private IBizDayworkItemRejectService bizDayworkItemRejectService;

    @Resource
    private IBizDayworkItemConsultService bizDayworkItemConsultService;

    @Resource
    private IBizCarrierRejectService bizCarrierRejectService;

    @Resource
    private IBizDayworkService bizDayworkService;

    @Resource
    private IBizProductService bizProductService;

    @Resource
    private IBizDayworkItemService bizDayworkItemService;

    @Resource
    private IBizLotService bizLotService;

    @Resource
    private IBizCarrierService bizCarrierService;
    @Resource
    private IBizCarrierCategoryService bizCarrierCategoryService;

    @Resource
    private IBizProductService productService;
    @Resource
    private IBizReturnReceiptDetailService bizReturnReceiptDetailService;
    @Resource
    private IBizProductionPlanDetailService productionPlanDetailService;
    @Resource
    private IBizProcessInspectionDetailService processInspectionDetailService;
    @Resource
    private IBizOutsourcedOrderDetailService bizOutsourcedOrderDetailService;
    @Resource
    private IBizOutsourcedOrderDetailProcessService bizOutsourcedOrderDetailProcessService;
    @Resource
    private IBizDayworkItemConsultPictureService bizDayworkItemConsultPictureService;
    @Resource
    private RedisCache redisCache;
    @Resource
    private IBizProductionResourceGroupDetailService bizProductionResourceGroupDetailService;
    @Resource
    private IBizProductionResourceGroupService bizProductionResourceGroupService;
    @Resource
    private IBizInspectionStandardsCheckService bizInspectionStandardsCheckService;
    @Resource
    private IBizInspectionChamberService bizInspectionChamberService;
    @Resource
    private IBizProcessInspectionPictureService bizProcessInspectionPictureService;
    @Resource
    private IBizProductionPlanDetailService bizProductionPlanDetailService;
    @Resource
    private IBizProcessInspectionRelationService bizProcessInspectionRelationService;
    @Resource
    private IBizProcessInspectionRelationDetailService bizProcessInspectionRelationDetailService;
    @Resource
    private IBizTechnologicalProcessDetailDrawingService bizTechnologicalProcessDetailDrawingService;
    @Resource
    private IBizProcessService bizProcessService;

    final private String processInspectionNoKey = "processInspectionCode";

    /**
     * 查询序检主列表
     */
//    @PreAuthorize("@ss.hasPermi('business:inspecion:list')")
    @GetMapping("/list")
    @Log(title = "查询序检列表", businessType = BusinessType.SELECT)
    public TableDataInfo list(BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {

        if (bizProcessInspecion.getStartTime() != null) {
            Date startTimeDate = bizProcessInspecion.getStartTime();
            // 使用 SimpleDateFormat 格式化 Date 对象为特定格式的字符串
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault());
            String time = dateFormat.format(startTimeDate);
            bizProcessInspecion.setStartTimeString(time + " 00:00:00");
//            bizProcessInspecion.setEndTimeString(time + " 23:59:59");
        }
        if (bizProcessInspecion.getEndTime() != null) {
            Date endTimeDate = bizProcessInspecion.getEndTime();
            // 使用 SimpleDateFormat 格式化 Date 对象为特定格式的字符串
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault());
            String time = dateFormat.format(endTimeDate);
            bizProcessInspecion.setEndTimeString(time + " 23:59:59");
        }
        startPage();
        List<BizProcessInspecion> list = bizProcessInspecionService.getList(bizProcessInspecion);
        return getDataTable(list);
    }

    //交检审核列表
    @PostMapping("/examineList")
    @Log(title = "查询交检审核列表", businessType = BusinessType.SELECT)
    public TableDataInfo examineList(@RequestBody BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {

        if (bizProcessInspecion.getStartTime() != null) {
            Date startTimeDate = bizProcessInspecion.getStartTime();
            // 使用 SimpleDateFormat 格式化 Date 对象为特定格式的字符串
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault());
            String time = dateFormat.format(startTimeDate);

            bizProcessInspecion.setStartTimeString(time + " 00:00:00");
            bizProcessInspecion.setEndTimeString(time + " 23:59:59");
        }
        //交检审核人能看到的是交检操作者是自己所在的资源组的内容,且该批次未周转下道工序
        //当前登陆者所在的资源组
        Long loginUserId = SecurityUtils.getLoginUser().getUserId();
        List<BizProductionResourceGroupDetail> resourceGroupDetailList = bizProductionResourceGroupDetailService.query()
                .eq("dept_id", bizProcessInspecion.getDeptId())
                .eq("common_id", loginUserId)
                .eq("common_type", "person")
                .list();
        List<Long> resourceGroupIds = resourceGroupDetailList.stream().map(BizProductionResourceGroupDetail::getProductionResourceGroupId).collect(Collectors.toList());
        if (resourceGroupIds.isEmpty()) {
            return getDataTable(new ArrayList<>());
        }
        List<BizProcessInspecion> list = bizProcessInspecionService.getExaminList(bizProcessInspecion);
        if (list.isEmpty()) {
            return getDataTable(new ArrayList<>());
        }
        List<Long> dayworkIds = list.stream().map(BizProcessInspecion::getDayworkId).collect(Collectors.toList());
        List<BizDayworkItem> dayworkItemList = bizDayworkItemService.getDayworkItemByDayworkIds(dayworkIds);

        List<Long> processInspectionUserIds = list.stream().map(BizProcessInspecion::getUserId).collect(Collectors.toList());
        List<BizProductionResourceGroupDetail> resourceGroupList = bizProductionResourceGroupDetailService.query().eq("dept_id", bizProcessInspecion.getDeptId())
                .in("common_id", processInspectionUserIds)
                .eq("common_type", "person").list();
        List<BizProcessInspecion> newList = new ArrayList<>();
        for (BizProcessInspecion item : list) {
            List<BizProductionResourceGroupDetail> detailList = resourceGroupList.stream()
                    .filter(resourceGroup -> resourceGroup.getCommonId().equals(item.getUserId()))
                    .collect(Collectors.toList());
            for (BizProductionResourceGroupDetail info : detailList) {
                if (resourceGroupIds.contains(info.getProductionResourceGroupId())) {
                    //判断如果已经转到下序，则列表不显示
                    BizDayworkItem lastDayworkItem = dayworkItemList.stream().filter(l -> item.getDayworkId().equals(l.getDayworkId())).sorted(Comparator.comparing(BizDayworkItem::getCreateTime).reversed()).collect(Collectors.toList()).get(0);
                    String lastProcessStepNumber = lastDayworkItem.getProcessStepNumber();
                    BizDayworkItem currentDayworkItem = dayworkItemList.stream().filter(l -> l.getId().equals(item.getDayworkItemId())).collect(Collectors.toList()).get(0);
                    String currentProcessStepNumber = currentDayworkItem.getProcessStepNumber();
                    if (lastProcessStepNumber.equals(currentProcessStepNumber)) {
                        newList.add(item);
                        break; // 退出内部的for循环
                    }

                }
            }
        }
        return getDataTable(newList);
    }

    //修改交检审核状态
    @PutMapping("/updateAdoptStatus")
    @Log(title = "修改交检审核状态", businessType = BusinessType.UPDATE)
    public AjaxResult updateAdoptStatus(@RequestBody BizProcessInspecion bizProcessInspecion) {
        Long userId = SecurityUtils.getLoginUser().getUserId();
        bizProcessInspecion.setAdoptId(userId);
        bizProcessInspecion.setAuditStatus(bizProcessInspecion.getAuditStatus());
        bizProcessInspecionService.updateById(bizProcessInspecion);
        return success();
    }

    /**
     * 查询序检主列表(外协序检)
     */
//    @PreAuthorize("@ss.hasPermi('business:inspecion:list')")
    @GetMapping("/outsourcedlist")
    @Log(title = "查询序检主列表", businessType = BusinessType.SELECT)
    public TableDataInfo outsourcedlist(BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {
        if (bizProcessInspecion.getStartTime() != null) {
            Date startTimeDate = bizProcessInspecion.getStartTime();
            // 使用 SimpleDateFormat 格式化 Date 对象为特定格式的字符串
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault());
            String time = dateFormat.format(startTimeDate);
            bizProcessInspecion.setStartTimeString(time + " 00:00:00");
//            bizProcessInspecion.setEndTimeString(time + " 23:59:59");
        }
        if (bizProcessInspecion.getEndTime() != null) {
            Date endTimeDate = bizProcessInspecion.getEndTime();
            // 使用 SimpleDateFormat 格式化 Date 对象为特定格式的字符串
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault());
            String time = dateFormat.format(endTimeDate);
            bizProcessInspecion.setEndTimeString(time + " 23:59:59");
        }
        startPage();
        List<BizProcessInspecion> list = bizProcessInspecionService.getOutsourcedlist(bizProcessInspecion);
        return getDataTable(list);
    }

    /**
     * 查询产品信息
     */
    @PostMapping("/getProductConsult")
    @Log(title = "查询产品信息", businessType = BusinessType.SELECT)
    public AjaxResult getProductConsult(@RequestBody BizLot lot) throws NoSuchFieldException, IllegalAccessException {
        BizProduct product = bizProductService.getById(lot.getProductId());
        if (product == null) {
            throw (new RuntimeException("该产品不存在"));
        }
        BizProcess process = bizProcessService.query().eq("id", lot.getProcessId()).one();
        List<BizTechnologicalProcessDetailDrawing> technologicalProcessDetailDrawingList = bizTechnologicalProcessDetailDrawingService.query().eq("product_id", product.getId()).eq("process_code", process.getProcessCode()).orderByDesc("create_time").list();

        if(!technologicalProcessDetailDrawingList.isEmpty()){
            SysUser user = sysUserService.selectUserById(technologicalProcessDetailDrawingList.get(0).getCreatorId());
            product.setTechnicianName(user == null ? "" : user.getNickName());
            product.setTechnicianId(user == null ? 0L : user.getUserId());
        }else{
            product.setTechnicianName("");
            product.setTechnicianId(0L);
        }
//
//        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
//        product.setTechnicianName(user == null ? "" : user.getNickName());

        return success(product);
    }

    /**
     * 查询序捡详情
     *
     * @param processInspecion
     * @return
     * @throws NoSuchFieldException
     * @throws IllegalAccessException
     */
    //@PostMapping("/selectInspecion")
//    public AjaxResult selectInspecion(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
//        List<BizDayworkItemConsult> itemConsults = bizDayworkItemConsultService.query().eq("process_inspection_id", processInspecion.getId()).eq("is_process_inspection", 1).list();
//        List<BizDayworkItemReject> itemRejects = bizDayworkItemRejectService.query().eq("process_inspection_id", processInspecion.getId()).list();
//        processInspecion.setDayworkItemConsults(itemConsults);
//        processInspecion.setDayworkItemRejects(itemRejects);
//
//        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getDayworkItemId());
//
//        BizLot lot = new BizLot();
//        lot.setId(processInspecion.getLotId());
//        lot = bizLotService.getAllList(lot).get(0);
//
//        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
//            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
//            lot.setProcessAlias(process.getProcessAlias());
//        } else {
//            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
//            lot.setProcessAlias(process.getProcessAlias());
//        }
//
//        lot.setAllCarrierName(dayworkItem.getCarrierName());
//        lot.setDayworkItemId(dayworkItem.getId());
//        lot.setCarrierCode(dayworkItem.getCarrierCode());
//        lot.setCarrierId(dayworkItem.getCarrierId());
//        lot.setPudName(dayworkItem.getProdNum() == 0 ? lot.getProductionQuantity() : dayworkItem.getProdNum());
//        processInspecion.setLot(lot);
//        return success(processInspecion);
//    }
    @Log(title = "查询序检详情", businessType = BusinessType.SELECT)
    @PostMapping("/selectInspecion")
    public AjaxResult selectInspecion(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        List<Long> inspectionIds = new ArrayList<>();
        inspectionIds.add(processInspecion.getId());
        List<BizDayworkItemConsult> itemConsults = bizDayworkItemConsultService.getListByInspectionIds(inspectionIds);
        List<BizProcessInspectionDetail> itemRejects = processInspectionDetailService.query().eq("process_inspection_id", processInspecion.getId()).list();
        List<BizProcessInspectionPicture> itemInspectionPictureList = bizProcessInspectionPictureService.query().eq("process_inspection_id", processInspecion.getId()).list();
        processInspecion.setDayworkItemConsults(itemConsults);
        processInspecion.setProcessInspectionDetails(itemRejects);
        processInspecion.setProcessInspectionPictureList(itemInspectionPictureList);

        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getDayworkItemId());

        BizLot lot = new BizLot();
        lot.setId(processInspecion.getLotId());
        lot = bizLotService.getAllList(lot).get(0);

        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        }

        lot.setAllCarrierName(dayworkItem.getCarrierName());
        lot.setNickName(processInspecion.getNickName());
        lot.setUserId(processInspecion.getUserId());
        lot.setEquipmentDetailCode(processInspecion.getEquipmentDetailCode());
        lot.setEquipmentDetailId(processInspecion.getEquipmentDetailId());
        lot.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
        lot.setProcessCode(dayworkItem.getProcessCode());
        lot.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        lot.setProcessId(dayworkItem.getProcessId());
        lot.setDayworkItemId(dayworkItem.getId());
        lot.setCarrierCode(processInspecion.getCarrierCode());
        lot.setCarrierId(processInspecion.getCarrierId());
        lot.setPudName(dayworkItem.getProdNum() == 0 ? lot.getProductionQuantity() : dayworkItem.getProdNum());
        processInspecion.setLot(lot);
        return success(processInspecion);
    }

    //判断载具
    @PostMapping("/checkCarrier")
    @Log(title = "判断检测载具是否可用", businessType = BusinessType.SELECT)
    public AjaxResult checkCarrier(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        String carrierCode = dayworkCarrier.getCarrierCode();
        BizCarrier carrier = bizCarrierService.query().eq("code", carrierCode).eq("is_inspection", 1).one();
        if (carrier == null) {
            return error("未找到该检测箱号");
        }
        //查询箱号是否废弃
        BizCarrierReject carrierReject = bizCarrierRejectService.query().eq("carrier_id", carrier.getId()).eq("deleted", 0).eq("is_abandoned", 1).last("limit 1").one();
        if (carrierReject != null) {
            return error("该检测载具已废弃");
        }
        //查找该箱号关联批次
        List<BizDayworkCarrier> dayworkCarrierList = bizDayworkCarrierService.query().eq("carrier_id", carrier.getId()).eq("is_changed", 0).list();
        if (dayworkCarrierList.isEmpty()) {
            return error("该检测载具未绑定批次");
        }
        return success();
    }

    //根据检测载具查询检测信息
    @PostMapping("getInspectionInfoByInspectionCarrierCode")
    @Log(title = "根据检测载具查询检测信息", businessType = BusinessType.SELECT)
    public AjaxResult getInspectionInfoByInspectionCarrierCode(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        String carrierCode = dayworkCarrier.getCarrierCode();
        BizCarrier carrier = bizCarrierService.query().eq("code", carrierCode).eq("is_inspection", 1).one();
        //查询箱号是否废弃
        BizCarrierReject carrierReject = bizCarrierRejectService.query().eq("carrier_id", carrier.getId()).eq("deleted", 0).eq("is_abandoned", 1).last("limit 1").one();
        //查找该箱号关联批次
        List<BizDayworkCarrier> dayworkCarrierList = bizDayworkCarrierService.query().eq("carrier_id", carrier.getId()).eq("is_changed", 0).list();
        List<Long> processInspectionIds = dayworkCarrierList.stream().map(BizDayworkCarrier::getProcessInspectionId).collect(Collectors.toList());
        if (processInspectionIds.isEmpty()) {
            return error("该检测载具未绑定批次");
        }
        List<BizProcessInspecion> processInspectionList = bizProcessInspecionService.getInfoByIds(processInspectionIds);

        //咨询
        List<BizDayworkItemConsult> itemConsults = bizDayworkItemConsultService.getListByInspectionIds(processInspectionIds);
        //明细
        List<BizProcessInspectionDetail> itemDetails = processInspectionDetailService.query().in("process_inspection_id", processInspectionIds).list();
        //废品
        List<BizDayworkItemReject> itemRejects = bizDayworkItemRejectService.query().in("process_inspection_id", processInspectionIds).list();
        //拍照上传
        List<BizProcessInspectionPicture> itemInspectionPictureList = bizProcessInspectionPictureService.query().in("process_inspection_id", processInspectionIds).list();
        List<BizProcessInspecion> list = new ArrayList<>();
        processInspectionIds.forEach(item -> {
            BizProcessInspecion processInspection = new BizProcessInspecion();
            processInspection = processInspectionList.stream().filter(x -> x.getId().equals(item)).collect(Collectors.toList()).get(0);
            List<BizDayworkItemConsult> consults = itemConsults.stream().filter(x -> x.getProcessInspectionId().equals(item)).collect(Collectors.toList());
            processInspection.setDayworkItemConsults(consults);
            List<BizProcessInspectionDetail> details = itemDetails.stream().filter(x -> x.getProcessInspectionId().equals(item)).collect(Collectors.toList());
            processInspection.setProcessInspectionDetails(details);
            List<BizDayworkItemReject> rejects = itemRejects.stream().filter(x -> x.getProcessInspectionId().equals(item)).collect(Collectors.toList());
            processInspection.setDayworkItemRejects(rejects);
            List<BizProcessInspectionPicture> pictures = itemInspectionPictureList.stream().filter(x -> x.getProcessInspectionId().equals(item)).collect(Collectors.toList());
            processInspection.setProcessInspectionPictureList(pictures);
            list.add(processInspection);
        });
        return success(list);
    }

    //    @PostMapping("/selectOutsourcedInspecion")
    public AjaxResult selectOutsourcedInspecion(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        List<BizDayworkItemConsult> itemConsults = bizDayworkItemConsultService.query().eq("process_inspection_id", processInspecion.getId()).eq("is_outsourced_inspection", 1).list();
        List<BizDayworkItemReject> itemRejects = bizDayworkItemRejectService.query().eq("process_inspection_id", processInspecion.getId()).list();
        List<BizProcessInspectionDetail> reject = processInspectionDetailService.query().eq("process_inspection_id", processInspecion.getId()).list();
        List<BizProcessInspectionPicture> itemInspectionPictureList = bizProcessInspectionPictureService.query().eq("process_inspection_id", processInspecion.getId()).list();
        processInspecion.setProcessInspectionPictureList(itemInspectionPictureList);
        processInspecion.setDayworkItemConsults(itemConsults);
        processInspecion.setDayworkItemRejects(itemRejects);
        processInspecion.setRejects(reject);

        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getDayworkItemId());

        BizLot lot = new BizLot();
        lot.setId(processInspecion.getLotId());
        lot = bizLotService.getAllList(lot).get(0);

        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
            lot.setProcessCode(process.getProcessCode());
            lot.setProcessId(process.getProcessId());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
            lot.setProcessCode(process.getProcessCode());
            lot.setProcessId(process.getProcessId());
        }
        lot.setNickName(processInspecion.getNickName());
        lot.setUserId(processInspecion.getUserId());
        lot.setEquipmentDetailCode(processInspecion.getEquipmentDetailCode());
        lot.setEquipmentDetailId(processInspecion.getEquipmentDetailId());
        lot.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
        lot.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        lot.setDeptName(dayworkItem.getDeptName());
        lot.setAllCarrierName(dayworkItem.getCarrierName());
        lot.setDayworkItemId(dayworkItem.getId());
        lot.setCarrierCode(dayworkItem.getCarrierCode());
        lot.setCarrierId(dayworkItem.getCarrierId());
        lot.setPudName(dayworkItem.getProdNum() == 0 ? lot.getProductionQuantity() : dayworkItem.getProdNum());
        processInspecion.setLot(lot);
        return success(processInspecion);
    }

    @PostMapping("/selectOutsourcedInspecion")
    @Log(title = "获取外协检查信息", businessType = BusinessType.SELECT)
    public AjaxResult selectOutsourcedInspecionV1(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        List<Long> inspectionIds = new ArrayList<>();
        inspectionIds.add(processInspecion.getId());
        List<BizDayworkItemConsult> itemConsults = bizDayworkItemConsultService.getListByInspectionIds(inspectionIds);
//        List<BizDayworkItemConsult> itemConsults = bizDayworkItemConsultService.query().eq("process_inspection_id", processInspecion.getId()).list();
        List<BizDayworkItemReject> itemRejects = bizDayworkItemRejectService.query().eq("process_inspection_id", processInspecion.getId()).list();
        List<BizProcessInspectionDetail> reject = processInspectionDetailService.query().eq("process_inspection_id", processInspecion.getId()).list();
        processInspecion.setDayworkItemRejects(itemRejects);
        processInspecion.setRejects(reject);
        List<BizProcessInspectionPicture> itemInspectionPictureList = bizProcessInspectionPictureService.query().eq("process_inspection_id", processInspecion.getId()).list();
        processInspecion.setProcessInspectionPictureList(itemInspectionPictureList);
        List<BizProcessInspecion> oldProcessInspection = bizProcessInspecionService.query().in("outsource_order_detail_id", processInspecion.getOutsourceOrderDetailId()).ne("id", processInspecion.getId()).list();
        List<BizProcessInspectionDetail> oldReject = processInspectionDetailService.query().in("process_inspection_id", oldProcessInspection.isEmpty() ? Collections.singletonList(0L) : oldProcessInspection.stream().map(BizProcessInspecion::getId).collect(Collectors.toList())).list();
        processInspecion.setOldRejects(oldReject);

        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getDayworkItemId());

        BizLot lot = new BizLot();
        lot.setId(processInspecion.getLotId());
        lot = bizLotService.getAllList(lot).get(0);

        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        }

        lot.setAllCarrierName(dayworkItem.getCarrierName());
        lot.setDayworkItemId(dayworkItem.getId());
        lot.setCarrierCode(dayworkItem.getCarrierCode());
        lot.setCarrierId(dayworkItem.getCarrierId());
        lot.setProcessId(dayworkItem.getProcessId());
        lot.setProcessStepNumber(dayworkItem.getProcessStepNumber());
        lot.setPudName(dayworkItem.getProdNum() == 0 ? lot.getProductionQuantity() : dayworkItem.getProdNum());
        processInspecion.setLot(lot);
        List<BizProcessInspectionRelationDetail> relatedList = bizProcessInspectionRelationDetailService.query().eq("process_inspection_relation_id", processInspecion.getRelatedId()).list();
        List<BizProcessInspecion> related = bizProcessInspecionService.query().in("id", relatedList.isEmpty() ? Collections.singletonList(0L) : relatedList.stream().map(BizProcessInspectionRelationDetail::getProcessInspectionId).collect(Collectors.toList())).ne("id", processInspecion.getId()).list();
        List<BizDayworkCarrier> relatedCarriers = bizDayworkCarrierService.query().in("daywork_id", related.isEmpty()? Collections.singletonList(0L) : related.stream().map(BizProcessInspecion::getDayworkId).collect(Collectors.toList())).eq("process_inspection_id", 0).eq("is_changed", 0).list();
        related.forEach(l -> {
            BizProcessInspectionRelationDetail item = relatedList.stream().filter(r -> r.getProcessInspectionId().equals(l.getId())).findFirst().orElse(null);
            l.setIsMaster(item == null ? 0 : item.getIsMaster());
            l.setAllCarriers(relatedCarriers.stream().filter(t -> t.getDayworkId().equals(l.getDayworkId())).map(BizDayworkCarrier::getCarrierCode).collect(Collectors.joining(",")));
        });
        processInspecion.setRelateList(related);
        itemConsults.addAll(bizDayworkItemConsultService.query().in("process_inspection_id", related.isEmpty() ? Collections.singletonList(0L) : related.stream().map(BizProcessInspecion::getId).collect(Collectors.toList())).eq("is_outsourced_inspection",1).list());
        processInspecion.setDayworkItemConsults(itemConsults);
        return success(processInspecion);
    }

    /**
     * 新增外协序检信息
     */
//    @PostMapping("/saveOutsourcedInspecion")
    @Transactional
    public AjaxResult saveOutsourcedInspecion(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());

        SysUser user = sysUserService.selectUserById(processInspecion.getUser().getUserId());
        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        System.out.println("DayworkID:" + dayworkItem.getDayworkId());
        System.out.println("getTechnologicalProcessDetailId:" + dayworkItem.getTechnologicalProcessDetailId());
        BizReturnReceiptDetail bizReturnReceiptDetail = bizReturnReceiptDetailService.query().eq("daywork_id", dayworkItem.getDayworkId()).eq("technological_process_detail_id", dayworkItem.getTechnologicalProcessDetailId()).eq("deleted", 0).one();
        System.out.println("回收:" + bizReturnReceiptDetail);
        //根据dayworkItem的工序ID和dayworkId查询回收Id
        processInspecion.setUserId(user == null ? 0L : user.getUserId());
        processInspecion.setNickName(user == null ? "" : user.getNickName());
        processInspecion.setReturnReceiptDetailId(bizReturnReceiptDetail.getId());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setIsOutsourcedInspection(1);
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        processInspecion.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
        processInspecion.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }

        if (processInspecion.getInspectionStatus() != 0) {
            processInspecion.setEndTime(new Date());
            processInspecion.setFirstUpdaterId(processInspecion.getFirstUpdaterId() == null || processInspecion.getFirstUpdaterId().equals(0L) ? user != null ? user.getUserId() : 0L : processInspecion.getFirstUpdaterId());
        }

        //保存
        bizProcessInspecionService.saveOrUpdate(processInspecion);

        //保存废品信息
        List<BizDayworkItemReject> rejects = processInspecion.getDayworkItemRejects();
        //保存前先删除原先废品信息
        BizDayworkItemReject rej = new BizDayworkItemReject();
        rej.setProcessInspectionId(processInspecion.getId());
        bizDayworkItemRejectService.removeReject(rej);
        if (rejects != null && rejects.size() > 0) {
            for (BizDayworkItemReject reject : rejects) {
                reject.setId(null);
                reject.setUserId(user == null ? 0L : user.getUserId());
                reject.setProcessInspectionId(processInspecion.getId());
                reject.setDayworkId(0L);
                reject.setProductionPlanId(daywork.getProductionPlanId());
                reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                reject.setLotId(processInspecion.getLot().getId());
                reject.setDayworkItemId(dayworkItem.getId());
            }
        }
        bizDayworkItemRejectService.saveBatch(rejects);
        //保存咨询信息

        List<BizDayworkItemConsult> consults = processInspecion.getDayworkItemConsults().stream().filter(item -> item.getId() == null).collect(Collectors.toList());
        List<BizDayworkItemConsult> needAdd = new ArrayList<>();
        if (consults != null && consults.size() > 0) {
            for (BizDayworkItemConsult consult : consults) {
                BizDayworkItemConsult consultInfo = new BizDayworkItemConsult();
                consultInfo.setStatus(consult.getStatus());
                consultInfo.setContent(consult.getContent());
                consultInfo.setConsultDepartment(consult.getConsultDepartment());
                consultInfo.setUserId(user == null ? 0L : user.getUserId());
                consultInfo.setNickName(user == null ? "" : user.getNickName());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setProcessInspectionId(processInspecion.getId());
                consultInfo.setProductionPlanId(daywork.getProductionPlanId());
                consultInfo.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                consultInfo.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                consultInfo.setLotId(processInspecion.getLot().getId());
                consultInfo.setLotCode(processInspecion.getLot().getLotCode());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setDayworkItemId(dayworkItem.getId());
                consultInfo.setProductId(processInspecion.getLot().getProductId());
                consultInfo.setProductDescription(processInspecion.getLot().getProductDescription());
                consultInfo.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                consultInfo.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                consultInfo.setProcessId(dayworkItem.getProcessId());
                consultInfo.setProcessAlias(processInspecion.getLot().getProcessAlias());
                consultInfo.setIsOutsourcedInspection(1);
                consultInfo.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
                needAdd.add(consultInfo);
            }
            bizDayworkItemConsultService.saveBatch(needAdd);
            List<BizDayworkItemConsultPicture> pictures = new ArrayList<>();
            needAdd.forEach(ne -> {
                consults.forEach(item -> {
                    item.getPictures().forEach(info -> {
                        BizDayworkItemConsultPicture consultPicture = new BizDayworkItemConsultPicture();
                        consultPicture.setDayworkItemConsultId(ne.getId());
                        consultPicture.setPictureName(info.getPictureName());
                        consultPicture.setUrl(info.getUrl());
                        pictures.add(consultPicture);
                    });
                });
            });
            bizDayworkItemConsultPictureService.saveBatch(pictures);
        }


        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        if(dayworkItem.getDeleted() == 1) {
            return success();
        }
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem));
    }

    @PostMapping("/getProcessInspectionByDayworkId")
    @Log(title = "获取质检信息", businessType = BusinessType.SELECT)
    public AjaxResult getProcessInspectionByDayworkId(@RequestBody BizProcessInspecion processInspecion) {

        List<BizProcessInspecion> processInspecionList = bizProcessInspecionService.getInspection(processInspecion);
        //List<BizProcessInspectionDetail> list = bizProcessInspecionService.getInspectionByDayworkId(processInspecion);
        List<BizDayworkItemConsult> bizDayworkItemConsultList = new ArrayList<>();
        List<BizProcessInspectionPicture> bizProcessInspectionPictureList = new ArrayList<>();
        List<BizProcessInspectionDetail> newProcessDetailList = new ArrayList<>();
        if (!processInspecionList.isEmpty()) {
            List<Long> inspectionIds = processInspecionList.stream().map(BizProcessInspecion::getId).collect(Collectors.toList());
            List<BizProcessInspectionDetail> processInspectionDetailList = bizProcessInspecionService.getDetailByProcessInspectionIds(inspectionIds);
            for (BizProcessInspecion item : processInspecionList) {
                List<BizProcessInspectionDetail> detailList = processInspectionDetailList.stream().filter(detail -> detail.getProcessInspectionId().equals(item.getId())).collect(Collectors.toList());
                if (detailList.size() > 0) {
                    detailList.forEach(detail -> {
                        detail.setRemark(item.getRemark());
                        detail.setStatus(item.getStatus());
                        detail.setProcessAlias(item.getProcessAlias());
                        detail.setInspectionName(item.getNickName());
                        detail.setInspectionType(item.getType());
                        newProcessDetailList.add(detail);
                    });
                } else {
                    BizProcessInspectionDetail newProcess = new BizProcessInspectionDetail();
                    newProcess.setProcessInspectionId(item.getId());
                    newProcess.setStatus(item.getStatus());
                    newProcess.setRemark(item.getRemark());
                    newProcess.setProcessAlias(item.getProcessAlias());
                    newProcess.setInspectionName(item.getNickName());
                    newProcess.setInspectionType(item.getType());
                    newProcessDetailList.add(newProcess);
                }
            }
            //咨询信息
            bizDayworkItemConsultList = bizDayworkItemConsultService.getListByInspectionIds(inspectionIds);
            //上传照片信息
            bizProcessInspectionPictureList = bizProcessInspectionPictureService.query().in("process_inspection_id", inspectionIds).list();
        }
        // 并将每个分组的元素收集到一个 List 中
        Map<Long, List<BizProcessInspectionDetail>> groupedByProcessInspectionId =
                newProcessDetailList.stream()
                        .collect(Collectors.groupingBy(
                                BizProcessInspectionDetail::getProcessInspectionId, // 分组依据的函数
                                Collectors.toList() // 指定分组内的元素收集为 List
                        ));
        List<BizProcessInspecion> result = new ArrayList<>();
        for (Map.Entry<Long, List<BizProcessInspectionDetail>> entry : groupedByProcessInspectionId.entrySet()) {
            List<BizProcessInspectionDetail> details = entry.getValue();
            BizProcessInspecion bizProcessInspecion = new BizProcessInspecion();
            bizProcessInspecion.setProcessAlias(details.get(0).getProcessAlias());
            bizProcessInspecion.setNickName(details.get(0).getInspectionName());
            bizProcessInspecion.setRemark(details.get(0).getRemark());
            bizProcessInspecion.setStatus(details.get(0).getStatus());
            bizProcessInspecion.setType(details.get(0).getInspectionType());
            bizProcessInspecion.setId(details.get(0).getProcessInspectionId());
            BizProcessInspecion processInspection = processInspecionList.stream().filter(item -> item.getId().equals(bizProcessInspecion.getId())).findFirst().orElse(new BizProcessInspecion());
            bizProcessInspecion.setReviewerName(processInspection.getReviewerName());
            List<BizProcessInspectionDetail> rejectList = new ArrayList<>();
            for (BizProcessInspectionDetail reject : details) {
                BizProcessInspectionDetail rejectInfo = new BizProcessInspectionDetail();
                rejectInfo.setCheckStandard(reject.getCheckStandard());
                rejectInfo.setRejectNum(reject.getRejectNum());
                rejectInfo.setCheckResult(reject.getCheckResult());
                if (reject.getCheckResult() != null || reject.getCheckStandard() != null || reject.getRejectNum() != null) {
                    rejectList.add(rejectInfo);
                }

            }
            bizProcessInspecion.setProcessInspectionDetails(rejectList);
            if (!bizDayworkItemConsultList.isEmpty()) {
                bizProcessInspecion.setDayworkItemConsults(bizDayworkItemConsultList.stream().filter(item -> item.getProcessInspectionId().equals(bizProcessInspecion.getId())).collect(Collectors.toList()));
            }
            if (!bizProcessInspectionPictureList.isEmpty()) {
                bizProcessInspecion.setProcessInspectionPictureList(bizProcessInspectionPictureList.stream().filter(item -> item.getProcessInspectionId().equals(bizProcessInspecion.getId())).collect(Collectors.toList()));
            }

            result.add(bizProcessInspecion);
        }
        return success(result);
    }

    /**
     * 新增序检信息
     */
    @PostMapping("/saveProcessInspecion")
    @Transactional
    @Log(title = "新增序检信息", businessType = BusinessType.INSERT)
    public AjaxResult saveProcessInspecion(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());

        SysUser user = sysUserService.selectUserById(processInspecion.getUser().getUserId());
        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        processInspecion.setUserId(user == null ? 0L : user.getUserId());
        processInspecion.setNickName(user == null ? "" : user.getNickName());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProductDescription(processInspecion.getLot().getProductDescription());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }

        if (processInspecion.getStatus() != 0) {
            processInspecion.setEndTime(new Date());

        }
        //保存
        bizProcessInspecionService.saveOrUpdate(processInspecion);
        //保存废品信息
        List<BizDayworkItemReject> rejects = processInspecion.getDayworkItemRejects();
        //保存前先删除原先废品信息
        BizDayworkItemReject rej = new BizDayworkItemReject();
        rej.setProcessInspectionId(processInspecion.getId());
        bizDayworkItemRejectService.removeReject(rej);
        if (rejects != null && rejects.size() > 0) {
            for (BizDayworkItemReject reject : rejects) {
                reject.setId(null);
                reject.setUserId(user == null ? 0L : user.getUserId());
                reject.setProcessInspectionId(processInspecion.getId());
                reject.setDayworkId(0L);
                reject.setProductionPlanId(daywork.getProductionPlanId());
                reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                reject.setLotId(processInspecion.getLot().getId());
                reject.setDayworkItemId(dayworkItem.getId());
            }
        }
        bizDayworkItemRejectService.saveBatch(rejects);
        //保存咨询信息

        List<BizDayworkItemConsult> consults = processInspecion.getDayworkItemConsults().stream().filter(item -> item.getId() == null).collect(Collectors.toList());
        List<BizDayworkItemConsult> needAdd = new ArrayList<>();
        if (consults != null && consults.size() > 0) {
            for (BizDayworkItemConsult consult : consults) {
                BizDayworkItemConsult consultInfo = new BizDayworkItemConsult();
                consultInfo.setConsultDepartment(consult.getConsultDepartment());
                consultInfo.setUserId(user == null ? 0L : user.getUserId());
                consultInfo.setNickName(user == null ? "" : user.getNickName());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setContent(consult.getContent());
                consultInfo.setProcessInspectionId(processInspecion.getId());
                consultInfo.setProductionPlanId(daywork.getProductionPlanId());
                consultInfo.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                consultInfo.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                consultInfo.setLotId(processInspecion.getLot().getId());
                consultInfo.setLotCode(processInspecion.getLot().getLotCode());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setDayworkItemId(dayworkItem.getId());
                consultInfo.setProductId(processInspecion.getLot().getProductId());
                consultInfo.setProductDescription(processInspecion.getLot().getProductDescription());
                consultInfo.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                consultInfo.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                consultInfo.setProcessId(dayworkItem.getProcessId());
                consultInfo.setProcessAlias(processInspecion.getLot().getProcessAlias());
                consultInfo.setIsProcessInspection(1);
                consultInfo.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
                consultInfo.setStatus(consult.getStatus());
                consultInfo.setPictures(consult.getPictures());
                needAdd.add(consultInfo);
            }
            bizDayworkItemConsultService.saveBatch(needAdd);
            List<BizDayworkItemConsultPicture> pictures = new ArrayList<>();
            needAdd.forEach(ne -> {
                ne.getPictures().forEach(info -> {
                    BizDayworkItemConsultPicture consultPicture = new BizDayworkItemConsultPicture();
                    consultPicture.setDayworkItemConsultId(ne.getId());
                    consultPicture.setPictureName(info.getPictureName());
                    consultPicture.setUrl(info.getUrl());
                    pictures.add(consultPicture);
                });
            });
            bizDayworkItemConsultPictureService.saveBatch(pictures);
        }


        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        if(dayworkItem.getDeleted() == 1) {
            return success();
        }
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem));
    }

    private boolean checkOutsourceDayworkItem(Long outsourceOrderDetailId) {
        // 检测外协序检后是否已经周转到下一工序。
        BizDayworkItem bizDayworkItem = bizDayworkItemService.query().eq("outsource_detail_id", outsourceOrderDetailId).orderByDesc("create_time").last("limit 1").one();
        BizDaywork bizDaywork = bizDayworkService.getById(bizDayworkItem.getDayworkId());
        List<BizDayworkItem> items = bizDayworkItemService.query().eq("daywork_id", bizDayworkItem.getDayworkId()).list();
        // 下序已经报工
        // 下序未报工
        return items.stream().noneMatch(v -> v.getCreateTime().compareTo(bizDayworkItem.getCreateTime()) > 0 && v.getStatus().compareTo("0") >= 0 && v.getStatus().compareTo("3") <= 0);
    }

    private boolean checkDayworkItem(Long dayworkItemId) {
        BizDayworkItem bizDayworkItem = bizDayworkItemService.query().eq("id", dayworkItemId).one();
        BizDaywork bizDaywork = bizDayworkService.getById(bizDayworkItem.getDayworkId());
        List<BizDayworkItem> items = bizDayworkItemService.query().eq("daywork_id", bizDayworkItem.getDayworkId()).list();
        boolean allStepsLessThanOrEqual = items.stream()
                .noneMatch(v -> {
                    try {
                        int itemStepNumber = Integer.parseInt(v.getProcessStepNumber());
                        int bizStepNumber = Integer.parseInt(bizDayworkItem.getProcessStepNumber());
                        return itemStepNumber > bizStepNumber;
                    } catch (NumberFormatException e) {
                        // 如果转换失败，可以记录日志或处理异常
                        e.printStackTrace();
                        return false; // 或者根据需要返回其他值
                    }
                });
        return allStepsLessThanOrEqual;
    }

    @Transactional
    //@PostMapping("/saveOutsourcedInspecion")
    public AjaxResult saveOutsourcedInspectionV3(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
//        // 判断是否已有未确认外协检查
//        if (bizProcessInspecionService.query().eq("outsource_order_detail_id", processInspecion.getOutsourceOrderDetailId()).eq("status", 0).count() > 0) {
//            return error("已有未完成外协检查单。");
//        }
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());
        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        BizReturnReceiptDetail bizReturnReceiptDetail = bizReturnReceiptDetailService.query().eq("daywork_id", dayworkItem.getDayworkId()).eq("technological_process_detail_id", dayworkItem.getTechnologicalProcessDetailId()).eq("deleted", 0).one();

        SysUser user = sysUserService.selectUserById(processInspecion.getUser().getUserId());

        // 判断是否已经进行过质检
        BizProcessInspecion previousItem = bizProcessInspecionService.query().eq("outsource_order_detail_id", bizReturnReceiptDetail.getOutsourceDetailId()).isNull("recheck_date").ne("id", processInspecion.getId() == null ? 0L : processInspecion.getId()).one();
        BizOutsourcedOrderDetail outsourceOrderDetail = bizOutsourcedOrderDetailService.query().eq("id", bizReturnReceiptDetail.getOutsourceDetailId()).one();

//        if (previousItem != null) {
//            if (processInspecion.getId() == null) {
//                previousItem.setRecheckDate(DateTime.now().toJdkDate());
//            }
//            previousItem.setIsInspectionCarrierChanged(0);
//            previousItem.setInspectionCarrierChangedTime(null);
//            previousItem.setStatus(processInspecion.getStatus());
//            previousItem.setRejectNum(processInspecion.getRejectNum());
//            previousItem.setExaminingNum(processInspecion.getExaminingNum());
//            previousItem.setDisqualificationNum(processInspecion.getDisqualificationNum());
//        } else {
//            previousItem = new BizProcessInspecion(processInspecion);
        String beginCode = "WDD" + DateTime.now().toString("yyMM");
        String previous = redisCache.hasKey(processInspectionNoKey) ? redisCache.getCacheObject(processInspectionNoKey) : "";
        String codeValue = "";
        if (previous.isEmpty() || !previous.startsWith(beginCode)) {
            previous = (beginCode + "000001");
        } else {
            codeValue =  previous.startsWith(beginCode) ? previous : beginCode + "000000";
            int lastFourDigits = Integer.parseInt(codeValue.substring(7, 13));
            lastFourDigits += 1; // 加1
            codeValue = codeValue.substring(0, 7) + String.format("%06d", lastFourDigits);
            previous = codeValue;
        }
        processInspecion.setOutsourcedInspectionNo(previous);
        redisCache.setCacheObject(processInspectionNoKey, previous);
//        }
        processInspecion.setReturnReceiptDetailId(bizReturnReceiptDetail.getId());
        processInspecion.setUserId(user == null ? 0L : user.getUserId());
        processInspecion.setNickName(user == null ? "" : user.getNickName());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setIsOutsourcedInspection(1);
        processInspecion.setProductDescription(processInspecion.getLot().getProductDescription());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        processInspecion.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
        processInspecion.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        processInspecion.setOutsourceOrderId(outsourceOrderDetail.getMasterId());

        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }
        if (processInspecion.getInspectionStatus() != 0) {
            processInspecion.setEndTime(new Date());
            processInspecion.setFirstUpdaterId(processInspecion.getFirstUpdaterId() == null || processInspecion.getFirstUpdaterId().equals(0L) ? user != null ? user.getUserId() : 0L : processInspecion.getFirstUpdaterId());

        }
        if (!checkOutsourceDayworkItem(processInspecion.getOutsourceOrderDetailId())) {
            return error("编辑失败，下序已经生产加工");
        }

        BizCarrier carrier = bizCarrierService.query().eq("id", processInspecion.getInspectionCarrierId()).one();
        List<BizDayworkCarrier> dayworkCarriers = bizDayworkCarrierService.query().eq("carrier_id", carrier.getId()).eq("is_changed", 0).list();
        if (dayworkCarriers.stream().anyMatch(v -> !v.getDayworkId().equals(daywork.getId()))) {
            return error("检测载具已绑定其他批次");
        }
        if (previousItem != null && previousItem.getAuditStatus() == 1) {
            return error("已通过结算审核，请取消后再进行质检。");
        }
        //保存
        if (previousItem != null) {
            previousItem.setRecheckDate(DateTime.now().toJdkDate());
            bizProcessInspecionService.updateById(previousItem);
        }
        bizProcessInspecionService.saveOrUpdate(processInspecion);

        if (dayworkCarriers.stream().noneMatch(v -> v.getDayworkId().equals(daywork.getId())) && (processInspecion.getIsInspectionCarrierChanged() == null || processInspecion.getIsInspectionCarrierChanged().equals(0))) {
            // 该检测载具需要绑定到当前批次
            BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
            newDayworkCarrier.setDayworkId(processInspecion.getDayworkId());
            newDayworkCarrier.setCarrierCode(carrier.getCode());
            newDayworkCarrier.setCarrierId(carrier.getId());
            newDayworkCarrier.setLotCode(processInspecion.getLotCode());
            newDayworkCarrier.setLotId(processInspecion.getLotId());
            newDayworkCarrier.setProductionPlanDetailId(processInspecion.getProductionPlanDetailId());
            newDayworkCarrier.setProcessInspectionId(processInspecion.getId());
            bizDayworkCarrierService.save(newDayworkCarrier);
        }
        List<BizDayworkItemConsult> consults = new ArrayList<>();
        if (processInspecion.getId() != null) {
            //保存废品信息
            List<BizProcessInspectionDetail> rejects = processInspecion.getRejects();
            //保存前先删除原先废品信息
            BizDayworkItemReject rej = new BizDayworkItemReject();
            rej.setProcessInspectionId(processInspecion.getId());
            bizDayworkItemRejectService.removeReject(rej);
            BizProcessInspectionDetail inspectionDetail = new BizProcessInspectionDetail();
            inspectionDetail.setProcessInspectionId(processInspecion.getId());
            processInspectionDetailService.physicalDelete(inspectionDetail);
            BizDayworkItemReject newRej = new BizDayworkItemReject();
            newRej.setRejectNum(0L);

            if (rejects != null && rejects.size() > 0) {
                for (BizProcessInspectionDetail reject : rejects) {
                    reject.setId(null);
                    reject.setUserId(user == null ? 0L : user.getUserId());
                    reject.setProcessInspectionId(processInspecion.getId());
                    reject.setDayworkId(0L);
                    reject.setProductionPlanId(daywork.getProductionPlanId());
                    reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                    reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                    reject.setLotId(processInspecion.getLot().getId());
                    reject.setDayworkItemId(dayworkItem.getId());
                    Long rejectNum = reject.getRejectNum() == null ? 0L : reject.getRejectNum();
                    newRej.setRejectNum(rejectNum);
                    newRej.setUserId(user == null ? 0L : user.getUserId());
                    newRej.setProcessInspectionId(processInspecion.getId());
                    newRej.setDayworkId(0L);
                    newRej.setProductionPlanId(daywork.getProductionPlanId());
                    newRej.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                    newRej.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                    newRej.setLotId(processInspecion.getLot().getId());
                    newRej.setDayworkItemId(dayworkItem.getId());
                    newRej.setReason("外协检查废品");
                    newRej.setDeptId(processInspecion.getDeptId());
                }
            }
            processInspectionDetailService.saveBatch(rejects);
            bizDayworkItemRejectService.save(newRej);
            //保存咨询信息
            consults.addAll(processInspecion.getDayworkItemConsults() == null ? new ArrayList<>() : processInspecion.getDayworkItemConsults());
            if (consults.size() > 0) {
                for (BizDayworkItemConsult consult : consults) {
                    consult.setUserId(user == null ? 0L : user.getUserId());
                    consult.setNickName(user == null ? "" : user.getNickName());
                    consult.setDayworkId(daywork.getId());
                    consult.setProcessInspectionId(processInspecion.getId());
                    consult.setProductionPlanId(daywork.getProductionPlanId());
                    consult.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                    consult.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                    consult.setLotId(processInspecion.getLot().getId());
                    consult.setLotCode(processInspecion.getLot().getLotCode());
                    consult.setDayworkId(daywork.getId());
                    consult.setDayworkItemId(dayworkItem.getId());
                    consult.setProductId(processInspecion.getLot().getProductId());
                    consult.setProductDescription(processInspecion.getLot().getProductDescription());
                    consult.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                    consult.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                    consult.setProcessId(dayworkItem.getProcessId());
                    consult.setProcessAlias(processInspecion.getLot().getProcessAlias());
                    consult.setIsOutsourcedInspection(1);
                    consult.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
                }
            }
        }

        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem) && (consults.isEmpty() || bizDayworkItemConsultService.saveOrUpdateBatch(consults)));
    }

    @Transactional
    @PostMapping("/saveOutsourcedInspecion")
    @Log(title = "新增外协序检信息", businessType = BusinessType.INSERT)
    public AjaxResult saveOutsourcedInspectionV4(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
//        // 判断是否已有未确认外协检查
//        if (bizProcessInspecionService.query().eq("outsource_order_detail_id", processInspecion.getOutsourceOrderDetailId()).eq("status", 0).count() > 0) {
//            return error("已有未完成外协检查单。");
//        }
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());
        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getLot().getDayworkItemId());
//        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        BizReturnReceiptDetail bizReturnReceiptDetail = bizReturnReceiptDetailService.query().eq("daywork_id", dayworkItem.getDayworkId()).eq("technological_process_detail_id", dayworkItem.getTechnologicalProcessDetailId()).eq("deleted", 0).one();

        SysUser user = sysUserService.selectUserById(processInspecion.getUser().getUserId());

        // 判断是否已经进行过质检
        BizProcessInspecion previousItem = bizProcessInspecionService.query().eq("outsource_order_detail_id", bizReturnReceiptDetail.getOutsourceDetailId()).isNull("recheck_date").ne("id", processInspecion.getId() == null ? 0L : processInspecion.getId()).one();
        BizOutsourcedOrderDetail outsourceOrderDetail = bizOutsourcedOrderDetailService.query().eq("id", bizReturnReceiptDetail.getOutsourceDetailId()).one();

//        if (previousItem != null) {
//            if (processInspecion.getId() == null) {
//                previousItem.setRecheckDate(DateTime.now().toJdkDate());
//            }
//            previousItem.setIsInspectionCarrierChanged(0);
//            previousItem.setInspectionCarrierChangedTime(null);
//            previousItem.setStatus(processInspecion.getStatus());
//            previousItem.setRejectNum(processInspecion.getRejectNum());
//            previousItem.setExaminingNum(processInspecion.getExaminingNum());
//            previousItem.setDisqualificationNum(processInspecion.getDisqualificationNum());
//        } else {
//            previousItem = new BizProcessInspecion(processInspecion);
        String beginCode = "WDD" + DateTime.now().toString("yyMM");
        String previous = redisCache.hasKey(processInspectionNoKey) ? redisCache.getCacheObject(processInspectionNoKey) : "";
        String codeValue = "";
        if (previous.isEmpty()) {
            List<BizProcessInspecion> processInspectionList = bizProcessInspecionService.getAllList();
            if(!processInspectionList.isEmpty()) {
                codeValue = processInspectionList.get(0).getOutsourcedInspectionNo();
                int lastFourDigits = Integer.parseInt(codeValue.substring(7, 13));
                lastFourDigits ++; // 加1
                codeValue = codeValue.substring(0, 7) + String.format("%06d", lastFourDigits);
                previous = codeValue;
            }else{
                previous = (beginCode + "000001");
            }
        } else {
            codeValue = previous.startsWith(beginCode) ? previous : beginCode + "000000";
            int lastFourDigits = Integer.parseInt(codeValue.substring(7, 13));
            lastFourDigits ++; // 加1
            codeValue = codeValue.substring(0, 7) + String.format("%06d", lastFourDigits);
            previous = codeValue;
        }
        processInspecion.setOutsourcedInspectionNo(previous);
        redisCache.setCacheObject(processInspectionNoKey, previous);
//        }
        processInspecion.setReturnReceiptDetailId(bizReturnReceiptDetail.getId());
//        processInspecion.setUserId(user == null ? 0L : user.getUserId());
//        processInspecion.setNickName(user == null ? "" : user.getNickName());
        if (processInspecion.getId() == null) {
            processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
            processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
            processInspecion.setInspectionStatus(0);
        }
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setIsOutsourcedInspection(1);
        processInspecion.setProcessStepNumber(dayworkItem.getProcessStepNumber());
        processInspecion.setProductDescription(processInspecion.getLot().getProductDescription());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        processInspecion.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
        processInspecion.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        processInspecion.setOutsourceOrderId(outsourceOrderDetail.getMasterId());
        processInspecion.setReviewerId(SecurityUtils.getLoginUser().getUserId());
        processInspecion.setProcessStepNumber(processInspecion.getLot().getProcessStepNumber());
        processInspecion.setType("outsourcedInspector");

        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }
        if (processInspecion.getInspectionStatus() != 0) {
            processInspecion.setEndTime(new Date());
            processInspecion.setFirstUpdaterId(processInspecion.getFirstUpdaterId() == null || processInspecion.getFirstUpdaterId().equals(0L) ? user != null ? user.getUserId() : 0L : processInspecion.getFirstUpdaterId());

        }
        if (!checkOutsourceDayworkItem(processInspecion.getOutsourceOrderDetailId())) {
            return error("编辑失败，下序已经生产加工");
        }

        BizCarrier carrier = bizCarrierService.query().eq("id", processInspecion.getInspectionCarrierId()).one();
        List<BizDayworkCarrier> dayworkCarriers = bizDayworkCarrierService.query().eq("carrier_id", carrier.getId()).eq("is_changed", 0).list();
        if (dayworkCarriers.stream().anyMatch(v -> !v.getDayworkId().equals(daywork.getId()))) {
            return error("检测载具已绑定其他批次");
        }
        if (previousItem != null && previousItem.getAuditStatus() == 1) {
            return error("已通过结算审核，请取消后再进行质检。");
        }
        Boolean isSave = false;
        if (processInspecion.getId() == null) {
            isSave = true;
        }
        //保存
        if (previousItem != null) {
            previousItem.setRecheckDate(DateTime.now().toJdkDate());
            bizProcessInspecionService.updateById(previousItem);
        }
        bizProcessInspecionService.saveOrUpdate(processInspecion);
        //移交仪器室
        if (processInspecion.getFlag() != null && processInspecion.getFlag()) {
            BizProcessInspecion inspection = new BizProcessInspecion();
            inspection.setUserId(processInspecion.getLot().getUserId());
            inspection.setNickName(processInspecion.getLot().getNickName());
            inspection.setCarrierId(processInspecion.getCarrierId());
            inspection.setCarrierCode(processInspecion.getCarrierCode());
            inspection.setDayworkId(daywork.getId());
            inspection.setProcessStepNumber(processInspecion.getLot().getProcessStepNumber());
            inspection.setProductionPlanId(daywork.getProductionPlanId());
            inspection.setDayworkItemId(dayworkItem.getId());
            inspection.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
            inspection.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
            inspection.setProductDescription(processInspecion.getLot().getProductDescription());
            inspection.setLotId(processInspecion.getLot().getId());
            inspection.setLotCode(processInspecion.getLot().getLotCode());
            inspection.setDeptId(daywork.getDeptId());
            inspection.setProductId(processInspecion.getLot().getProductId());
            inspection.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
            inspection.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
            inspection.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
            inspection.setProcessStepNumber(processInspecion.getProcessStepNumber());
            inspection.setEquipmentDetailCode(processInspecion.getLot().getEquipmentDetailCode());
            inspection.setEquipmentDetailId(processInspecion.getLot().getEquipmentDetailId());
            inspection.setInspectionChamberId(processInspecion.getInspectionChamberId());
            //inspection.setReviewerId(SecurityUtils.getLoginUser().getUserId());
            inspection.setProcessStepNumber(processInspecion.getProcessStepNumber());
            inspection.setFromId(processInspecion.getId());
            inspection.setType("instrumentRoomInspection");
            if (inspection.getStartTime() == null) {
                inspection.setStartTime(new Date());
            }
            bizProcessInspecionService.save(inspection);
//            //保存检查明细
//            List<BizInspectionStandardsCheck> inspectionStandardsCheckList = bizInspectionStandardsCheckService.query()
//                    .eq("inspection_code", "instrumentRoomInspection")
//                    .eq("product_id", inspection.getProductId())
//                    .eq("process_id", processInspecion.getLot().getProcessId()).list();
//            List<BizProcessInspectionDetail> detailList = new ArrayList<>();
//            inspectionStandardsCheckList.forEach(inspectionStandardsCheck -> {
//                BizProcessInspectionDetail detail = new BizProcessInspectionDetail();
//                detail.setId(null);
//                detail.setUserId(processInspecion.getLot().getUserId());
//                detail.setProcessInspectionId(inspection.getId());
//                detail.setDayworkId(0L);
//                detail.setProductionPlanId(daywork.getProductionPlanId());
//                detail.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
//                detail.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
//                detail.setLotId(processInspecion.getLot().getId());
//                detail.setDayworkItemId(dayworkItem.getId());
//                detail.setCheckStandard(inspectionStandardsCheck.getStandard());
//                detail.setInspectionStandardsId(inspectionStandardsCheck.getId());
//                detailList.add(detail);
//            });
//            if(!detailList.isEmpty()) {
//                processInspectionDetailService.saveBatch(detailList);
//            }
            //绑定检测载具
            BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
            newDayworkCarrier.setDayworkId(inspection.getDayworkId());
            newDayworkCarrier.setCarrierCode(inspection.getInspectionCarrierCode());
            newDayworkCarrier.setCarrierId(inspection.getInspectionCarrierId());
            newDayworkCarrier.setLotCode(inspection.getLotCode());
            newDayworkCarrier.setLotId(inspection.getLotId());
            newDayworkCarrier.setProductionPlanDetailId(inspection.getProductionPlanDetailId());
            newDayworkCarrier.setProcessInspectionId(inspection.getId());
            bizDayworkCarrierService.save(newDayworkCarrier);
        }
        if (isSave) {
            if (dayworkCarriers.stream().noneMatch(v -> v.getDayworkId().equals(daywork.getId())) && (processInspecion.getIsInspectionCarrierChanged() == null || processInspecion.getIsInspectionCarrierChanged().equals(0))) {
                // 该检测载具需要绑定到当前批次
                BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
                newDayworkCarrier.setDayworkId(processInspecion.getDayworkId());
                newDayworkCarrier.setCarrierCode(carrier.getCode());
                newDayworkCarrier.setCarrierId(carrier.getId());
                newDayworkCarrier.setLotCode(processInspecion.getLotCode());
                newDayworkCarrier.setLotId(processInspecion.getLotId());
                newDayworkCarrier.setProductionPlanDetailId(processInspecion.getProductionPlanDetailId());
                newDayworkCarrier.setProcessInspectionId(processInspecion.getId());
                bizDayworkCarrierService.save(newDayworkCarrier);
            }
        }
        List<BizDayworkItemConsult> consults = new ArrayList<>();
        if (processInspecion.getId() != null) {
            //保存废品信息
            List<BizProcessInspectionDetail> rejects = processInspecion.getRejects();
            //保存前先删除原先废品信息
            BizDayworkItemReject rej = new BizDayworkItemReject();
            rej.setProcessInspectionId(processInspecion.getId());
            bizDayworkItemRejectService.removeReject(rej);
            BizProcessInspectionDetail inspectionDetail = new BizProcessInspectionDetail();
            inspectionDetail.setProcessInspectionId(processInspecion.getId());
            processInspectionDetailService.physicalDelete(inspectionDetail);
            BizDayworkItemReject newRej = new BizDayworkItemReject();
            newRej.setRejectNum(0L);

            if (rejects != null && rejects.size() > 0) {
                for (BizProcessInspectionDetail reject : rejects) {
                    reject.setId(null);
                    reject.setUserId(user == null ? 0L : user.getUserId());
                    reject.setProcessInspectionId(processInspecion.getId());
                    reject.setDayworkId(0L);
                    reject.setProductionPlanId(daywork.getProductionPlanId());
                    reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                    reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                    reject.setLotId(processInspecion.getLot().getId());
                    reject.setDayworkItemId(dayworkItem.getId());
                    Long rejectNum = reject.getRejectNum() == null ? 0L : reject.getRejectNum();
                    newRej.setRejectNum(rejectNum);
                    newRej.setUserId(user == null ? 0L : user.getUserId());
                    newRej.setProcessInspectionId(processInspecion.getId());
                    newRej.setDayworkId(0L);
                    newRej.setProductionPlanId(daywork.getProductionPlanId());
                    newRej.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                    newRej.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                    newRej.setLotId(processInspecion.getLot().getId());
                    newRej.setDayworkItemId(dayworkItem.getId());
                    newRej.setReason("外协检查废品");
                    newRej.setDeptId(processInspecion.getDeptId());
                }
            }
            processInspectionDetailService.saveBatch(rejects);
            bizDayworkItemRejectService.save(newRej);
            //保存上传图片信息
            if (!isSave) {
                List<BizProcessInspectionPicture> oldProcessInspectionList = bizProcessInspectionPictureService.query().eq("process_inspection_id", processInspecion.getId()).list();
                if (processInspecion.getProcessInspectionPictureList() != null && !processInspecion.getProcessInspectionPictureList().isEmpty()) {
                    processInspecion.getProcessInspectionPictureList().forEach(item -> {
                        item.setProcessInspectionId(processInspecion.getId());
                    });
                }
                saveOrUpdateBatch(bizProcessInspectionPictureService, processInspecion.getProcessInspectionPictureList() == null || processInspecion.getProcessInspectionPictureList().isEmpty() ? new ArrayList<>() : processInspecion.getProcessInspectionPictureList(), oldProcessInspectionList);
            }
            //保存咨询信息
            consults.addAll(processInspecion.getDayworkItemConsults() == null ? new ArrayList<>() : processInspecion.getDayworkItemConsults());
            if (consults.size() > 0) {
                for (BizDayworkItemConsult consult : consults) {
                    consult.setUserId(user == null ? 0L : user.getUserId());
                    consult.setNickName(user == null ? "" : user.getNickName());
                    consult.setDayworkId(daywork.getId());
                    consult.setProcessInspectionId(processInspecion.getId());
                    consult.setProductionPlanId(daywork.getProductionPlanId());
                    consult.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                    consult.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                    consult.setLotId(processInspecion.getLot().getId());
                    consult.setLotCode(processInspecion.getLot().getLotCode());
                    consult.setDayworkId(daywork.getId());
                    consult.setDayworkItemId(dayworkItem.getId());
                    consult.setProductId(processInspecion.getLot().getProductId());
                    consult.setProductDescription(processInspecion.getLot().getProductDescription());
                    consult.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                    consult.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                    consult.setProcessId(dayworkItem.getProcessId());
                    consult.setProcessAlias(processInspecion.getLot().getProcessAlias());
                    consult.setIsOutsourcedInspection(1);
                    //consult.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
                }
                bizDayworkItemConsultService.saveOrUpdateBatch(consults);
            }
        }

        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        if(dayworkItem.getDeleted() == 1) {
            return success();
        }
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem));
    }


    @Transactional
//    @PostMapping("/saveOutsourcedInspecion")
    public AjaxResult saveOutsourcedInspectionV2(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
//        // 判断是否已有未确认外协检查
//        if (bizProcessInspecionService.query().eq("outsource_order_detail_id", processInspecion.getOutsourceOrderDetailId()).eq("status", 0).count() > 0) {
//            return error("已有未完成外协检查单。");
//        }
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());
        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        BizReturnReceiptDetail bizReturnReceiptDetail = bizReturnReceiptDetailService.query().eq("daywork_id", dayworkItem.getDayworkId()).eq("technological_process_detail_id", dayworkItem.getTechnologicalProcessDetailId()).eq("deleted", 0).one();
        BizOutsourcedOrderDetail outsourceOrderDetail = bizOutsourcedOrderDetailService.query().eq("id", bizReturnReceiptDetail.getOutsourceDetailId()).one();

        // 判断是否已经进行过质检
//        BizProcessInspecion previousItem = bizProcessInspecionService.query().eq("outsource_order_detail_id", bizReturnReceiptDetail.getOutsourceDetailId()).one();
//        previousItem.setRecheckDate(DateTime.now().toJdkDate());

        SysUser user = sysUserService.selectUserById(processInspecion.getUser().getUserId());
        processInspecion.setReturnReceiptDetailId(bizReturnReceiptDetail.getId());
        processInspecion.setUserId(user == null ? 0L : user.getUserId());
        processInspecion.setNickName(user == null ? "" : user.getNickName());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setProductDescription(processInspecion.getLot().getProductDescription());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        processInspecion.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
        processInspecion.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        processInspecion.setOutsourceOrderId(outsourceOrderDetail.getMasterId());
        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }
        if (processInspecion.getInspectionStatus() != 0) {
            processInspecion.setEndTime(new Date());
            processInspecion.setFirstUpdaterId(processInspecion.getFirstUpdaterId() == null || processInspecion.getFirstUpdaterId().equals(0L) ? user != null ? user.getUserId() : 0L : processInspecion.getFirstUpdaterId());

        }
        if (!checkOutsourceDayworkItem(processInspecion.getOutsourceOrderDetailId())) {
            return error("编辑失败，下序已经生产加工");
        }

        BizCarrier carrier = bizCarrierService.query().eq("id", processInspecion.getInspectionCarrierId()).one();
        List<BizDayworkCarrier> dayworkCarriers = bizDayworkCarrierService.query().eq("carrier_id", carrier.getId()).eq("is_changed", 0).list();
        if (dayworkCarriers.stream().anyMatch(v -> !v.getDayworkId().equals(processInspecion.getDayworkId()))) {
            return error("检测载具已绑定其他批次");
        }

        //保存
        bizProcessInspecionService.saveOrUpdate(processInspecion);

        if (dayworkCarriers.stream().noneMatch(v -> v.getDayworkId().equals(processInspecion.getDayworkId())) && (processInspecion.getIsInspectionCarrierChanged() == null || processInspecion.getIsInspectionCarrierChanged().equals(0))) {
            // 该检测载具需要绑定到当前批次
            BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
            newDayworkCarrier.setDayworkId(processInspecion.getDayworkId());
            newDayworkCarrier.setCarrierCode(carrier.getCode());
            newDayworkCarrier.setCarrierId(carrier.getId());
            newDayworkCarrier.setLotCode(processInspecion.getLotCode());
            newDayworkCarrier.setLotId(processInspecion.getLotId());
            newDayworkCarrier.setProductionPlanDetailId(processInspecion.getProductionPlanDetailId());
            newDayworkCarrier.setProcessInspectionId(processInspecion.getId());
            bizDayworkCarrierService.save(newDayworkCarrier);
        }

        //保存废品信息
        List<BizProcessInspectionDetail> rejects = processInspecion.getRejects();
        //保存前先删除原先废品信息
        BizDayworkItemReject rej = new BizDayworkItemReject();
        rej.setProcessInspectionId(processInspecion.getId());
        bizDayworkItemRejectService.removeReject(rej);
        BizProcessInspectionDetail inspectionDetail = new BizProcessInspectionDetail();
        inspectionDetail.setProcessInspectionId(processInspecion.getId());
        processInspectionDetailService.physicalDelete(inspectionDetail);
        BizDayworkItemReject newRej = new BizDayworkItemReject();
        newRej.setRejectNum(0L);

        if (rejects != null && rejects.size() > 0) {
            for (BizProcessInspectionDetail reject : rejects) {
                reject.setId(null);
                reject.setUserId(user == null ? 0L : user.getUserId());
                reject.setProcessInspectionId(processInspecion.getId());
                reject.setDayworkId(0L);
                reject.setProductionPlanId(daywork.getProductionPlanId());
                reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                reject.setLotId(processInspecion.getLot().getId());
                reject.setDayworkItemId(dayworkItem.getId());
                Long rejectNum = reject.getRejectNum() == null ? 0L : reject.getRejectNum();
                newRej.setRejectNum(newRej.getRejectNum());
                newRej.setUserId(user == null ? 0L : user.getUserId());
                newRej.setProcessInspectionId(processInspecion.getId());
                newRej.setDayworkId(0L);
                newRej.setProductionPlanId(daywork.getProductionPlanId());
                newRej.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                newRej.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                newRej.setLotId(processInspecion.getLot().getId());
                newRej.setDayworkItemId(dayworkItem.getId());
                newRej.setReason("外协检查废品");
                newRej.setDeptId(processInspecion.getDeptId());
            }
        }
        processInspectionDetailService.saveBatch(rejects);
        bizDayworkItemRejectService.save(newRej);
        //保存咨询信息

        List<BizDayworkItemConsult> consults = processInspecion.getDayworkItemConsults().stream().filter(item -> item.getId() == null).collect(Collectors.toList());
        List<BizDayworkItemConsult> needAdd = new ArrayList<>();
        if (consults != null && consults.size() > 0) {
            for (BizDayworkItemConsult consult : consults) {
                BizDayworkItemConsult consultInfo = new BizDayworkItemConsult();
                consultInfo.setPictures(consult.getPictures());
                consultInfo.setUserId(user == null ? 0L : user.getUserId());
                consultInfo.setNickName(user == null ? "" : user.getNickName());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setContent(consult.getContent());
                consultInfo.setConsultDepartment(consult.getConsultDepartment());
                consultInfo.setProcessInspectionId(processInspecion.getId());
                consultInfo.setProductionPlanId(daywork.getProductionPlanId());
                consultInfo.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                consultInfo.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                consultInfo.setLotId(processInspecion.getLot().getId());
                consultInfo.setLotCode(processInspecion.getLot().getLotCode());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setDayworkItemId(dayworkItem.getId());
                consultInfo.setProductId(processInspecion.getLot().getProductId());
                consultInfo.setProductDescription(processInspecion.getLot().getProductDescription());
                consultInfo.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                consultInfo.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                consultInfo.setProcessId(dayworkItem.getProcessId());
                consultInfo.setProcessAlias(processInspecion.getLot().getProcessAlias());
                consultInfo.setIsOutsourcedInspection(1);
                consultInfo.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
                needAdd.add(consultInfo);
            }
            bizDayworkItemConsultService.saveBatch(needAdd);
            List<BizDayworkItemConsultPicture> pictures = new ArrayList<>();
            needAdd.forEach(ne -> {
                ne.getPictures().forEach(info -> {
                    BizDayworkItemConsultPicture consultPicture = new BizDayworkItemConsultPicture();
                    consultPicture.setDayworkItemConsultId(ne.getId());
                    consultPicture.setPictureName(info.getPictureName());
                    consultPicture.setUrl(info.getUrl());
                    pictures.add(consultPicture);
                });
            });
            bizDayworkItemConsultPictureService.saveBatch(pictures);
        }

        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        if(dayworkItem.getDeleted() == 1) {
            return success();
        }
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem));
    }

    @Transactional
    @PostMapping("/saveOnSiteInspection")
    @Log(title = "新增巡检信息", businessType = BusinessType.INSERT)
    public AjaxResult saveOnSiteInspection(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        if (processInspecion.getFlag() != null && processInspecion.getFlag() && (processInspecion.getIsInspectionCarrierChanged() != null && processInspecion.getIsInspectionCarrierChanged().equals(1))) {
            throw new RuntimeException("检测载具已解绑，移交仪器室，请绑定载具");
        }
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());
        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getLot().getDayworkItemId());
//        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        processInspecion.setUserId(processInspecion.getLot().getUserId());
        processInspecion.setNickName(processInspecion.getLot().getNickName());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProcessStepNumber(processInspecion.getLot().getProcessStepNumber());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setProductDescription(processInspecion.getLot().getProductDescription());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(processInspecion.getDeptId() != null && !processInspecion.getDeptId().equals(0L) ? processInspecion.getDeptId() : daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        processInspecion.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
        processInspecion.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        processInspecion.setEquipmentDetailCode(processInspecion.getLot().getEquipmentDetailCode());
        processInspecion.setEquipmentDetailId(processInspecion.getLot().getEquipmentDetailId());
        processInspecion.setReviewerId(SecurityUtils.getLoginUser().getUserId());
        processInspecion.setType("patrolInspection");
        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }
        if (processInspecion.getInspectionStatus() != 0) {
            processInspecion.setEndTime(new Date());
            processInspecion.setFirstUpdaterId(processInspecion.getFirstUpdaterId() == null || processInspecion.getFirstUpdaterId().equals(0L) ? SecurityUtils.getLoginUser().getUserId() : processInspecion.getFirstUpdaterId());
        }

        // 20241015 取消下序生产时不能编辑限制
//        if (!checkDayworkItem(processInspecion.getLot().getDayworkItemId())) {
//            return error("编辑失败，下序已经生产加工");
//        }

        //BizCarrier carrier = bizCarrierService.query().eq("id", processInspecion.getInspectionCarrierId()).one();
        List<BizDayworkCarrier> dayworkCarriers = bizDayworkCarrierService.query().eq("carrier_id", processInspecion.getInspectionCarrierId()).eq("is_changed", 0).list();
        if (dayworkCarriers.stream().anyMatch(v -> !v.getDayworkId().equals(processInspecion.getDayworkId()))) {
            return error("检测载具已绑定其他批次");
        }
        Boolean isSave = false;
        if (processInspecion.getId() == null) {
            isSave = true;
        }
        //保存
        bizProcessInspecionService.saveOrUpdate(processInspecion);
        if (isSave) {
            if (dayworkCarriers.stream().noneMatch(v -> v.getDayworkId().equals(processInspecion.getDayworkId())) && (processInspecion.getIsInspectionCarrierChanged() == null || processInspecion.getIsInspectionCarrierChanged().equals(0))) {
                // 该检测载具需要绑定到当前批次
                BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
                newDayworkCarrier.setDayworkId(processInspecion.getDayworkId());
                newDayworkCarrier.setTenantId(processInspecion.getTenantId());
                newDayworkCarrier.setCarrierCode(processInspecion.getInspectionCarrierCode());
                newDayworkCarrier.setCarrierId(processInspecion.getInspectionCarrierId());
                newDayworkCarrier.setLotCode(processInspecion.getLotCode());
                newDayworkCarrier.setLotId(processInspecion.getLotId());
                newDayworkCarrier.setProductionPlanDetailId(processInspecion.getProductionPlanDetailId());
                newDayworkCarrier.setProcessInspectionId(processInspecion.getId());
                bizDayworkCarrierService.save(newDayworkCarrier);
            }
        }
        //移交仪器室
        if (processInspecion.getFlag() != null && processInspecion.getFlag()) {
            BizProcessInspecion inspection = new BizProcessInspecion();
            inspection.setUserId(processInspecion.getLot().getUserId());
            inspection.setTenantId(processInspecion.getTenantId());
            inspection.setNickName(processInspecion.getLot().getNickName());
            inspection.setCarrierId(processInspecion.getCarrierId());
            inspection.setCarrierCode(processInspecion.getCarrierCode());
            inspection.setDayworkId(daywork.getId());
            inspection.setProcessStepNumber(processInspecion.getLot().getProcessStepNumber());
            inspection.setProductionPlanId(daywork.getProductionPlanId());
            inspection.setDayworkItemId(dayworkItem.getId());
            inspection.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
            inspection.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
            inspection.setProductDescription(processInspecion.getLot().getProductDescription());
            inspection.setLotId(processInspecion.getLot().getId());
            inspection.setLotCode(processInspecion.getLot().getLotCode());
            inspection.setDeptId(processInspecion.getDeptId() != null && !processInspecion.getDeptId().equals(0L) ? processInspecion.getDeptId() : daywork.getDeptId());
            inspection.setProductId(processInspecion.getLot().getProductId());
            inspection.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
            inspection.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
            inspection.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
            inspection.setProcessStepNumber(processInspecion.getProcessStepNumber());
            inspection.setEquipmentDetailCode(processInspecion.getLot().getEquipmentDetailCode());
            inspection.setEquipmentDetailId(processInspecion.getLot().getEquipmentDetailId());
            inspection.setInspectionChamberId(processInspecion.getInspectionChamberId());
            //inspection.setReviewerId(SecurityUtils.getLoginUser().getUserId());
            inspection.setProcessStepNumber(processInspecion.getProcessStepNumber());
            inspection.setFromId(processInspecion.getId());
            inspection.setType("instrumentRoomInspection");
            if (inspection.getStartTime() == null) {
                inspection.setStartTime(new Date());
            }
            bizProcessInspecionService.save(inspection);
//            //保存检查明细
//            List<BizInspectionStandardsCheck> inspectionStandardsCheckList = bizInspectionStandardsCheckService.query()
//                    .eq("inspection_code", "instrumentRoomInspection")
//                    .eq("product_id", inspection.getProductId())
//                    .eq("process_id", processInspecion.getLot().getProcessId()).list();
//            List<BizProcessInspectionDetail> detailList = new ArrayList<>();
//            inspectionStandardsCheckList.forEach(inspectionStandardsCheck -> {
//                BizProcessInspectionDetail detail = new BizProcessInspectionDetail();
//                detail.setId(null);
//                detail.setUserId(processInspecion.getLot().getUserId());
//                detail.setProcessInspectionId(inspection.getId());
//                detail.setDayworkId(0L);
//                detail.setProductionPlanId(daywork.getProductionPlanId());
//                detail.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
//                detail.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
//                detail.setLotId(processInspecion.getLot().getId());
//                detail.setDayworkItemId(dayworkItem.getId());
//                detail.setCheckStandard(inspectionStandardsCheck.getStandard());
//                detail.setInspectionStandardsId(inspectionStandardsCheck.getId());
//                detailList.add(detail);
//            });
//            if(!detailList.isEmpty()) {
//                processInspectionDetailService.saveBatch(detailList);
//            }
            //绑定检测载具
            BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
            newDayworkCarrier.setDayworkId(inspection.getDayworkId());
            newDayworkCarrier.setTenantId(processInspecion.getTenantId());
            newDayworkCarrier.setCarrierCode(inspection.getInspectionCarrierCode());
            newDayworkCarrier.setCarrierId(inspection.getInspectionCarrierId());
            newDayworkCarrier.setLotCode(inspection.getLotCode());
            newDayworkCarrier.setLotId(inspection.getLotId());
            newDayworkCarrier.setProductionPlanDetailId(inspection.getProductionPlanDetailId());
            newDayworkCarrier.setProcessInspectionId(inspection.getId());
            bizDayworkCarrierService.save(newDayworkCarrier);
        }

        // 判断假设已经全项检查
        if (processInspecion.getIsAllChecked() != null && processInspecion.getIsAllChecked().equals(1)) {
            // 解绑检测载具
            // 查询所有检测载具
            List<BizDayworkCarrier> bizDayworkCarriers = bizDayworkCarrierService.query().eq("daywork_id", processInspecion.getDayworkId()).eq("process_inspection_id", processInspecion.getId()).eq("is_changed", 0).list();
            bizDayworkCarriers.forEach(l -> {
                l.setIsChanged(1);
            });
            bizDayworkCarrierService.updateBatchById(bizDayworkCarriers);
            processInspecion.setIsInspectionCarrierChanged(1);
            bizProcessInspecionService.updateById(processInspecion);
        }

        //保存废品信息
        List<BizProcessInspectionDetail> rejects = processInspecion.getProcessInspectionDetails();
        //保存前先删除原先废品信息
        List<BizProcessInspectionDetail> processInspectionDetailList = processInspectionDetailService.query().eq("process_inspection_id", processInspecion.getId()).list();
        processInspectionDetailService.removeBatchByIds(processInspectionDetailList);

        if (rejects != null && rejects.size() > 0) {
            for (BizProcessInspectionDetail reject : rejects) {
                reject.setId(null);
                reject.setTenantId(processInspecion.getTenantId());
                reject.setUserId(processInspecion.getLot().getUserId());
                reject.setProcessInspectionId(processInspecion.getId());
                reject.setDayworkId(0L);
                reject.setProductionPlanId(daywork.getProductionPlanId());
                reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                reject.setLotId(processInspecion.getLot().getId());
                reject.setDayworkItemId(dayworkItem.getId());
            }
        }
        processInspectionDetailService.saveBatch(rejects);
        //保存上传图片信息
        List<BizProcessInspectionPicture> oldProcessInspectionList = bizProcessInspectionPictureService.query().eq("process_inspection_id", processInspecion.getId()).list();
        if (processInspecion.getProcessInspectionPictureList() != null && !processInspecion.getProcessInspectionPictureList().isEmpty()) {
            processInspecion.getProcessInspectionPictureList().forEach(item -> {
                item.setProcessInspectionId(processInspecion.getId());
            });
        }
        saveOrUpdateBatch(bizProcessInspectionPictureService, processInspecion.getProcessInspectionPictureList() == null || processInspecion.getProcessInspectionPictureList().isEmpty() ? new ArrayList<>() : processInspecion.getProcessInspectionPictureList(), oldProcessInspectionList);
        //保存咨询信息
        List<BizDayworkItemConsult> consults = processInspecion.getDayworkItemConsults().stream().filter(item -> item.getId() == null).collect(Collectors.toList());
        List<BizDayworkItemConsult> needAdd = new ArrayList<>();
        if (consults != null && consults.size() > 0) {
            for (BizDayworkItemConsult consult : consults) {
                BizDayworkItemConsult consultInfo = new BizDayworkItemConsult();
                consultInfo.setPictures(consult.getPictures());
                consultInfo.setTenantId(processInspecion.getTenantId());
                consultInfo.setUserId(SecurityUtils.getLoginUser().getUserId());
                consultInfo.setNickName(SecurityUtils.getLoginUser().getUser().getNickName());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setContent(consult.getContent());
                consultInfo.setConsultDepartment(consult.getConsultDepartment());
                consultInfo.setProcessInspectionId(processInspecion.getId());
                consultInfo.setProductionPlanId(daywork.getProductionPlanId());
                consultInfo.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                consultInfo.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                consultInfo.setLotId(processInspecion.getLot().getId());
                consultInfo.setLotCode(processInspecion.getLot().getLotCode());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setDayworkItemId(dayworkItem.getId());
                consultInfo.setProductId(processInspecion.getLot().getProductId());
                consultInfo.setProductDescription(processInspecion.getLot().getProductDescription());
                consultInfo.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                consultInfo.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                consultInfo.setProcessId(dayworkItem.getProcessId());
                consultInfo.setIsProcessInspection(1);
                consultInfo.setProcessAlias(processInspecion.getLot().getProcessAlias());
                //consultInfo.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
                consultInfo.setTechnicianId(consult.getTechnicianId());
                needAdd.add(consultInfo);
            }
            bizDayworkItemConsultService.saveBatch(needAdd);
            List<BizDayworkItemConsultPicture> pictures = new ArrayList<>();
            needAdd.forEach(ne -> {
                ne.getPictures().forEach(info -> {
                    BizDayworkItemConsultPicture consultPicture = new BizDayworkItemConsultPicture();
                    consultPicture.setDayworkItemConsultId(ne.getId());
                    consultPicture.setPictureName(info.getPictureName());
                    consultPicture.setUrl(info.getUrl());
                    consultPicture.setTenantId(processInspecion.getTenantId());
                    pictures.add(consultPicture);
                });
            });
            bizDayworkItemConsultPictureService.saveBatch(pictures);
        }

        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        dayworkItem.setTenantId(processInspecion.getTenantId());
        if(dayworkItem.getDeleted() == 1) {
            return success();
        }
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem));
    }

    @Transactional
    @PostMapping("/saveFirstInspection")
    @Log(title = "新增首序检信息", businessType = BusinessType.INSERT)
    public AjaxResult saveFirstInspection(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        if (processInspecion.getFlag() != null && processInspecion.getFlag() && (processInspecion.getIsInspectionCarrierChanged() != null && processInspecion.getIsInspectionCarrierChanged().equals(1))) {
            throw new RuntimeException("检测载具已解绑，移交仪器室，请绑定载具");
        }
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());
        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getLot().getDayworkItemId());
//        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        processInspecion.setUserId(processInspecion.getLot().getUserId());
        processInspecion.setNickName(processInspecion.getLot().getNickName());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProcessStepNumber(processInspecion.getLot().getProcessStepNumber());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setProductDescription(processInspecion.getLot().getProductDescription());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(processInspecion.getDeptId() != null && !processInspecion.getDeptId().equals(0L) ? processInspecion.getDeptId() : daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        processInspecion.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
        processInspecion.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        processInspecion.setEquipmentDetailCode(processInspecion.getLot().getEquipmentDetailCode());
        processInspecion.setEquipmentDetailId(processInspecion.getLot().getEquipmentDetailId());
        if (processInspecion.getId() != null || processInspecion.getIsReviewer()) {
            processInspecion.setReviewerId(SecurityUtils.getLoginUser().getUserId());
        }
        processInspecion.setType("firstArticleInspection");
        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }
        if (processInspecion.getInspectionStatus() != 0) {
            processInspecion.setEndTime(new Date());
            processInspecion.setFirstUpdaterId(processInspecion.getFirstUpdaterId() == null || processInspecion.getFirstUpdaterId().equals(0L) ? SecurityUtils.getLoginUser().getUserId() : processInspecion.getFirstUpdaterId());
        }

        // 20241015 取消下序生产时不能编辑限制
//        if (!checkDayworkItem(processInspecion.getLot().getDayworkItemId())) {
//            return error("编辑失败，下序已经生产加工");
//        }

        //BizCarrier carrier = bizCarrierService.query().eq("id", processInspecion.getInspectionCarrierId()).one();
        List<BizDayworkCarrier> dayworkCarriers = bizDayworkCarrierService.query().eq("carrier_id", processInspecion.getInspectionCarrierId()).eq("is_changed", 0).list();
        if (dayworkCarriers.stream().anyMatch(v -> !v.getDayworkId().equals(processInspecion.getDayworkId()))) {
            return error("检测载具已绑定其他批次");
        }
        Boolean isSave = false;
        if (processInspecion.getId() == null) {
            isSave = true;
        }
        //保存
        bizProcessInspecionService.saveOrUpdate(processInspecion);
        if (isSave) {
            if (dayworkCarriers.stream().noneMatch(v -> v.getDayworkId().equals(processInspecion.getDayworkId())) && (processInspecion.getIsInspectionCarrierChanged() == null || processInspecion.getIsInspectionCarrierChanged().equals(0))) {
                // 该检测载具需要绑定到当前批次
                BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
                newDayworkCarrier.setDayworkId(processInspecion.getDayworkId());
                newDayworkCarrier.setTenantId(processInspecion.getTenantId());
                newDayworkCarrier.setCarrierCode(processInspecion.getInspectionCarrierCode());
                newDayworkCarrier.setCarrierId(processInspecion.getInspectionCarrierId());
                newDayworkCarrier.setLotCode(processInspecion.getLotCode());
                newDayworkCarrier.setLotId(processInspecion.getLotId());
                newDayworkCarrier.setProductionPlanDetailId(processInspecion.getProductionPlanDetailId());
                newDayworkCarrier.setProcessInspectionId(processInspecion.getId());
                bizDayworkCarrierService.save(newDayworkCarrier);
            }
        }
        //移交仪器室
        if (processInspecion.getFlag() != null && processInspecion.getFlag()) {
            BizProcessInspecion inspection = new BizProcessInspecion();
            inspection.setUserId(processInspecion.getLot().getUserId());
            inspection.setNickName(processInspecion.getLot().getNickName());
            inspection.setCarrierId(processInspecion.getCarrierId());
            inspection.setTenantId(processInspecion.getTenantId());
            inspection.setCarrierCode(processInspecion.getCarrierCode());
            inspection.setDayworkId(daywork.getId());
            inspection.setProcessStepNumber(processInspecion.getLot().getProcessStepNumber());
            inspection.setProductionPlanId(daywork.getProductionPlanId());
            inspection.setDayworkItemId(dayworkItem.getId());
            inspection.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
            inspection.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
            inspection.setProductDescription(processInspecion.getLot().getProductDescription());
            inspection.setLotId(processInspecion.getLot().getId());
            inspection.setLotCode(processInspecion.getLot().getLotCode());
            inspection.setDeptId(processInspecion.getDeptId() != null && !processInspecion.getDeptId().equals(0L) ? processInspecion.getDeptId() : daywork.getDeptId());
            inspection.setProductId(processInspecion.getLot().getProductId());
            inspection.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
            inspection.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
            inspection.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
            inspection.setProcessStepNumber(processInspecion.getProcessStepNumber());
            inspection.setEquipmentDetailCode(processInspecion.getLot().getEquipmentDetailCode());
            inspection.setEquipmentDetailId(processInspecion.getLot().getEquipmentDetailId());
            inspection.setInspectionChamberId(processInspecion.getInspectionChamberId());
            //inspection.setReviewerId(SecurityUtils.getLoginUser().getUserId());
            inspection.setProcessStepNumber(processInspecion.getProcessStepNumber());
            inspection.setFromId(processInspecion.getId());
            inspection.setType("instrumentRoomInspection");
            if (inspection.getStartTime() == null) {
                inspection.setStartTime(new Date());
            }
            bizProcessInspecionService.save(inspection);
            //保存检查明细
//            List<BizInspectionStandardsCheck> inspectionStandardsCheckList = bizInspectionStandardsCheckService.query()
//                    .eq("inspection_code", "instrumentRoomInspection")
//                    .eq("product_id", inspection.getProductId())
//                    .eq("process_id", processInspecion.getLot().getProcessId()).list();
//            List<BizProcessInspectionDetail> detailList = new ArrayList<>();
//            inspectionStandardsCheckList.forEach(inspectionStandardsCheck -> {
//                BizProcessInspectionDetail detail = new BizProcessInspectionDetail();
//                detail.setId(null);
//                detail.setUserId(processInspecion.getLot().getUserId());
//                detail.setProcessInspectionId(inspection.getId());
//                detail.setDayworkId(0L);
//                detail.setProductionPlanId(daywork.getProductionPlanId());
//                detail.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
//                detail.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
//                detail.setLotId(processInspecion.getLot().getId());
//                detail.setDayworkItemId(dayworkItem.getId());
//                detail.setCheckStandard(inspectionStandardsCheck.getStandard());
//                detail.setInspectionStandardsId(inspectionStandardsCheck.getId());
//                detailList.add(detail);
//            });
//            if(!detailList.isEmpty()) {
//                processInspectionDetailService.saveBatch(detailList);
//            }
            //绑定检测载具
            BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
            newDayworkCarrier.setDayworkId(inspection.getDayworkId());
            newDayworkCarrier.setTenantId(processInspecion.getTenantId());
            newDayworkCarrier.setCarrierCode(inspection.getInspectionCarrierCode());
            newDayworkCarrier.setCarrierId(inspection.getInspectionCarrierId());
            newDayworkCarrier.setLotCode(inspection.getLotCode());
            newDayworkCarrier.setLotId(inspection.getLotId());
            newDayworkCarrier.setProductionPlanDetailId(inspection.getProductionPlanDetailId());
            newDayworkCarrier.setProcessInspectionId(inspection.getId());
            bizDayworkCarrierService.save(newDayworkCarrier);
        }

        // 判断假设已经全项检查
        if (processInspecion.getIsAllChecked() != null && processInspecion.getIsAllChecked().equals(1)) {
            // 解绑检测载具
            // 查询所有检测载具
            List<BizDayworkCarrier> bizDayworkCarriers = bizDayworkCarrierService.query().eq("daywork_id", processInspecion.getDayworkId()).eq("process_inspection_id", processInspecion.getId()).eq("is_changed", 0).list();
            bizDayworkCarriers.forEach(l -> {
                l.setIsChanged(1);
            });
            bizDayworkCarrierService.updateBatchById(bizDayworkCarriers);
            processInspecion.setIsInspectionCarrierChanged(1);
            bizProcessInspecionService.updateById(processInspecion);
        }

        //保存废品信息
        List<BizProcessInspectionDetail> rejects = processInspecion.getProcessInspectionDetails();
        //保存前先删除原先废品信息
        List<BizProcessInspectionDetail> processInspectionDetailList = processInspectionDetailService.query().eq("process_inspection_id", processInspecion.getId()).list();
        processInspectionDetailService.removeBatchByIds(processInspectionDetailList);

        if (rejects != null && rejects.size() > 0) {
            for (BizProcessInspectionDetail reject : rejects) {
                reject.setId(null);
                reject.setTenantId(processInspecion.getTenantId());
                reject.setUserId(processInspecion.getLot().getUserId());
                reject.setProcessInspectionId(processInspecion.getId());
                reject.setDayworkId(0L);
                reject.setProductionPlanId(daywork.getProductionPlanId());
                reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                reject.setLotId(processInspecion.getLot().getId());
                reject.setDayworkItemId(dayworkItem.getId());
            }
        }
        processInspectionDetailService.saveBatch(rejects);
        //保存上传图片信息
        List<BizProcessInspectionPicture> oldProcessInspectionList = bizProcessInspectionPictureService.query().eq("process_inspection_id", processInspecion.getId()).list();
        if (processInspecion.getProcessInspectionPictureList() != null && !processInspecion.getProcessInspectionPictureList().isEmpty()) {
            processInspecion.getProcessInspectionPictureList().forEach(item -> {
                item.setProcessInspectionId(processInspecion.getId());
            });
        }
        saveOrUpdateBatch(bizProcessInspectionPictureService, processInspecion.getProcessInspectionPictureList() == null || processInspecion.getProcessInspectionPictureList().isEmpty() ? new ArrayList<>() : processInspecion.getProcessInspectionPictureList(), oldProcessInspectionList);
        //保存咨询信息

        List<BizDayworkItemConsult> consults = processInspecion.getDayworkItemConsults().stream().filter(item -> item.getId() == null).collect(Collectors.toList());
        List<BizDayworkItemConsult> needAdd = new ArrayList<>();
        if (consults != null && consults.size() > 0) {
            for (BizDayworkItemConsult consult : consults) {
                BizDayworkItemConsult consultInfo = new BizDayworkItemConsult();
                consultInfo.setPictures(consult.getPictures());
                consultInfo.setTenantId(processInspecion.getTenantId());
                consultInfo.setUserId(SecurityUtils.getLoginUser().getUserId());
                consultInfo.setNickName(SecurityUtils.getLoginUser().getUser().getNickName());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setContent(consult.getContent());
                consultInfo.setConsultDepartment(consult.getConsultDepartment());
                consultInfo.setProcessInspectionId(processInspecion.getId());
                consultInfo.setProductionPlanId(daywork.getProductionPlanId());
                consultInfo.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                consultInfo.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                consultInfo.setLotId(processInspecion.getLot().getId());
                consultInfo.setLotCode(processInspecion.getLot().getLotCode());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setDayworkItemId(dayworkItem.getId());
                consultInfo.setProductId(processInspecion.getLot().getProductId());
                consultInfo.setProductDescription(processInspecion.getLot().getProductDescription());
                consultInfo.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                consultInfo.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                consultInfo.setProcessId(dayworkItem.getProcessId());
                consultInfo.setIsProcessInspection(1);
                consultInfo.setProcessAlias(processInspecion.getLot().getProcessAlias());
                //consultInfo.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
                consultInfo.setTechnicianId(consult.getTechnicianId());
                needAdd.add(consultInfo);
            }
            bizDayworkItemConsultService.saveBatch(needAdd);
            List<BizDayworkItemConsultPicture> pictures = new ArrayList<>();
            needAdd.forEach(ne -> {
                ne.getPictures().forEach(info -> {
                    BizDayworkItemConsultPicture consultPicture = new BizDayworkItemConsultPicture();
                    consultPicture.setDayworkItemConsultId(ne.getId());
                    consultPicture.setPictureName(info.getPictureName());
                    consultPicture.setTenantId(processInspecion.getTenantId());
                    consultPicture.setUrl(info.getUrl());
                    pictures.add(consultPicture);
                });
            });
            bizDayworkItemConsultPictureService.saveBatch(pictures);
        }

        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        dayworkItem.setTenantId(processInspecion.getTenantId());
        if(dayworkItem.getDeleted() == 1) {
            return success();
        }
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem));
    }

    //查找该检查是否已经移交给了仪器室
    @PostMapping("/getInstrumentRoomInspection")
    @Log(title = "判断检查是否交给仪器室", businessType = BusinessType.SELECT)
    public AjaxResult getInstrumentRoomInspection(@RequestBody BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {
        List<BizProcessInspecion> list = bizProcessInspecionService.query().eq("from_id", bizProcessInspecion.getId()).eq("type", "instrumentRoomInspection").list();
        return success(list);
    }


//    /**
//     * 根据箱号查询是否为一箱多批
//     */
    //原代码备份
//    @PostMapping("/getCarrierInfo")
//    public AjaxResult getCarrierInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
//        List<BizDayworkCarrier> list = bizDayworkCarrierService.getList(dayworkCarrier);
//        if (list.size() == 0) {
//            throw new RuntimeException("该箱号未绑定任何批次");
//        }
//
//        return success(list);
//    }

    /**
     * 根据箱号查询是否为检查载具，或一箱多批
     */
//    @PostMapping("/getCarrierInfo")
    @PostMapping("/getCarrierInfoProcess")
    @Log(title = "判断箱号是否为检查载具", businessType = BusinessType.SELECT)
    public AjaxResult getCarrierInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
//        List<BizDayworkCarrier> list = bizDayworkCarrierService.query().eq("carrier_code",dayworkCarrier.getCarrierCode()).eq("deleted",0).eq("is_changed",0).list();
        dayworkCarrier.setIsChanged(0);
        //先查询到箱号信息
        BizCarrier carrier = bizCarrierService.query().eq("code", dayworkCarrier.getCarrierCode()).eq("deleted", 0).one();
        if (carrier == null) {
            return error("未找到该箱号");
        }
        //查询箱号是否废弃
        BizCarrierReject carrierReject = bizCarrierRejectService.query().eq("carrier_id", carrier.getId()).eq("deleted", 0).one();
        if (carrierReject != null) {
            return error("该载具已废弃");
        }

        //根据箱号查询绑定信息，如果是检查载具并且没有绑定信息返回载具信息，如果是检查载具且有绑定信息，返回警告
        List<BizDayworkCarrier> list = bizDayworkCarrierService.getList(dayworkCarrier);
        if (carrier.getIsInspection().equals(1) && list.size() == 0) {
            return success(carrier);
        } else if (carrier.getIsInspection().equals(1) && list.size() > 0) {
            return error("检查载具:" + carrier.getCode() + "已经绑定其他批次");
        }
        if (list.size() == 0) {
            throw new RuntimeException("该箱号未绑定任何批次");
        }
        return success(list);
    }

    @PostMapping("/getCarrierInfo")
    @Log(title = "获取载具信息", businessType = BusinessType.SELECT)
    public AjaxResult getCarrierInfoByType(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        // 判断是哪种箱
        String carrierCode = dayworkCarrier.getCarrierCode();
        List<BizCarrier> carrierList = bizCarrierService.query().eq("code", carrierCode).list();

        if (carrierList.isEmpty()) {
            return error("未找到该箱号");
        }
        BizCarrier carrier = carrierList.get(0);
        //查询箱号是否废弃
        List<BizCarrierReject> carrierReject = bizCarrierRejectService.query().eq("carrier_id", carrier.getId()).eq("deleted", 0).orderByDesc("create_time").list();
        if (!carrierReject.isEmpty() && carrierReject.get(0).getIsAbandoned() == 1) {
            return error("该载具已废弃");
        }

        Map<String, Object> map = new HashMap<>();
        dayworkCarrier.setIsChanged(0);
        List<BizDayworkCarrier> list = bizDayworkCarrierService.getInfo(dayworkCarrier);
        //List<BizDayworkCarrier> list = bizDayworkCarrierService.query().eq("carrier_code", dayworkCarrier.getCarrierCode()).eq("is_changed",0).list();
        List<BizDaywork> dayworks = bizDayworkService.query().in("id", list.isEmpty() ? Collections.singletonList(0L) : list.stream().map(BizDayworkCarrier::getDayworkId).collect(Collectors.toList())).list();
        if (carrier.getIsInspection().equals(1)) {
            // 为检查载具返回检查载具信息
            if (carrier.getIsInspection().equals(1) && list.size() == 0) {
                map.put("inspectionCarrier", carrier);
                return success(map);
            } else if (carrier.getIsInspection().equals(1) && list.size() > 0) {
                // 需要显示已经绑定的批次。
                return error("检查载具:" + carrier.getCode() + "已经绑定其他批次" + dayworks.stream().map(BizDaywork::getLotCode).collect(Collectors.joining()));
            }
        } else {
            if (list.size() == 0) {
                throw new RuntimeException("该箱号未绑定任何批次。");
            }
            map.put("dayworkCarriers", list);
            return success(map);
        }
        return success();
    }

    //首件扫码专用
    @PostMapping("/getFirstCarrierInfo")
    @Log(title = "为首序检获取载具信息", businessType = BusinessType.SELECT)
    public AjaxResult getFirstCarrierInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        // 判断是哪种箱
        String carrierCode = dayworkCarrier.getCarrierCode();
        BizCarrier carrier = bizCarrierService.query().eq("code", carrierCode).one();
        if (carrier == null) {
            return error("未找到该箱号");
        }
        //查询箱号是否废弃
        BizCarrierReject carrierReject = bizCarrierRejectService.query().eq("carrier_id", carrier.getId()).eq("deleted", 0).eq("is_abandoned", 1).last("limit 1").one();
        if (carrierReject != null) {
            return error("该载具已废弃");
        }

        Map<String, Object> map = new HashMap<>();
        dayworkCarrier.setIsChanged(0);
//        List<BizDayworkCarrier> list = bizDayworkCarrierService.getList(dayworkCarrier);
        List<BizDayworkCarrier> list = bizDayworkCarrierService.query().eq("carrier_code", dayworkCarrier.getCarrierCode()).eq("is_changed", 0).list();
        List<BizDaywork> dayworks = bizDayworkService.query().in("id", list.isEmpty() ? Collections.singletonList(0L) : list.stream().map(BizDayworkCarrier::getDayworkId).collect(Collectors.toList())).list();
        if (carrier.getIsInspection().equals(1)) {
            // 为检查载具返回检查载具信息
            if (carrier.getIsInspection().equals(1) && list.size() == 0) {
                map.put("inspectionCarrier", carrier);
                return success(map);
            } else if (carrier.getIsInspection().equals(1) && list.size() > 0) {
                // 需要显示已经绑定的批次。
                return error("检查载具:" + carrier.getCode() + "已经绑定其他批次" + dayworks.stream().map(BizDaywork::getLotCode).collect(Collectors.joining()));
            }
        } else {
            if (list.size() == 0) {
                throw new RuntimeException("请扫检测载具。");
            }
            map.put("dayworkCarriers", list);
            return success(map);
        }
        return success();
    }

    //交检
    @Transactional
    @PostMapping("/saveDeliveryInspection")
    @Log(title = "新增交检信息", businessType = BusinessType.INSERT)
    public AjaxResult saveDeliveryInspection(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        if (processInspecion.getFlag() != null && processInspecion.getFlag() && (processInspecion.getIsInspectionCarrierChanged() != null && processInspecion.getIsInspectionCarrierChanged().equals(1))) {
            throw new RuntimeException("检测载具已解绑，移交仪器室，请绑定载具");
        }
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());
        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getLot().getDayworkItemId());
//        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        processInspecion.setUserId(processInspecion.getLot().getUserId());
        processInspecion.setNickName(processInspecion.getLot().getNickName());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProcessStepNumber(processInspecion.getLot().getProcessStepNumber());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setProductDescription(processInspecion.getLot().getProductDescription());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(processInspecion.getDeptId() != null && !processInspecion.getDeptId().equals(0L) ? processInspecion.getDeptId() : daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        processInspecion.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
        processInspecion.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        processInspecion.setEquipmentDetailCode(processInspecion.getLot().getEquipmentDetailCode());
        processInspecion.setEquipmentDetailId(processInspecion.getLot().getEquipmentDetailId());
        processInspecion.setReviewerId(SecurityUtils.getLoginUser().getUserId());
        processInspecion.setType("deliveryInspection");
        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }
        if (processInspecion.getInspectionStatus() != 0) {
            processInspecion.setEndTime(new Date());
            processInspecion.setFirstUpdaterId(processInspecion.getFirstUpdaterId() == null || processInspecion.getFirstUpdaterId().equals(0L) ? SecurityUtils.getLoginUser().getUserId() : processInspecion.getFirstUpdaterId());
        }
//        if (!checkDayworkItem(processInspecion.getLot().getDayworkItemId())) {
//            return error("编辑失败，下序已经生产加工");
//        }

        //BizCarrier carrier = bizCarrierService.query().eq("id", processInspecion.getInspectionCarrierId()).one();
        List<BizDayworkCarrier> dayworkCarriers = bizDayworkCarrierService.query().eq("carrier_id", processInspecion.getInspectionCarrierId()).eq("is_changed", 0).list();
        if (dayworkCarriers.stream().anyMatch(v -> !v.getDayworkId().equals(processInspecion.getDayworkId()))) {
            return error("检测载具已绑定其他批次");
        }
        Boolean isSave = false;
        if (processInspecion.getId() == null) {
            isSave = true;
        }
        //保存
        bizProcessInspecionService.saveOrUpdate(processInspecion);
        if (isSave) {
            if (dayworkCarriers.stream().noneMatch(v -> v.getDayworkId().equals(processInspecion.getDayworkId())) && (processInspecion.getIsInspectionCarrierChanged() == null || processInspecion.getIsInspectionCarrierChanged().equals(0))) {
                // 该检测载具需要绑定到当前批次
                BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
                newDayworkCarrier.setDayworkId(processInspecion.getDayworkId());
                newDayworkCarrier.setCarrierCode(processInspecion.getInspectionCarrierCode());
                newDayworkCarrier.setCarrierId(processInspecion.getInspectionCarrierId());
                newDayworkCarrier.setLotCode(processInspecion.getLotCode());
                newDayworkCarrier.setLotId(processInspecion.getLotId());
                newDayworkCarrier.setTenantId(processInspecion.getTenantId());
                newDayworkCarrier.setProductionPlanDetailId(processInspecion.getProductionPlanDetailId());
                newDayworkCarrier.setProcessInspectionId(processInspecion.getId());
                bizDayworkCarrierService.save(newDayworkCarrier);
            }
        }
        //移交仪器室
        if (processInspecion.getFlag() != null && processInspecion.getFlag()) {
            BizProcessInspecion inspection = new BizProcessInspecion();
            inspection.setUserId(processInspecion.getLot().getUserId());
            inspection.setTenantId(processInspecion.getTenantId());
            inspection.setNickName(processInspecion.getLot().getNickName());
            inspection.setCarrierId(processInspecion.getCarrierId());
            inspection.setCarrierCode(processInspecion.getCarrierCode());
            inspection.setDayworkId(daywork.getId());
            inspection.setProcessStepNumber(processInspecion.getLot().getProcessStepNumber());
            inspection.setProductionPlanId(daywork.getProductionPlanId());
            inspection.setDayworkItemId(dayworkItem.getId());
            inspection.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
            inspection.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
            inspection.setProductDescription(processInspecion.getLot().getProductDescription());
            inspection.setLotId(processInspecion.getLot().getId());
            inspection.setLotCode(processInspecion.getLot().getLotCode());
            inspection.setDeptId(processInspecion.getDeptId() != null && !processInspecion.getDeptId().equals(0L) ? processInspecion.getDeptId() : daywork.getDeptId());
            inspection.setProductId(processInspecion.getLot().getProductId());
            inspection.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
            inspection.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
            inspection.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
            inspection.setProcessStepNumber(processInspecion.getProcessStepNumber());
            inspection.setEquipmentDetailCode(processInspecion.getLot().getEquipmentDetailCode());
            inspection.setEquipmentDetailId(processInspecion.getLot().getEquipmentDetailId());
            inspection.setInspectionChamberId(processInspecion.getInspectionChamberId());
            //inspection.setReviewerId(SecurityUtils.getLoginUser().getUserId());
            inspection.setProcessStepNumber(processInspecion.getProcessStepNumber());
            inspection.setFromId(processInspecion.getId());
            inspection.setType("instrumentRoomInspection");
            if (inspection.getStartTime() == null) {
                inspection.setStartTime(new Date());
            }
            bizProcessInspecionService.save(inspection);
//            //保存检查明细
//            List<BizInspectionStandardsCheck> inspectionStandardsCheckList = bizInspectionStandardsCheckService.query()
//                    .eq("inspection_code", "instrumentRoomInspection")
//                    .eq("product_id", inspection.getProductId())
//                    .eq("process_id", processInspecion.getLot().getProcessId()).list();
//            List<BizProcessInspectionDetail> detailList = new ArrayList<>();
//            inspectionStandardsCheckList.forEach(inspectionStandardsCheck -> {
//                BizProcessInspectionDetail detail = new BizProcessInspectionDetail();
//                detail.setId(null);
//                detail.setUserId(processInspecion.getLot().getUserId());
//                detail.setProcessInspectionId(inspection.getId());
//                detail.setDayworkId(0L);
//                detail.setProductionPlanId(daywork.getProductionPlanId());
//                detail.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
//                detail.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
//                detail.setLotId(processInspecion.getLot().getId());
//                detail.setDayworkItemId(dayworkItem.getId());
//                detail.setCheckStandard(inspectionStandardsCheck.getStandard());
//                detail.setInspectionStandardsId(inspectionStandardsCheck.getId());
//                detailList.add(detail);
//            });
//            if(!detailList.isEmpty()) {
//                processInspectionDetailService.saveBatch(detailList);
//            }
            //绑定检测载具
            BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
            newDayworkCarrier.setDayworkId(inspection.getDayworkId());
            newDayworkCarrier.setCarrierCode(inspection.getInspectionCarrierCode());
            newDayworkCarrier.setCarrierId(inspection.getInspectionCarrierId());
            newDayworkCarrier.setTenantId(inspection.getTenantId());
            newDayworkCarrier.setLotCode(inspection.getLotCode());
            newDayworkCarrier.setLotId(inspection.getLotId());
            newDayworkCarrier.setProductionPlanDetailId(inspection.getProductionPlanDetailId());
            newDayworkCarrier.setProcessInspectionId(inspection.getId());
            bizDayworkCarrierService.save(newDayworkCarrier);
        }
        // 判断假设已经全项检查
        if (processInspecion.getIsAllChecked() != null && processInspecion.getIsAllChecked().equals(1)) {
            // 解绑检测载具
            // 查询所有检测载具
            List<BizDayworkCarrier> bizDayworkCarriers = bizDayworkCarrierService.query().eq("daywork_id", processInspecion.getDayworkId()).eq("process_inspection_id", processInspecion.getId()).eq("is_changed", 0).list();
            bizDayworkCarriers.forEach(l -> {
                l.setIsChanged(1);
            });
            bizDayworkCarrierService.updateBatchById(bizDayworkCarriers);
            processInspecion.setIsInspectionCarrierChanged(1);
            bizProcessInspecionService.updateById(processInspecion);
        }
        //保存废品信息
        List<BizProcessInspectionDetail> rejects = processInspecion.getProcessInspectionDetails();
        //保存前先删除原先废品信息
        List<BizProcessInspectionDetail> processInspectionDetailList = processInspectionDetailService.query().eq("process_inspection_id", processInspecion.getId()).list();
        processInspectionDetailService.removeBatchByIds(processInspectionDetailList);

        if (rejects != null && rejects.size() > 0) {
            for (BizProcessInspectionDetail reject : rejects) {
                reject.setId(null);
                reject.setUserId(processInspecion.getLot().getUserId());
                reject.setProcessInspectionId(processInspecion.getId());
                reject.setDayworkId(0L);
                reject.setTenantId(processInspecion.getTenantId());
                reject.setProductionPlanId(daywork.getProductionPlanId());
                reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                reject.setLotId(processInspecion.getLot().getId());
                reject.setDayworkItemId(dayworkItem.getId());
            }
        }
        processInspectionDetailService.saveBatch(rejects);
        //保存上传图片信息
        List<BizProcessInspectionPicture> oldProcessInspectionList = bizProcessInspectionPictureService.query().eq("process_inspection_id", processInspecion.getId()).list();
        if (processInspecion.getProcessInspectionPictureList() != null && !processInspecion.getProcessInspectionPictureList().isEmpty()) {
            processInspecion.getProcessInspectionPictureList().forEach(item -> {
                item.setProcessInspectionId(processInspecion.getId());
            });
        }
        saveOrUpdateBatch(bizProcessInspectionPictureService, processInspecion.getProcessInspectionPictureList() == null || processInspecion.getProcessInspectionPictureList().isEmpty() ? new ArrayList<>() : processInspecion.getProcessInspectionPictureList(), oldProcessInspectionList);
        //保存咨询信息

        List<BizDayworkItemConsult> consults = processInspecion.getDayworkItemConsults().stream().filter(item -> item.getId() == null).collect(Collectors.toList());
        List<BizDayworkItemConsult> needAdd = new ArrayList<>();
        if (consults != null && consults.size() > 0) {
            for (BizDayworkItemConsult consult : consults) {
                BizDayworkItemConsult consultInfo = new BizDayworkItemConsult();
                consultInfo.setPictures(consult.getPictures());
                consultInfo.setTenantId(processInspecion.getTenantId());
                consultInfo.setUserId(SecurityUtils.getLoginUser().getUserId());
                consultInfo.setNickName(SecurityUtils.getLoginUser().getUser().getNickName());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setContent(consult.getContent());
                consultInfo.setConsultDepartment(consult.getConsultDepartment());
                consultInfo.setProcessInspectionId(processInspecion.getId());
                consultInfo.setProductionPlanId(daywork.getProductionPlanId());
                consultInfo.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                consultInfo.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                consultInfo.setLotId(processInspecion.getLot().getId());
                consultInfo.setLotCode(processInspecion.getLot().getLotCode());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setDayworkItemId(dayworkItem.getId());
                consultInfo.setProductId(processInspecion.getLot().getProductId());
                consultInfo.setProductDescription(processInspecion.getLot().getProductDescription());
                consultInfo.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                consultInfo.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                consultInfo.setProcessId(dayworkItem.getProcessId());
                consultInfo.setIsProcessInspection(1);
                consultInfo.setProcessAlias(processInspecion.getLot().getProcessAlias());
                //consultInfo.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
                consultInfo.setTechnicianId(consult.getTechnicianId());
                needAdd.add(consultInfo);
            }
            bizDayworkItemConsultService.saveBatch(needAdd);
            List<BizDayworkItemConsultPicture> pictures = new ArrayList<>();
            needAdd.forEach(ne -> {
                ne.getPictures().forEach(info -> {
                    BizDayworkItemConsultPicture consultPicture = new BizDayworkItemConsultPicture();
                    consultPicture.setDayworkItemConsultId(ne.getId());
                    consultPicture.setPictureName(info.getPictureName());
                    consultPicture.setTenantId(processInspecion.getTenantId());
                    consultPicture.setUrl(info.getUrl());
                    pictures.add(consultPicture);
                });
            });
            bizDayworkItemConsultPictureService.saveBatch(pictures);
        }

        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        dayworkItem.setTenantId(processInspecion.getTenantId());
        if(dayworkItem.getDeleted() == 1) {
            return success();
        }
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem));
    }

    //仪器室检转到其他仪器室
    @Transactional
    @PostMapping("/updateChamberId")
    @Log(title = "转移仪器室", businessType = BusinessType.UPDATE)
    public AjaxResult updateChamberId(@RequestBody BizProcessInspecion bizProcessInspecion) {
        BizProcessInspecion processInspection = bizProcessInspecionService.query().eq("id", bizProcessInspecion.getId()).one();
        processInspection.setInspectionChamberId(bizProcessInspecion.getInspectionChamberId());
        bizProcessInspecionService.updateById(processInspection);
        //同时修改主检查的仪器室id
        if (processInspection.getFromId() != 0L) {
            BizProcessInspecion fromInspection = bizProcessInspecionService.query().eq("id", bizProcessInspecion.getFromId()).one();
            fromInspection.setInspectionChamberId(bizProcessInspecion.getInspectionChamberId());
            bizProcessInspecionService.updateById(fromInspection);
        }
        return success();
    }

    //仪器室检
    @Transactional
    @PostMapping("/saveInstrumentRoomInspection")
    @Log(title = "新增仪器室检查", businessType = BusinessType.INSERT)
    public AjaxResult saveInstrumentRoomInspection(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());
//        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getLot().getDayworkItemId());
        processInspecion.setUserId(processInspecion.getLot().getUserId());
        processInspecion.setNickName(processInspecion.getLot().getNickName());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProcessStepNumber(processInspecion.getLot().getProcessStepNumber());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setProductDescription(processInspecion.getLot().getProductDescription());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(processInspecion.getDeptId() != null && !processInspecion.getDeptId().equals(0L) ? processInspecion.getDeptId() : daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        processInspecion.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
        processInspecion.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        processInspecion.setEquipmentDetailCode(processInspecion.getLot().getEquipmentDetailCode());
        processInspecion.setEquipmentDetailId(processInspecion.getLot().getEquipmentDetailId());
        processInspecion.setReviewerId(SecurityUtils.getLoginUser().getUserId());
        processInspecion.setType("instrumentRoomInspection");
        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }
        if (processInspecion.getInspectionStatus() != 0) {
            processInspecion.setEndTime(new Date());
            processInspecion.setFirstUpdaterId(processInspecion.getFirstUpdaterId() == null || processInspecion.getFirstUpdaterId().equals(0L) ? SecurityUtils.getLoginUser().getUserId() : processInspecion.getFirstUpdaterId());
        }
        // 20241015 取消下序生产时不能编辑限制
//        if (!checkDayworkItem(processInspecion.getLot().getDayworkItemId())) {
//            return error("编辑失败，下序已经生产加工");
//        }

//        BizCarrier carrier = bizCarrierService.query().eq("id", processInspecion.getInspectionCarrierId()).one();
//        List<BizDayworkCarrier> dayworkCarriers = bizDayworkCarrierService.query().eq("carrier_id", processInspecion.getInspectionCarrierId()).eq("is_changed", 0).list();
//        if (dayworkCarriers.stream().anyMatch(v -> !v.getDayworkId().equals(processInspecion.getDayworkId()))) {
//            return error("检测载具已绑定其他批次");
//        }
        //保存
        bizProcessInspecionService.saveOrUpdate(processInspecion);

//            if (dayworkCarriers.stream().noneMatch(v -> v.getDayworkId().equals(processInspecion.getDayworkId())) && (processInspecion.getIsInspectionCarrierChanged() == null || processInspecion.getIsInspectionCarrierChanged().equals(0))) {
//                // 该检测载具需要绑定到当前批次
//                BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
//                newDayworkCarrier.setDayworkId(processInspecion.getDayworkId());
//                newDayworkCarrier.setCarrierCode(processInspecion.getInspectionCarrierCode());
//                newDayworkCarrier.setCarrierId(processInspecion.getInspectionCarrierId());
//                newDayworkCarrier.setLotCode(processInspecion.getLotCode());
//                newDayworkCarrier.setLotId(processInspecion.getLotId());
//                newDayworkCarrier.setProductionPlanDetailId(processInspecion.getProductionPlanDetailId());
//                newDayworkCarrier.setProcessInspectionId(processInspecion.getId());
//                bizDayworkCarrierService.save(newDayworkCarrier);
//            }
        //保存废品信息
        List<BizProcessInspectionDetail> rejects = processInspecion.getProcessInspectionDetails();
        //保存前先删除原先废品信息
        List<BizProcessInspectionDetail> processInspectionDetailList = processInspectionDetailService.query().eq("process_inspection_id", processInspecion.getId()).list();
        processInspectionDetailService.removeBatchByIds(processInspectionDetailList);

        if (rejects != null && rejects.size() > 0) {
            for (BizProcessInspectionDetail reject : rejects) {
                reject.setId(null);
                reject.setUserId(processInspecion.getLot().getUserId());
                reject.setProcessInspectionId(processInspecion.getId());
                reject.setDayworkId(0L);
                reject.setTenantId(processInspecion.getTenantId());
                reject.setProductionPlanId(daywork.getProductionPlanId());
                reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                reject.setLotId(processInspecion.getLot().getId());
                reject.setDayworkItemId(dayworkItem.getId());
            }
        }
        processInspectionDetailService.saveBatch(rejects);
        //保存上传图片信息
        List<BizProcessInspectionPicture> oldProcessInspectionList = bizProcessInspectionPictureService.query().eq("process_inspection_id", processInspecion.getId()).list();
        if (processInspecion.getProcessInspectionPictureList() != null && !processInspecion.getProcessInspectionPictureList().isEmpty()) {
            processInspecion.getProcessInspectionPictureList().forEach(item -> {
                item.setProcessInspectionId(processInspecion.getId());
            });
        }
        saveOrUpdateBatch(bizProcessInspectionPictureService, processInspecion.getProcessInspectionPictureList() == null || processInspecion.getProcessInspectionPictureList().isEmpty() ? new ArrayList<>() : processInspecion.getProcessInspectionPictureList(), oldProcessInspectionList);
        //保存咨询信息

        List<BizDayworkItemConsult> consults = processInspecion.getDayworkItemConsults().stream().filter(item -> item.getId() == null).collect(Collectors.toList());
        List<BizDayworkItemConsult> needAdd = new ArrayList<>();
        if (consults != null && consults.size() > 0) {
            for (BizDayworkItemConsult consult : consults) {
                BizDayworkItemConsult consultInfo = new BizDayworkItemConsult();
                consultInfo.setPictures(consult.getPictures());
                consultInfo.setTenantId(processInspecion.getTenantId());
                consultInfo.setUserId(SecurityUtils.getLoginUser().getUserId());
                consultInfo.setNickName(SecurityUtils.getLoginUser().getUser().getNickName());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setContent(consult.getContent());
                consultInfo.setConsultDepartment(consult.getConsultDepartment());
                consultInfo.setProcessInspectionId(processInspecion.getId());
                consultInfo.setProductionPlanId(daywork.getProductionPlanId());
                consultInfo.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                consultInfo.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                consultInfo.setLotId(processInspecion.getLot().getId());
                consultInfo.setLotCode(processInspecion.getLot().getLotCode());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setDayworkItemId(dayworkItem.getId());
                consultInfo.setProductId(processInspecion.getLot().getProductId());
                consultInfo.setProductDescription(processInspecion.getLot().getProductDescription());
                consultInfo.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                consultInfo.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                consultInfo.setProcessId(dayworkItem.getProcessId());
                consultInfo.setIsProcessInspection(1);
                consultInfo.setProcessAlias(processInspecion.getLot().getProcessAlias());
                //consultInfo.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
                consultInfo.setTechnicianId(consult.getTechnicianId());
                needAdd.add(consultInfo);
            }
            bizDayworkItemConsultService.saveBatch(needAdd);
            List<BizDayworkItemConsultPicture> pictures = new ArrayList<>();
            needAdd.forEach(ne -> {
                ne.getPictures().forEach(info -> {
                    BizDayworkItemConsultPicture consultPicture = new BizDayworkItemConsultPicture();
                    consultPicture.setDayworkItemConsultId(ne.getId());
                    consultPicture.setTenantId(processInspecion.getTenantId());
                    consultPicture.setPictureName(info.getPictureName());
                    consultPicture.setUrl(info.getUrl());
                    pictures.add(consultPicture);
                });
            });
            bizDayworkItemConsultPictureService.saveBatch(pictures);
        }

        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        dayworkItem.setTenantId(processInspecion.getTenantId());
        if(dayworkItem.getDeleted() == 1) {
            return success();
        }
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem));
    }

    //该方法只在首件送检时选择仪器室时使用
    @Transactional
    @PostMapping("/saveFirstInstrumentRoomInspection")
    @Log(title = "新增首序送检仪器室", businessType = BusinessType.INSERT)
    public AjaxResult saveFirstInstrumentRoomInspection(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        if (processInspecion.getFlag() != null && processInspecion.getFlag() && (processInspecion.getIsInspectionCarrierChanged() != null && processInspecion.getIsInspectionCarrierChanged().equals(1))) {
            throw new RuntimeException("检测载具已解绑，移交仪器室，请绑定载具");
        }
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());
        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getLot().getDayworkItemId());
//        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        processInspecion.setUserId(processInspecion.getLot().getUserId());
        processInspecion.setNickName(processInspecion.getLot().getNickName());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProcessStepNumber(processInspecion.getLot().getProcessStepNumber());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setProductDescription(processInspecion.getLot().getProductDescription());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(processInspecion.getDeptId() != null && !processInspecion.getDeptId().equals(0L) ? processInspecion.getDeptId() : daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        processInspecion.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
        processInspecion.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        processInspecion.setEquipmentDetailCode(processInspecion.getLot().getEquipmentDetailCode());
        processInspecion.setEquipmentDetailId(processInspecion.getLot().getEquipmentDetailId());
        processInspecion.setType("instrumentRoomInspection");
        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }
        if (processInspecion.getInspectionStatus() != 0) {
            processInspecion.setEndTime(new Date());
            processInspecion.setFirstUpdaterId(processInspecion.getFirstUpdaterId() == null || processInspecion.getFirstUpdaterId().equals(0L) ? SecurityUtils.getLoginUser().getUserId() : processInspecion.getFirstUpdaterId());
        }

        // 20241015 取消下序生产时不能编辑限制
//        if (!checkDayworkItem(processInspecion.getLot().getDayworkItemId())) {
//            return error("编辑失败，下序已经生产加工");
//        }
        BizCarrier carrier = bizCarrierService.query().eq("id", processInspecion.getInspectionCarrierId()).one();
        List<BizDayworkCarrier> dayworkCarriers = bizDayworkCarrierService.query().eq("carrier_id", processInspecion.getInspectionCarrierId()).eq("is_changed", 0).list();
        if (dayworkCarriers.stream().anyMatch(v -> !v.getDayworkId().equals(processInspecion.getDayworkId()))) {
            return error("检测载具已绑定其他批次");
        }
        //保存
        bizProcessInspecionService.saveOrUpdate(processInspecion);

        if (dayworkCarriers.stream().noneMatch(v -> v.getDayworkId().equals(processInspecion.getDayworkId())) && (processInspecion.getIsInspectionCarrierChanged() == null || processInspecion.getIsInspectionCarrierChanged().equals(0))) {
            // 该检测载具需要绑定到当前批次
            BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
            newDayworkCarrier.setDayworkId(processInspecion.getDayworkId());
            newDayworkCarrier.setCarrierCode(processInspecion.getInspectionCarrierCode());
            newDayworkCarrier.setCarrierId(processInspecion.getInspectionCarrierId());
            newDayworkCarrier.setTenantId(processInspecion.getTenantId());
            newDayworkCarrier.setLotCode(processInspecion.getLotCode());
            newDayworkCarrier.setLotId(processInspecion.getLotId());
            newDayworkCarrier.setProductionPlanDetailId(processInspecion.getProductionPlanDetailId());
            newDayworkCarrier.setProcessInspectionId(processInspecion.getId());
            bizDayworkCarrierService.save(newDayworkCarrier);
        }
        //保存废品信息
        List<BizProcessInspectionDetail> rejects = processInspecion.getProcessInspectionDetails();
        if (rejects != null && rejects.size() > 0) {
            for (BizProcessInspectionDetail reject : rejects) {
                reject.setId(null);
                reject.setUserId(processInspecion.getLot().getUserId());
                reject.setProcessInspectionId(processInspecion.getId());
                reject.setDayworkId(0L);
                reject.setTenantId(processInspecion.getTenantId());
                reject.setProductionPlanId(daywork.getProductionPlanId());
                reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                reject.setLotId(processInspecion.getLot().getId());
                reject.setDayworkItemId(dayworkItem.getId());
            }
            processInspectionDetailService.saveBatch(rejects);
        } else {
            //如果没选择检查标准，则自动带入所有的检查
            List<BizInspectionStandardsCheck> inspectionStandardsCheckList = bizInspectionStandardsCheckService.query()
                    .eq("inspection_code", "instrumentRoomInspection")
                    .eq("product_id", processInspecion.getProductId())
                    .eq("process_id", processInspecion.getLot().getProcessId()).list();
            List<BizProcessInspectionDetail> detailList = new ArrayList<>();
            inspectionStandardsCheckList.forEach(inspectionStandardsCheck -> {
                BizProcessInspectionDetail detail = new BizProcessInspectionDetail();
                detail.setId(null);
                detail.setTenantId(processInspecion.getTenantId());
                detail.setUserId(processInspecion.getLot().getUserId());
                detail.setProcessInspectionId(processInspecion.getId());
                detail.setDayworkId(0L);
                detail.setProductionPlanId(daywork.getProductionPlanId());
                detail.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                detail.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                detail.setLotId(processInspecion.getLot().getId());
                detail.setDayworkItemId(dayworkItem.getId());
                detail.setCheckStandard(inspectionStandardsCheck.getStandard());
                detail.setInspectionStandardsId(inspectionStandardsCheck.getId());
                detailList.add(detail);
            });
            processInspectionDetailService.saveBatch(detailList);
        }
        //保存上传图片信息
        List<BizProcessInspectionPicture> oldProcessInspectionList = bizProcessInspectionPictureService.query().eq("process_inspection_id", processInspecion.getId()).list();
        if (processInspecion.getProcessInspectionPictureList() != null && !processInspecion.getProcessInspectionPictureList().isEmpty()) {
            processInspecion.getProcessInspectionPictureList().forEach(item -> {
                item.setProcessInspectionId(processInspecion.getId());
            });
        }
        saveOrUpdateBatch(bizProcessInspectionPictureService, processInspecion.getProcessInspectionPictureList() == null || processInspecion.getProcessInspectionPictureList().isEmpty() ? new ArrayList<>() : processInspecion.getProcessInspectionPictureList(), oldProcessInspectionList);
        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        dayworkItem.setTenantId(processInspecion.getTenantId());
        if(dayworkItem.getDeleted() == 1) {
            return success();
        }
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem));
    }

    //出厂检
    @Transactional
    @PostMapping("/saveFactoryInspection")
    @Log(title = "出厂检人员新增出厂检", businessType = BusinessType.INSERT)
    public AjaxResult saveFactoryInspection(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        if (processInspecion.getFlag() != null && processInspecion.getFlag() && (processInspecion.getIsInspectionCarrierChanged() != null && processInspecion.getIsInspectionCarrierChanged().equals(1))) {
            throw new RuntimeException("检测载具已解绑，移交仪器室，请绑定载具");
        }
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());
        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getLot().getDayworkItemId());
//        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        processInspecion.setUserId(SecurityUtils.getLoginUser().getUser().getUserId());
        processInspecion.setNickName(SecurityUtils.getLoginUser().getUser().getNickName());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProcessStepNumber(processInspecion.getLot().getProcessStepNumber());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setProductDescription(processInspecion.getLot().getProductDescription());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        if (processInspecion.getId() == null) {
            processInspecion.setInspectionCarrierCode(processInspecion.getLot().getInspectionCarrierCode());
            processInspecion.setInspectionCarrierId(processInspecion.getLot().getInspectionCarrierId());
        } else {
            processInspecion.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
            processInspecion.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        }
//        if (processInspecion.getId() == null) {
//            processInspecion.setStatus(0);
//        }
        //出厂检创建时不添加检察员id
        if (processInspecion.getId() != null) {
            processInspecion.setReviewerId(SecurityUtils.getLoginUser().getUserId());
        } else {
            processInspecion.setReviewerId(processInspecion.getReviewerId());
        }
        processInspecion.setType("factoryInspection");
        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }
        if (processInspecion.getInspectionStatus() != 0) {
            processInspecion.setEndTime(new Date());
            processInspecion.setFirstUpdaterId(processInspecion.getFirstUpdaterId() == null || processInspecion.getFirstUpdaterId().equals(0L) ? SecurityUtils.getLoginUser().getUserId() : processInspecion.getFirstUpdaterId());
        }
        //BizCarrier carrier = bizCarrierService.query().eq("id", processInspecion.getInspectionCarrierId()).one();
        List<BizDayworkCarrier> dayworkCarriers = bizDayworkCarrierService.query().eq("carrier_id", processInspecion.getInspectionCarrierId()).eq("is_changed", 0).list();
        if (dayworkCarriers.stream().anyMatch(v -> !v.getDayworkId().equals(processInspecion.getDayworkId()))) {
            return error("检测载具已绑定其他批次");
        }
        Boolean isSave = false;
        if (processInspecion.getId() == null) {
            isSave = true;
        }
        bizProcessInspecionService.saveOrUpdate(processInspecion);
        //载具保存
        if (isSave) {
            if (dayworkCarriers.stream().noneMatch(v -> v.getDayworkId().equals(processInspecion.getDayworkId())) && (processInspecion.getIsInspectionCarrierChanged() == null || processInspecion.getIsInspectionCarrierChanged().equals(0))) {
                // 该检测载具需要绑定到当前批次
                BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
                newDayworkCarrier.setDayworkId(processInspecion.getDayworkId());
                newDayworkCarrier.setTenantId(processInspecion.getTenantId());
                newDayworkCarrier.setCarrierCode(processInspecion.getInspectionCarrierCode());
                newDayworkCarrier.setCarrierId(processInspecion.getInspectionCarrierId());
                newDayworkCarrier.setLotCode(processInspecion.getLotCode());
                newDayworkCarrier.setLotId(processInspecion.getLotId());
                newDayworkCarrier.setProductionPlanDetailId(processInspecion.getProductionPlanDetailId());
                newDayworkCarrier.setProcessInspectionId(processInspecion.getId());
                bizDayworkCarrierService.save(newDayworkCarrier);
            }
        }

        // 判断假设已经全项检查
        if (processInspecion.getIsAllChecked() != null && processInspecion.getIsAllChecked().equals(1)) {
            // 解绑检测载具
            // 查询所有检测载具
            List<BizDayworkCarrier> bizDayworkCarriers = bizDayworkCarrierService.query().eq("daywork_id", processInspecion.getDayworkId()).eq("process_inspection_id", processInspecion.getId()).eq("is_changed", 0).list();
            bizDayworkCarriers.forEach(l -> {
                l.setIsChanged(1);
            });
            bizDayworkCarrierService.updateBatchById(bizDayworkCarriers);
            processInspecion.setIsInspectionCarrierChanged(1);
            bizProcessInspecionService.updateById(processInspecion);
        }

        //保存废品信息
        List<BizProcessInspectionDetail> rejects = processInspecion.getProcessInspectionDetails();
        //保存前先删除原先废品信息
        List<BizProcessInspectionDetail> processInspectionDetailList = processInspectionDetailService.query().eq("process_inspection_id", processInspecion.getId()).list();
        processInspectionDetailService.removeBatchByIds(processInspectionDetailList);

        if (rejects != null && rejects.size() > 0) {
            for (BizProcessInspectionDetail reject : rejects) {
                reject.setId(null);
                reject.setUserId(processInspecion.getLot().getUserId());
                reject.setProcessInspectionId(processInspecion.getId());
                reject.setTenantId(processInspecion.getTenantId());
                reject.setDayworkId(0L);
                reject.setProductionPlanId(daywork.getProductionPlanId());
                reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                reject.setLotId(processInspecion.getLot().getId());
                reject.setDayworkItemId(dayworkItem.getId());
            }
        }
        processInspectionDetailService.saveBatch(rejects);
        //保存上传图片信息
        System.out.println("===================");
        System.out.println(processInspecion);
        List<BizProcessInspectionPicture> oldProcessInspectionList = bizProcessInspectionPictureService.query().eq("process_inspection_id", processInspecion.getId()).list();
        if (processInspecion.getProcessInspectionPictureList() != null && !processInspecion.getProcessInspectionPictureList().isEmpty()) {
            processInspecion.getProcessInspectionPictureList().forEach(item -> {
                item.setProcessInspectionId(processInspecion.getId());
            });
        }
        saveOrUpdateBatch(bizProcessInspectionPictureService, processInspecion.getProcessInspectionPictureList() == null || processInspecion.getProcessInspectionPictureList().isEmpty() ? new ArrayList<>() : processInspecion.getProcessInspectionPictureList(), oldProcessInspectionList);

        //保存咨询信息
        if (processInspecion.getDayworkItemConsults() != null) {
            List<BizDayworkItemConsult> consults = processInspecion.getDayworkItemConsults().stream().filter(item -> item.getId() == null).collect(Collectors.toList());
            List<BizDayworkItemConsult> needAdd = new ArrayList<>();
            if (consults != null && consults.size() > 0) {
                for (BizDayworkItemConsult consult : consults) {
                    BizDayworkItemConsult consultInfo = new BizDayworkItemConsult();
                    consultInfo.setPictures(consult.getPictures());
                    consultInfo.setTenantId(processInspecion.getTenantId());
                    consultInfo.setUserId(SecurityUtils.getLoginUser().getUserId());
                    consultInfo.setNickName(SecurityUtils.getLoginUser().getUser().getNickName());
                    consultInfo.setDayworkId(daywork.getId());
                    consultInfo.setContent(consult.getContent());
                    consultInfo.setConsultDepartment(consult.getConsultDepartment());
                    consultInfo.setProcessInspectionId(processInspecion.getId());
                    consultInfo.setProductionPlanId(daywork.getProductionPlanId());
                    consultInfo.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                    consultInfo.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                    consultInfo.setLotId(processInspecion.getLot().getId());
                    consultInfo.setLotCode(processInspecion.getLot().getLotCode());
                    consultInfo.setDayworkId(daywork.getId());
                    consultInfo.setDayworkItemId(dayworkItem.getId());
                    consultInfo.setProductId(processInspecion.getLot().getProductId());
                    consultInfo.setProductDescription(processInspecion.getLot().getProductDescription());
                    consultInfo.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                    consultInfo.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                    consultInfo.setProcessId(dayworkItem.getProcessId());
                    consultInfo.setIsProcessInspection(1);
                    consultInfo.setProcessAlias(processInspecion.getLot().getProcessAlias());
                    //consultInfo.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
                    consultInfo.setTechnicianId(consult.getTechnicianId());
                    needAdd.add(consultInfo);
                }
                bizDayworkItemConsultService.saveBatch(needAdd);
                List<BizDayworkItemConsultPicture> pictures = new ArrayList<>();
                needAdd.forEach(ne -> {
                    ne.getPictures().forEach(info -> {
                        BizDayworkItemConsultPicture consultPicture = new BizDayworkItemConsultPicture();
                        consultPicture.setDayworkItemConsultId(ne.getId());
                        consultPicture.setTenantId(processInspecion.getTenantId());
                        consultPicture.setPictureName(info.getPictureName());
                        consultPicture.setUrl(info.getUrl());
                        pictures.add(consultPicture);
                    });
                });
                bizDayworkItemConsultPictureService.saveBatch(pictures);
            }
        }
        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        dayworkItem.setTenantId(processInspecion.getTenantId());
        if(dayworkItem.getDeleted() == 1) {
            return success();
        }
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem));
    }
    //没有出厂检权限新增
    @Transactional
    @PostMapping("/saveFirstFactoryInspection")
    @Log(title = "普通员工新增出厂检", businessType = BusinessType.INSERT)
    public AjaxResult saveFirstFactoryInspection(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        if (processInspecion.getFlag() != null && processInspecion.getFlag() && (processInspecion.getIsInspectionCarrierChanged() != null && processInspecion.getIsInspectionCarrierChanged().equals(1))) {
            throw new RuntimeException("检测载具已解绑，移交仪器室，请绑定载具");
        }
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());
        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getLot().getDayworkItemId());
//        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        processInspecion.setUserId(SecurityUtils.getLoginUser().getUser().getUserId());
        processInspecion.setNickName(SecurityUtils.getLoginUser().getUser().getNickName());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProcessStepNumber(processInspecion.getLot().getProcessStepNumber());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setProductDescription(processInspecion.getLot().getProductDescription());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        if (processInspecion.getId() == null) {
            processInspecion.setInspectionCarrierCode(processInspecion.getLot().getInspectionCarrierCode());
            processInspecion.setInspectionCarrierId(processInspecion.getLot().getInspectionCarrierId());
        } else {
            processInspecion.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
            processInspecion.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        }
        if (processInspecion.getId() == null) {
            processInspecion.setStatus(0);
        }
        //出厂检创建时不添加检察员id
        if (processInspecion.getId() != null) {
            processInspecion.setReviewerId(SecurityUtils.getLoginUser().getUserId());
        } else {
            processInspecion.setReviewerId(processInspecion.getReviewerId());
        }
        processInspecion.setType("factoryInspection");
        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }
        //BizCarrier carrier = bizCarrierService.query().eq("id", processInspecion.getInspectionCarrierId()).one();
        List<BizDayworkCarrier> dayworkCarriers = bizDayworkCarrierService.query().eq("carrier_id", processInspecion.getInspectionCarrierId()).eq("is_changed", 0).list();
        if (dayworkCarriers.stream().anyMatch(v -> !v.getDayworkId().equals(processInspecion.getDayworkId()))) {
            return error("检测载具已绑定其他批次");
        }
        Boolean isSave = false;
        if (processInspecion.getId() == null) {
            isSave = true;
        }
        bizProcessInspecionService.saveOrUpdate(processInspecion);
        //载具保存
        if (isSave) {
            if (dayworkCarriers.stream().noneMatch(v -> v.getDayworkId().equals(processInspecion.getDayworkId())) && (processInspecion.getIsInspectionCarrierChanged() == null || processInspecion.getIsInspectionCarrierChanged().equals(0))) {
                // 该检测载具需要绑定到当前批次
                BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
                newDayworkCarrier.setDayworkId(processInspecion.getDayworkId());
                newDayworkCarrier.setTenantId(processInspecion.getTenantId());
                newDayworkCarrier.setCarrierCode(processInspecion.getInspectionCarrierCode());
                newDayworkCarrier.setCarrierId(processInspecion.getInspectionCarrierId());
                newDayworkCarrier.setLotCode(processInspecion.getLotCode());
                newDayworkCarrier.setLotId(processInspecion.getLotId());
                newDayworkCarrier.setProductionPlanDetailId(processInspecion.getProductionPlanDetailId());
                newDayworkCarrier.setProcessInspectionId(processInspecion.getId());
                bizDayworkCarrierService.save(newDayworkCarrier);
            }
        }
        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        dayworkItem.setTenantId(processInspecion.getTenantId());
        if(dayworkItem.getDeleted() == 1) {
            return success();
        }
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem));
    }

    /**
     * 根据箱号查询批次信息
     */
    @PostMapping("/getLotOutsourcedInfo")
    @Log(title = "根据箱号查询批次信息", businessType = BusinessType.SELECT)
    public AjaxResult getLotOutsourcedInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        List<String> codes = new ArrayList<>();
        codes.add("27110");
        codes.add("28030");
        codes.add("28080");
        codes.add("27070");
        List<String> processCodes = dayworkCarrier.getProcessCode() == null ? codes : dayworkCarrier.getProcessCode();
        System.out.println("查询工序编码：processCodes" + processCodes);
        //根据箱号查询当前绑定信息
        dayworkCarrier = bizDayworkCarrierService.getByCarrierCode(dayworkCarrier);
        if (dayworkCarrier == null) {
            throw new RuntimeException("该箱号未绑定任何批次");
        }
//        System.out.println("请求信息：dayworkCarrier" + dayworkCarrier);

        //根据dayworkId查询外协回收信息是否有该批次
        BizReturnReceiptDetail bizReturnReceiptDetail = new BizReturnReceiptDetail();
        bizReturnReceiptDetail.setDayworkId(dayworkCarrier.getDayworkId());
        List<BizReturnReceiptDetail> detailList = bizReturnReceiptDetailService.getList(bizReturnReceiptDetail);
        List<BizOutsourcedOrderDetailProcess> processes = bizOutsourcedOrderDetailProcessService.query().in("detail_id", detailList.isEmpty() ? Collections.singletonList(0L) : detailList.stream().map(BizReturnReceiptDetail::getOutsourceDetailId).collect(Collectors.toList())).list();

//        System.out.println("查询是否有该批次：detailList" + detailList);
        if (detailList.size() == 0) {
            throw new RuntimeException("该批次不是外协回收批次");
        }

        if (processes.stream().noneMatch(detail -> processCodes.stream().anyMatch(l -> l.trim().equals(detail.getProcessCode())))) {
            throw new RuntimeException("该批次不需要外协检查");
        }
//                        现在先不考虑 审核通过后才能外协检查限制先放到下一迭代 20240625
//        20240703 加回去
        if (detailList.stream().noneMatch(detail -> detail.getIsAudit().equals(1) && detail.getStatus().equals(1))) {
            throw new RuntimeException("该批次需要审核通过后才能外协检查");
        }
        detailList = detailList.stream()
                .filter(detail -> processes.stream().anyMatch(v -> v.getTechnologicalProcessDetailId().equals(detail.getTechnologicalProcessDetailId())))
                .filter(detail -> processCodes.stream().anyMatch(l -> l.trim().equals(detail.getProcessCode())) &&
                                detail.getIsAudit().equals(1)
//                        现在先不考虑 审核通过后才能外协检查限制先放到下一迭代 20240625
//                        20240703加回去
                                && detail.getStatus().equals(1)
                )
                .collect(Collectors.toList());
        //如果有多道工序，则根据工序的排序找到最后一条
        if (detailList.size() > 1) {
            //按照工序排序
            detailList.sort(Comparator.comparing(BizReturnReceiptDetail::getProcessStepNumber, Comparator.reverseOrder()));
        }
        //根据工序id查询报工信息
        BizDaywork daywork = new BizDaywork();
        daywork.setId(detailList.get(0).getDayworkId());
        daywork.setTechnologicalProcessDetailId(detailList.get(0).getTechnologicalProcessDetailId());

        List<BizDayworkItem> itemList = bizDayworkItemService.getByDaywork(daywork);
        if (itemList.size() == 0) {
            throw new RuntimeException("该批次未开始加工");
        }
        BizDayworkItem item = itemList.get(0);
        BizProductionPlanDetail productionPlanDetail = productionPlanDetailService.getById(item.getProductionPlanDetailId());
        BizProduct product = productService.getById(productionPlanDetail.getProductId());
        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
        if (user != null) {
            item.setTechnicianName(user.getNickName());
            item.setTechnicianId(user.getUserId());
        } else {
            item.setTechnicianName("");
            item.setTechnicianId(0L);
        }

        //查询是否为正常批次
        BizLot lot = new BizLot();
        lot.setId(item.getLotId());
        lot = bizLotService.getAllList(lot).get(0);
        if (lot.getIsWaste() == 1) {
            throw new RuntimeException("该批次已批废");
        }
        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        }
        lot.setAllCarrierName(item.getCarrierName());
        lot.setDayworkItemId(item.getId());
        lot.setCarrierCode(dayworkCarrier.getCarrierCode());
        lot.setCarrierId(dayworkCarrier.getCarrierId());
        lot.setProcessStepNumber(item.getProcessStepNumber());
        lot.setPudName(item.getProdNum() == 0 ? lot.getProductionQuantity() : item.getProdNum());
        // 获取外协工序列表
        BizOutsourcedOrderDetail detail = bizOutsourcedOrderDetailService.getById(detailList.get(0).getOutsourceDetailId());
        lot.setOutsourceOrderDetailId(detail != null ? detail.getId() : null);
        lot.setProcessList(detail != null ? detail.getProcessNames() : "");
        // 判断是否已经进行外协检测
        lot.setHasInspection(bizProcessInspecionService.query().eq("outsource_order_detail_id", detail == null ? 0L : detail.getId()).count() > 0 ? 1: 0);
        return success(lot);
    }

    //解绑检测载具
    @PostMapping("/unbindInspectionCarrier")
    @Log(title = "解绑检测载具", businessType = BusinessType.UPDATE)
    public AjaxResult unbindInspectionCarrier(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        List<BizDayworkCarrier> carrierList = bizDayworkCarrierService.query().eq("carrier_code", dayworkCarrier.getCarrierCode()).eq("is_changed", 0).list();
        //操作检查表
        List<BizProcessInspecion> list = bizProcessInspecionService.query()
                .eq("inspection_carrier_code", dayworkCarrier.getCarrierCode())
                .eq("is_inspection_carrier_changed", 0)
                .list();
        //判断如果是出厂检查创建者和最新那道序所有报工人员能领取
        if (list.stream().anyMatch(v -> v.getType().equals("factoryInspection"))) {
            Long dayworkId = carrierList.get(0).getDayworkId();
            List<BizDayworkItem> dayworkItemList = bizDayworkItemService.query().eq("daywork_id", dayworkId).orderByDesc("create_time").list();
            String lastProcessStepNumber = dayworkItemList.get(0).getProcessStepNumber();
            List<BizDayworkItem> lastDayworkItemList = dayworkItemList.stream().filter(item -> item.getProcessStepNumber().equals(lastProcessStepNumber)).collect(Collectors.toList());
            List<Long> lastUserIds = lastDayworkItemList.stream().map(BizDayworkItem::getUserId).distinct().collect(Collectors.toList());
            lastUserIds.add(list.get(0).getCreatorId());
            //当前登陆者
            Long loginUserId = SecurityUtils.getLoginUser().getUserId();
            if (lastUserIds.contains(loginUserId)) {
                carrierList.forEach(item -> {
                    item.setIsChanged(1);
                });
                list.forEach(item -> {
                    item.setIsInspectionCarrierChanged(1);
                    bizProcessInspecionService.updateBatchById(list);
                });
                return success(bizDayworkCarrierService.updateBatchById(carrierList));
            }
            return error("您未参与当前批次的加工，无法领取");
        } else {
            //检查选择的操作者能领取
            List<Long> lastUserIds = list.stream().map(BizProcessInspecion::getUserId).distinct().collect(Collectors.toList());
            //当前登陆者
            Long loginUserId = SecurityUtils.getLoginUser().getUserId();
            if (lastUserIds.contains(loginUserId)) {
                carrierList.forEach(item -> {
                    item.setIsChanged(1);
                });
                list.forEach(item -> {
                    item.setIsInspectionCarrierChanged(1);
                    bizProcessInspecionService.updateBatchById(list);
                });
                return success(bizDayworkCarrierService.updateBatchById(carrierList));
            }
            return error("您不是当前检查的操作者，无法领取");
        }
    }


    /**
     * 根据箱号查询批次信息
     */
//    @PreAuthorize("@ss.hasPermi('business:inspection:list')")
    @PostMapping("/getLotInfo")
    @Log(title = "根据箱号查询批次信息", businessType = BusinessType.SELECT)
    public AjaxResult getLotInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        //根据箱号查询当前绑定信息
        dayworkCarrier = bizDayworkCarrierService.getByCarrierCode(dayworkCarrier);
        if (dayworkCarrier == null) {
            return error("该箱号未绑定任何批次");
        }

        //获取报工信息
        BizDayworkCarrier finalDayworkCarrier = dayworkCarrier;
        List<BizDayworkItem> itemList = bizDayworkItemService.getByDayworkId(dayworkCarrier.getDayworkId()).stream().filter(item -> !item.getStatus().equals("0") && item.getDeptId().equals(finalDayworkCarrier.getDeptId())).collect(Collectors.toList());
        if (itemList.size() == 0) {
            return error("该批次当前工段没有报工");
        }
        BizDayworkItem item = itemList.get(0);
        List<BizDayworkItem> latestList = itemList.stream().filter(v -> v.getProcessStepNumber().equals(item.getProcessStepNumber())).collect(Collectors.toList());

        BizProductionPlanDetail productionPlanDetail = productionPlanDetailService.getById(item.getProductionPlanDetailId());
        BizProduct product = productService.getById(productionPlanDetail.getProductId());
        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
        if (user != null) {
            item.setTechnicianName(user.getNickName());
            item.setTechnicianId(user.getUserId());
        } else {
            item.setTechnicianName("");
            item.setTechnicianId(0L);
        }

        //查询是否为正常批次
        BizLot lot = new BizLot();
        lot.setId(item.getLotId());
        lot = bizLotService.getAllList(lot).get(0);
        if (lot.getIsWaste() == 1) {
            return error("该批次已批废");
        }
        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
            lot.setProcessCode(process.getProcessCode());
            lot.setProcessId(process.getProcessId());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
            lot.setProcessCode(process.getProcessCode());
            lot.setProcessId(process.getProcessId());
        }
        lot.setAllCarrierName(item.getCarrierName());
        if (latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("0")) {
            if (!latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("1") && !latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("2")) {
                return error("该批次未开始加工");
            }
        }
        if (latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("3")) {
            return error("当前批次，工序已完成");
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date date = new Date();
        String startTime = simpleDateFormat.format(date) + " 00:00:00";
        String endTime = simpleDateFormat.format(date) + " 23:59:59";
        List<BizDayworkItem> collect = latestList.stream().filter(v -> v.getStatus().equals("1") || v.getStatus().equals("2")).collect(Collectors.toList());
        lot.setDayworkItemList(collect.stream().filter(info -> {
            try {
                if (info.getEndTime() == null) {
                    // 如果为 null，可以选择返回 false（排除该项），或者根据需要处理
                    return true;
                }
                // 确保 info 的状态为 "1" 或 "2"，且结束时间不晚于当天结束时间
                // 并且结束时间不早于当天开始时间
                return info.getEndTime().before(sdf.parse(endTime))
                        && info.getEndTime().after(sdf.parse(startTime));
            } catch (ParseException e) {
                throw new RuntimeException(e);
            }
        }).collect(Collectors.toList()));
        lot.setDeptName(item.getDeptName());
        lot.setDayworkItemId(item.getId());
        lot.setProcessStepNumber(item.getProcessStepNumber());
        lot.setCarrierCode(dayworkCarrier.getCarrierCode());
        lot.setCarrierId(dayworkCarrier.getCarrierId());
        lot.setPudName(item.getProdNum() == 0 ? lot.getProductionQuantity() : item.getProdNum());
        return success(lot);
    }

    //巡检专用
    @PostMapping("/getPatrolLotInfo")
    @Log(title = "获取巡检批次信息", businessType = BusinessType.SELECT)
    public AjaxResult getPatrolLotInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        //根据箱号查询当前绑定信息
        Long deptId = dayworkCarrier.getDeptId();
        dayworkCarrier = bizDayworkCarrierService.getByCarrierCode(dayworkCarrier);
        if (dayworkCarrier == null) {
            return error("该箱号未绑定任何批次");
        }

        //获取报工信息

        List<BizDayworkItem> itemList = bizDayworkItemService.getByDayworkId(dayworkCarrier.getDayworkId()).stream().filter(item -> !item.getStatus().equals("0") && item.getDeptId().equals(deptId)).collect(Collectors.toList());
        if (itemList.size() == 0) {
            return error("该批次当前工段没有报工");
        }
        BizDayworkItem item = itemList.get(0);
        List<BizDayworkItem> latestList = itemList.stream().filter(v -> v.getProcessStepNumber().equals(item.getProcessStepNumber())).collect(Collectors.toList());

        BizProductionPlanDetail productionPlanDetail = productionPlanDetailService.getById(item.getProductionPlanDetailId());
        BizProduct product = productService.getById(productionPlanDetail.getProductId());
        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
        if (user != null) {
            item.setTechnicianName(user.getNickName());
            item.setTechnicianId(user.getUserId());
        } else {
            item.setTechnicianName("");
            item.setTechnicianId(0L);
        }

        //查询是否为正常批次
        BizLot lot = new BizLot();
        lot.setId(item.getLotId());
        lot = bizLotService.getAllList(lot).get(0);
        if (lot.getIsWaste() == 1) {
            return error("该批次已批废");
        }
        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
            lot.setProcessCode(process.getProcessCode());
            lot.setProcessId(process.getProcessId());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
            lot.setProcessCode(process.getProcessCode());
            lot.setProcessId(process.getProcessId());
        }
        lot.setAllCarrierName(item.getCarrierName());
        if (latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("0")) {
            if (!latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("1") && !latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("2")) {
                return error("该批次未开始加工");
            }
        }
        if (latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("3")) {
            return error("当前批次，工序已完成");
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date date = new Date();
        String startTime = simpleDateFormat.format(date) + " 00:00:00";
        String endTime = simpleDateFormat.format(date) + " 23:59:59";
        List<BizDayworkItem> collect = latestList.stream().filter(v -> v.getStatus().equals("1") || v.getStatus().equals("2")).collect(Collectors.toList());
        lot.setDayworkItemList(collect.stream().filter(info -> {
            try {
//                if (info.getEndTime() == null) {
//                    // 如果为 null，可以选择返回 false（排除该项），或者根据需要处理
//                    return true;
//                }
                // 确保 info 的状态为 "1" 或 "2"，且结束时间不晚于当天结束时间
                // 并且结束时间不早于当天开始时间
                return info.getStartTime().before(sdf.parse(endTime))
                        && info.getStartTime().after(sdf.parse(startTime));
            } catch (ParseException e) {
                throw new RuntimeException(e);
            }
        }).collect(Collectors.toList()));
        lot.setDeptName(item.getDeptName());
        lot.setDayworkItemId(item.getId());
        lot.setProcessStepNumber(item.getProcessStepNumber());
        lot.setCarrierCode(dayworkCarrier.getCarrierCode());
        lot.setCarrierId(dayworkCarrier.getCarrierId());
        lot.setPudName(item.getProdNum() == 0 ? lot.getProductionQuantity() : item.getProdNum());
        return success(lot);
    }

    @PostMapping("/getFirstInspectionLotInfo")
    @Log(title = "获取首序检批次信息", businessType = BusinessType.SELECT)
    public AjaxResult getFirstInspectionLotInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        //根据箱号查询当前绑定信息
        Long deptId = dayworkCarrier.getDeptId();
        dayworkCarrier = bizDayworkCarrierService.getByCarrierCode(dayworkCarrier);
        if (dayworkCarrier == null) {
            return error("该箱号未绑定任何批次");
        }

        //获取报工信息

        List<BizDayworkItem> itemList = bizDayworkItemService.getByDayworkId(dayworkCarrier.getDayworkId()).stream().filter(item -> !item.getStatus().equals("0") && item.getDeptId().equals(deptId)).collect(Collectors.toList());
        if (itemList.size() == 0) {
            return error("该批次当前工段没有报工");
        }
        BizDayworkItem item = itemList.get(0);
        List<BizDayworkItem> latestList = itemList.stream().filter(v -> v.getProcessStepNumber().equals(item.getProcessStepNumber())).collect(Collectors.toList());

        BizProductionPlanDetail productionPlanDetail = productionPlanDetailService.getById(item.getProductionPlanDetailId());
        BizProduct product = productService.getById(productionPlanDetail.getProductId());
        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
        if (user != null) {
            item.setTechnicianName(user.getNickName());
            item.setTechnicianId(user.getUserId());
        } else {
            item.setTechnicianName("");
            item.setTechnicianId(0L);
        }

        //查询是否为正常批次
        BizLot lot = new BizLot();
        lot.setId(item.getLotId());
        lot = bizLotService.getAllList(lot).get(0);
        if (lot.getIsWaste() == 1) {
            return error("该批次已批废");
        }
        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
            lot.setProcessCode(process.getProcessCode());
            lot.setProcessId(process.getProcessId());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
            lot.setProcessCode(process.getProcessCode());
            lot.setProcessId(process.getProcessId());
        }
        lot.setAllCarrierName(item.getCarrierName());
        if (latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("0")) {
            if (!latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("1") && !latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("2")) {
                return error("该批次未开始加工");
            }
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date date = new Date();
        String startTime = simpleDateFormat.format(date) + " 00:00:00";
        String endTime = simpleDateFormat.format(date) + " 23:59:59";
        List<BizDayworkItem> collect = latestList.stream().filter(v -> v.getStatus().equals("1") || v.getStatus().equals("2") || v.getStatus().equals("3")).collect(Collectors.toList());
        lot.setDayworkItemList(collect);
        lot.setDeptName(item.getDeptName());
        lot.setDayworkItemId(item.getId());
        lot.setProcessStepNumber(item.getProcessStepNumber());
        lot.setCarrierCode(dayworkCarrier.getCarrierCode());
        lot.setCarrierId(dayworkCarrier.getCarrierId());
        lot.setPudName(item.getProdNum() == 0 ? lot.getProductionQuantity() : item.getProdNum());
        return success(lot);
    }

    //交检专用
    @PostMapping("/getDeliveryLotInfo")
    @Log(title = "获取交检批次信息", businessType = BusinessType.SELECT)
    public AjaxResult getDeliveryLotInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        //根据箱号查询当前绑定信息
        dayworkCarrier = bizDayworkCarrierService.getByCarrierCode(dayworkCarrier);
        if (dayworkCarrier == null) {
            return error("该箱号未绑定任何批次");
        }

        //获取报工信息
        BizDayworkCarrier finalDayworkCarrier = dayworkCarrier;
        List<BizDayworkItem> itemList = bizDayworkItemService.getByDayworkId(dayworkCarrier.getDayworkId()).stream().filter(item -> !item.getStatus().equals("0") && item.getDeptId().equals(finalDayworkCarrier.getDeptId())).collect(Collectors.toList());
        if (itemList.size() == 0) {
            return error("该批次当前工段没有报工");
        }
        BizDayworkItem item = itemList.get(0);
        List<BizDayworkItem> latestList = itemList.stream().filter(v -> v.getProcessStepNumber().equals(item.getProcessStepNumber())).collect(Collectors.toList());

        BizProductionPlanDetail productionPlanDetail = productionPlanDetailService.getById(item.getProductionPlanDetailId());
        BizProduct product = productService.getById(productionPlanDetail.getProductId());
        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
        if (user != null) {
            item.setTechnicianName(user.getNickName());
            item.setTechnicianId(user.getUserId());
        } else {
            item.setTechnicianName("");
            item.setTechnicianId(0L);
        }

        //查询是否为正常批次
        BizLot lot = new BizLot();
        lot.setId(item.getLotId());
        lot = bizLotService.getAllList(lot).get(0);
        if (lot.getIsWaste() == 1) {
            return error("该批次已批废");
        }
        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
            lot.setProcessCode(process.getProcessCode());
            lot.setProcessId(process.getProcessId());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
            lot.setProcessCode(process.getProcessCode());
            lot.setProcessId(process.getProcessId());
        }
        lot.setAllCarrierName(item.getCarrierName());
        if (latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("0")) {
            if (!latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("1") && !latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("2")) {
                return error("该批次未开始加工,不可交检");
            }
        }
        if (latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("4") || latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("5") || latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("7")) {
            return error("当前批次已经周转");
        }
//        if (!latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("3")) {
//            return error("当前工序未完成，不能交检");
//        }
        List<BizDayworkItem> collect = latestList.stream().filter(v -> v.getStatus().equals("1") || v.getStatus().equals("2") || v.getStatus().equals("3")).collect(Collectors.toList());
        lot.setDayworkItemList(collect);
        lot.setDeptName(item.getDeptName());
        lot.setDayworkItemId(item.getId());
        lot.setProcessStepNumber(item.getProcessStepNumber());
        lot.setCarrierCode(dayworkCarrier.getCarrierCode());
        lot.setCarrierId(dayworkCarrier.getCarrierId());
        lot.setPudName(item.getProdNum() == 0 ? lot.getProductionQuantity() : item.getProdNum());
        return success(lot);
    }

    //首件检专用
    @PostMapping("/getLotInfoByFirstInspection")
    @Log(title = "或许首序检批次信息", businessType = BusinessType.SELECT)
    public AjaxResult getLotInfoByFirstInspection(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        //获取报工信息
        List<BizDayworkItem> itemList = bizDayworkItemService.getByDayworkId(dayworkCarrier.getDayworkId())
                .stream().filter(item -> !item.getStatus().equals("0") && item.getDeptId().equals(dayworkCarrier.getDeptId())).collect(Collectors.toList());
        if (itemList.size() == 0) {
            return error("该批次当前工段没有报工");
        }
        BizDayworkItem item = itemList.get(0);
        List<BizDayworkItem> latestList = itemList.stream().filter(v -> v.getProcessStepNumber().equals(item.getProcessStepNumber()) && item.getUserId().equals(SecurityUtils.getLoginUser().getUserId())).collect(Collectors.toList());

        BizProductionPlanDetail productionPlanDetail = productionPlanDetailService.getById(item.getProductionPlanDetailId());
        BizProduct product = productService.getById(productionPlanDetail.getProductId());
        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
        if (user != null) {
            item.setTechnicianName(user.getNickName());
            item.setTechnicianId(user.getUserId());
        } else {
            item.setTechnicianName("");
            item.setTechnicianId(0L);
        }

        //查询是否为正常批次
        BizLot lot = new BizLot();
        lot.setId(item.getLotId());
        lot = bizLotService.getAllList(lot).get(0);
        if (lot.getIsWaste() == 1) {
            return error("该批次已批废");
        }
        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
            lot.setProcessCode(process.getProcessCode());
            lot.setProcessId(process.getProcessId());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
            lot.setProcessCode(process.getProcessCode());
            lot.setProcessId(process.getProcessId());
        }
        lot.setAllCarrierName(item.getCarrierName());
        if (latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("0")) {
            if (!latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("1") && !latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("2")) {
                return error("该批次未开始加工");
            }
        }
        if (latestList.stream().map(BizDayworkItem::getStatus).collect(Collectors.toList()).contains("3")) {
            return error("当前批次，工序已完成");
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date date = new Date();
        String startTime = simpleDateFormat.format(date) + " 00:00:00";
        String endTime = simpleDateFormat.format(date) + " 23:59:59";
        List<BizDayworkItem> collect = latestList.stream().filter(v -> v.getStatus().equals("1") || v.getStatus().equals("2")).collect(Collectors.toList());
        lot.setDayworkItemList(collect.stream().filter(info -> {
            try {
//                if (info.getEndTime() == null) {
//                    // 如果为 null，可以选择返回 false（排除该项），或者根据需要处理
//                    return true;
//                }
                // 确保 info 的状态为 "1" 或 "2"，且开始时间不晚于当天结束时间
                // 并且开始时间不早于当天开始时间
                return info.getStartTime().before(sdf.parse(endTime))
                        && info.getStartTime().after(sdf.parse(startTime));
            } catch (ParseException e) {
                throw new RuntimeException(e);
            }
        }).collect(Collectors.toList()));
        //设置仪器室
        List<BizInspectionChamber> chamberList = bizInspectionChamberService.getList(new BizInspectionChamber());
        lot.setInspectionChamberList(chamberList);
        lot.setUserId(SecurityUtils.getLoginUser().getUserId());
        lot.setNickName(SecurityUtils.getLoginUser().getUser().getNickName());
        lot.setDeptName(item.getDeptName());
        lot.setDayworkItemId(item.getId());
        lot.setProcessStepNumber(item.getProcessStepNumber());
        List<BizDayworkCarrier> dayworkCarrierList = bizDayworkCarrierService.query().eq("daywork_id", dayworkCarrier.getDayworkId()).eq("is_changed", 0).list();
        if (!dayworkCarrierList.isEmpty()) {
            //如果绑了多个载具，只返回一个
            lot.setCarrierCode(dayworkCarrierList.get(0).getCarrierCode());
            lot.setCarrierId(dayworkCarrierList.get(0).getCarrierId());
        }
        lot.setPudName(item.getProdNum() == 0 ? lot.getProductionQuantity() : item.getProdNum());
        return success(lot);
    }

    @PostMapping("/getLotInfoByFactoryInspection")
    @Log(title = "获取出厂检批次信息", businessType = BusinessType.SELECT)
    public AjaxResult getLotInfoByFactoryInspection(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        //根据箱号查询当前绑定信息
        dayworkCarrier = bizDayworkCarrierService.getByCarrierCode(dayworkCarrier);
        if (dayworkCarrier == null) {
            return error("该箱号未绑定任何批次");
        }

        //获取报工信息
        List<BizDayworkItem> itemList = bizDayworkItemService.getByDayworkId(dayworkCarrier.getDayworkId());
        if (itemList.size() == 0) {
            return error("该批次未开始加工");
        }
        BizDayworkItem item = itemList.get(0);
        BizProductionPlanDetail productionPlanDetail = productionPlanDetailService.getById(item.getProductionPlanDetailId());
        BizProduct product = productService.getById(productionPlanDetail.getProductId());
        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
        if (user != null) {
            item.setTechnicianName(user.getNickName());
            item.setTechnicianId(user.getUserId());
        } else {
            item.setTechnicianName("");
            item.setTechnicianId(0L);
        }

        //查询是否为正常批次
        BizLot lot = new BizLot();
        lot.setId(item.getLotId());
        lot = bizLotService.getAllList(lot).get(0);
        if (lot.getIsWaste() == 1) {
            return error("该批次已批废");
        }
        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
            lot.setProcessCode(process.getProcessCode());
            lot.setProcessId(process.getProcessId());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
            lot.setProcessCode(process.getProcessCode());
            lot.setProcessId(process.getProcessId());
        }
        lot.setAllCarrierName(item.getCarrierName());
        lot.setDeptName(item.getDeptName());
        lot.setDayworkItemId(item.getId());
        lot.setProcessStepNumber(item.getProcessStepNumber());
        lot.setCarrierCode(dayworkCarrier.getCarrierCode());
        lot.setCarrierId(dayworkCarrier.getCarrierId());
        lot.setPudName(item.getProdNum() == 0 ? lot.getProductionQuantity() : item.getProdNum());
        return success(lot);
    }

//    /**
//     * 根据箱号查询批次信息
//     */
//    @PostMapping("/getLotInfo")
//    public AjaxResult getLotOrCarrierInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
//
//        /** 因为需要绑定检查载具所以需要先查询出载具信息 **/
//        List<BizCarrier> carriers=bizCarrierService.query().eq("code",dayworkCarrier.getCarrierCode()).list();
//        if(carriers.isEmpty()){
//            return error("箱号信息错误，未找到该载具");
//        }
//        BizCarrier carrier=carriers.get(0);
//        //根据箱号查询当前绑定信息
//        dayworkCarrier = bizDayworkCarrierService.getByCarrierCode(dayworkCarrier);
//        /** 判断载具类型，是检查载具并且没有 **/
//        if (dayworkCarrier == null) {
//            return error("该箱号未绑定任何批次");
//        }
//
//        //获取报工信息
//        List<BizDayworkItem> itemList = bizDayworkItemService.getByDayworkId(dayworkCarrier.getDayworkId());
//        if (itemList.size() == 0) {
//            return error("改批次未开始加工");
//        }
//        BizDayworkItem item = itemList.get(0);
//        BizProductionPlanDetail productionPlanDetail = productionPlanDetailService.getById(item.getProductionPlanDetailId());
//        BizProduct product = productService.getById(productionPlanDetail.getProductId());
//        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
//        if (user != null) {
//            item.setTechnicianName(user.getNickName());
//            item.setTechnicianId(user.getUserId());
//        } else {
//            item.setTechnicianName("");
//            item.setTechnicianId(0L);
//        }
//
//        //查询是否为正常批次
//        BizLot lot = new BizLot();
//        lot.setId(item.getLotId());
//        lot = bizLotService.getAllList(lot).get(0);
//        if (lot.getIsWaste() == 1) {
//            return error("改批次已批废");
//        }
//        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
//            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
//            lot.setProcessAlias(process.getProcessAlias());
//        } else {
//            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
//            lot.setProcessAlias(process.getProcessAlias());
//        }
//        lot.setAllCarrierName(item.getCarrierName());
//        lot.setDayworkItemId(item.getId());
//        lot.setCarrierCode(dayworkCarrier.getCarrierCode());
//        lot.setCarrierId(dayworkCarrier.getCarrierId());
//        lot.setPudName(item.getProdNum() == 0 ? lot.getProductionQuantity() : item.getProdNum());
//        return success(lot);
//    }


//

    /**
     * 导出序检主列表
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:export')")
    @Log(title = "导出序检列表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProcessInspecion);
        List<BizProcessInspecion> list = bizProcessInspecionService.getList(bizProcessInspecion);
        ExcelUtil<BizProcessInspecion> util = new ExcelUtil<BizProcessInspecion>(BizProcessInspecion.class);
        util.exportExcel(response, list, "序检主数据");
    }

    /**
     * 获取序检主详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:query')")
    @GetMapping(value = "/{id}")
    @Log(title = "获取序检信息", businessType = BusinessType.SELECT)
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bizProcessInspecionService.getById(id));
    }

    /**
     * 新增序检主
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:add')")
    @Log(title = "新增序检", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProcessInspecion);
        return toAjax(bizProcessInspecionService.save(bizProcessInspecion));
    }

    /**
     * 修改序检主
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:edit')")
    @Log(title = "编辑序检", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProcessInspecion);
        return toAjax(bizProcessInspecionService.updateById(bizProcessInspecion));
    }

    /**
     * 删除序检主
     */
    @Log(title = "删除序检", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    @Transactional
    public AjaxResult remove(@PathVariable List<Long> ids) {
        List<BizProcessInspectionDetail> bizProcessInspectionDetails = processInspectionDetailService.query().in("process_inspection_id", ids).list();
        List<BizDayworkCarrier> carriers = bizDayworkCarrierService.query().in("process_inspection_id", ids).list();
        List<BizDayworkItemReject> rejects = bizDayworkItemRejectService.query().in("process_inspection_id", ids).list();
        List<BizDayworkItemConsult> consults = bizDayworkItemConsultService.query().in("process_inspection_id", ids).list();
        if (consults.size() > 0) {
            List<BizDayworkItemConsultPicture> consultPictures = bizDayworkItemConsultPictureService.query().in("daywork_item_consult_id", consults.stream().map(BizDayworkItemConsult::getId).collect(Collectors.toList())).list();
            bizDayworkItemConsultPictureService.removeBatchByIds(consultPictures);
        }
        processInspectionDetailService.removeBatchByIds(bizProcessInspectionDetails);
        bizDayworkItemRejectService.removeBatchByIds(rejects);
        bizDayworkCarrierService.removeBatchByIds(carriers);
        bizDayworkItemConsultService.removeBatchByIds(consults);
        return toAjax(bizProcessInspecionService.removeBatchByIds(ids));
    }

    /**
     * 保存序检箱
     */
    @Log(title = "序检换箱", businessType = BusinessType.INSERT)
    @PostMapping("/saveInspectionCarrier")
    public AjaxResult saveInspectionCarrier(@RequestBody BizProcessInspecion bizProcessInspecion) {
        List<BizDayworkCarrier> bizDayworkCarriers = bizDayworkCarrierService.query().eq("process_inspection_id", bizProcessInspecion.getId()).eq("is_changed", 0).list();
        BizProcessInspecion current = bizProcessInspecionService.getById(bizProcessInspecion.getId());
        List<BizDayworkCarrier> newCarriers = bizProcessInspecion.getDayworkCarriers();
        if (bizProcessInspecion.getStatus().equals(0) && newCarriers.isEmpty()) {
            return error("该序检未完成，不能解绑载具");
        }
        List<BizCarrierReject> carrierRejects = bizProcessInspecion.getRejectCarrier();
        List<BizCarrier> carriers = bizCarrierService.query().in("id", carrierRejects.isEmpty() ?
                Collections.singletonList(0L) :
                carrierRejects.stream().map(BizCarrierReject::getCarrierId)
                        .collect(Collectors.toList())).list();
        carriers.forEach(l -> l.setIsAbandoned(1));
        bizCarrierService.updateBatchById(carriers);
        bizCarrierRejectService.saveBatch(carrierRejects);
        bizDayworkCarriers.forEach(l -> l.setIsChanged(1));
        bizDayworkCarrierService.updateBatchById(bizDayworkCarriers);
        if (newCarriers.isEmpty()) {
            current.setIsInspectionCarrierChanged(1);
            current.setInspectionCarrierChangedTime(new Date());
            bizProcessInspecionService.updateById(current);
        } else {
            current.setIsInspectionCarrierChanged(0);
            current.setInspectionCarrierChangedTime(null);
            current.setInspectionCarrierId(newCarriers.get(0).getCarrierId());
            current.setInspectionCarrierCode(newCarriers.get(0).getCarrierCode());
            bizProcessInspecionService.updateById(current);
        }
        newCarriers.forEach(l -> {
            l.setId(null);
            l.setIsChanged(0);
            l.setProcessInspectionId(bizProcessInspecion.getId());
            l.setDayworkId(bizProcessInspecion.getDayworkId());
            l.setProductionPlanId(bizProcessInspecion.getProductionPlanId());
            l.setLotId(bizProcessInspecion.getLotId());
        });
        bizDayworkCarrierService.saveOrUpdateBatch(newCarriers);
        return success();
    }

    /**
     * 根据载具编码获取关联
     */
    @Log(title = "根据载具编码获取关联检查", businessType = BusinessType.SELECT)
    @PostMapping("/getRelateInspectionByCarrier")
    public AjaxResult getRelateInspectionByCarrier(@RequestBody BizProcessInspecion bizProcessInspection) {
        String carrierCode = bizProcessInspection.getCarrierCode();
        List<BizCarrier> carrierList = bizCarrierService.query().eq("code", carrierCode).list();
        BizDayworkCarrier dayworkCarrier = new BizDayworkCarrier();
        dayworkCarrier.setCarrierCode(carrierCode);
        dayworkCarrier.setIsChanged(0);

        if (carrierList.isEmpty()) {
            return error("未找到该箱号");
        }
        BizCarrier carrier = carrierList.get(0);
        //查询箱号是否废弃
        List<BizCarrierReject> carrierReject = bizCarrierRejectService.query().eq("carrier_id", carrier.getId()).eq("deleted", 0).orderByDesc("create_time").list();
        if (!carrierReject.isEmpty() &&carrierReject.get(0).getIsAbandoned() ==1) {
            return error("该载具已废弃");
        }

        List<BizDayworkCarrier> list = bizDayworkCarrierService.getInfo(dayworkCarrier);
        //List<BizDayworkCarrier> list = bizDayworkCarrierService.query().eq("carrier_code", dayworkCarrier.getCarrierCode()).eq("is_changed",0).list();
        List<BizDaywork> dayworks = bizDayworkService.query().in("id", list.isEmpty() ? Collections.singletonList(0L) : list.stream().map(BizDayworkCarrier::getDayworkId).collect(Collectors.toList())).list();
        if (carrier.getIsInspection().equals(1)) {
            // 为检查载具返回检查载具信息
            return error("请勿使用检查载具");
        } else {
            if (list.size() == 0) {
                throw new RuntimeException("该箱号未绑定任何批次。");
            }
        }
        List<BizProductionPlanDetail> planByDaywork = bizProductionPlanDetailService.query().in("id", dayworks.stream().map(BizDaywork::getProductionPlanDetailId).collect(Collectors.toList())).list();
        BizProductionPlanDetail planByInspection = bizProductionPlanDetailService.query().eq("id", bizProcessInspection.getProductionPlanDetailId() == null ? 0L : bizProcessInspection.getProductionPlanDetailId()).one();
        // 判断这些批次是否和之前的是同产品同工艺同图号
        if (planByDaywork.stream().anyMatch(v -> !v.getProductCode().equals(planByInspection.getProductCode()) )) {
            throw new RuntimeException("添加批次的产品与检测批次不同，无法添加。");
        }
        // 根据批次获取所有的检测信息
        List<BizProcessInspecion> processInspecions = new ArrayList<>();
        dayworks.forEach(l -> {
            List<BizDayworkCarrier> itemCarrier = bizDayworkCarrierService.query().eq("daywork_id", l.getId()).eq("process_inspection_id", 0L).eq("is_changed", 0).list();
            BizProcessInspecion processInspecion = new BizProcessInspecion();
            List<String> codes = new ArrayList<>();
            codes.add("27110");
            codes.add("28030");
            codes.add("28080");
            codes.add("27070");
            List<String> processCodes = bizProcessInspection.getProcessCode() == null ? codes : bizProcessInspection.getProcessCode();
            System.out.println("查询工序编码：processCodes" + processCodes);
            //根据箱号查询当前绑定信息
            //根据dayworkId查询外协回收信息是否有该批次
            BizReturnReceiptDetail bizReturnReceiptDetail = new BizReturnReceiptDetail();
            bizReturnReceiptDetail.setDayworkId(l.getId());
            List<BizReturnReceiptDetail> detailList = bizReturnReceiptDetailService.getList(bizReturnReceiptDetail);
            List<BizOutsourcedOrderDetailProcess> processes = bizOutsourcedOrderDetailProcessService.query().in("detail_id", detailList.isEmpty() ? Collections.singletonList(0L) : detailList.stream().map(BizReturnReceiptDetail::getOutsourceDetailId).collect(Collectors.toList())).list();
            System.out.println("查询是否有该批次：detailList" + detailList);
            if (detailList.size() == 0) {
                throw new RuntimeException("该批次不是外协回收批次");
            }

            if (processes.stream().noneMatch(detail -> processCodes.stream().anyMatch(t -> t.trim().equals(detail.getProcessCode())))) {
                throw new RuntimeException("该批次不需要外协检查");
            }
//                        现在先不考虑 审核通过后才能外协检查限制先放到下一迭代 20240625
//        20240703 加回去
            if (detailList.stream().noneMatch(detail -> detail.getIsAudit().equals(1) && detail.getStatus().equals(1))) {
                throw new RuntimeException("该批次需要审核通过后才能外协检查");
            }

            detailList = detailList.stream()
                    .filter(detail -> processes.stream().anyMatch(v -> v.getTechnologicalProcessDetailId().equals(detail.getTechnologicalProcessDetailId())))
                    .filter(detail -> processCodes.stream().anyMatch(t -> t.trim().equals(detail.getProcessCode())) &&
                                    detail.getIsAudit().equals(1)
//                        现在先不考虑 审核通过后才能外协检查限制先放到下一迭代 20240625
//                        20240703加回去
                                    && detail.getStatus().equals(1)
                    )
                    .collect(Collectors.toList());
            //如果有多道工序，则根据工序的排序找到最后一条
            if (detailList.size() > 1) {
                //按照工序排序
                detailList.sort(Comparator.comparing(BizReturnReceiptDetail::getProcessStepNumber, Comparator.reverseOrder()));
            }
            //根据工序id查询报工信息
            BizDaywork daywork = new BizDaywork();
            daywork.setId(l.getId());
            daywork.setTechnologicalProcessDetailId(detailList.get(0).getTechnologicalProcessDetailId());

            List<BizDayworkItem> itemList = bizDayworkItemService.getByDaywork(daywork);
            if (itemList.size() == 0) {
                throw new RuntimeException("该批次未开始加工");
            }
            BizDayworkItem item = itemList.get(0);
            BizProductionPlanDetail productionPlanDetail = productionPlanDetailService.getById(item.getProductionPlanDetailId());
            BizProduct product = productService.getById(productionPlanDetail.getProductId());
            SysUser user = sysUserService.selectUserById(product.getTechnicianId());
            if (user != null) {
                item.setTechnicianName(user.getNickName());
                item.setTechnicianId(user.getUserId());
            } else {
                item.setTechnicianName("");
                item.setTechnicianId(0L);
            }

            //查询是否为正常批次
            BizLot lot = new BizLot();
            lot.setId(item.getLotId());
            lot = bizLotService.getAllList(lot).get(0);
            if (lot.getIsWaste() == 1) {
                throw new RuntimeException("该批次已批废");
            }
            if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
                BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
                lot.setProcessAlias(process.getProcessAlias());
            } else {
                BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
                lot.setProcessAlias(process.getProcessAlias());
            }
            lot.setAllCarrierName(item.getCarrierName());
            lot.setDayworkItemId(item.getId());
            lot.setCarrierCode(carrier.getCode());
            lot.setCarrierId(carrier.getId());
            lot.setProcessStepNumber(item.getProcessStepNumber());
            lot.setPudName(item.getProdNum() == 0 ? lot.getProductionQuantity() : item.getProdNum());
            // 获取外协工序列表
            BizOutsourcedOrderDetail detail = bizOutsourcedOrderDetailService.getById(detailList.get(0).getOutsourceDetailId());

            // 判断该批次是否已经有外协检查单，假设已有则不能进行关联
            BizProcessInspecion previousItem = bizProcessInspecionService.query().eq("outsource_order_detail_id", detail.getId()).ne("id", processInspecion.getId() == null ? 0L : processInspecion.getId()).last("limit 1").one();

            lot.setOutsourceOrderDetailId(detail != null ? detail.getId() : null);
            lot.setProcessList(detail != null ? detail.getProcessNames() : "");
            String beginCode = "WDD" + DateTime.now().toString("yyMM");
            String previous = redisCache.hasKey(processInspectionNoKey) ? redisCache.getCacheObject(processInspectionNoKey) : "";
            String codeValue = "";
            if (previous.isEmpty()) {
                List<BizProcessInspecion> processInspectionList = bizProcessInspecionService.getAllList();
                if(!processInspectionList.isEmpty()) {
                    codeValue = processInspectionList.get(0).getOutsourcedInspectionNo();
                    int lastFourDigits = Integer.parseInt(codeValue.substring(7, 13));
                    lastFourDigits ++; // 加1
                    codeValue = codeValue.substring(0, 7) + String.format("%06d", lastFourDigits);
                    previous = codeValue;
                }else{
                    previous = (beginCode + "000001");
                }
            } else {
                codeValue = previous.startsWith(beginCode) ? previous : beginCode + "000000";
                int lastFourDigits = Integer.parseInt(codeValue.substring(7, 13));
                lastFourDigits ++; // 加1
                codeValue = codeValue.substring(0, 7) + String.format("%06d", lastFourDigits);
                previous = codeValue;
            }
            processInspecion.setOutsourcedInspectionNo(previous);
            redisCache.setCacheObject(processInspectionNoKey, previous);
//        }
            processInspecion.setReturnReceiptDetailId(bizReturnReceiptDetail.getId());
//        processInspecion.setUserId(user == null ? 0L : user.getUserId());
//        processInspecion.setNickName(user == null ? "" : user.getNickName());
            if (processInspecion.getId() == null) {
                processInspecion.setCarrierId(carrier.getId());
                processInspecion.setCarrierCode(carrier.getCode());
            }
            processInspecion.setDayworkId(l.getId());
            processInspecion.setProductionPlanId(l.getProductionPlanId());
            processInspecion.setDayworkItemId(item.getId());
            processInspecion.setProductionPlanDetailId(l.getProductionPlanDetailId());
            processInspecion.setProductionPlanDetailSubDetailId(l.getProductionPlanDetailSubDetailId());
            processInspecion.setIsOutsourcedInspection(1);
            processInspecion.setProcessStepNumber(item.getProcessStepNumber());
            processInspecion.setProductDescription(product.getDescription());
            processInspecion.setLotId(lot.getId());
            processInspecion.setLotCode(lot.getLotCode());
            processInspecion.setDeptId(l.getDeptId());
            processInspecion.setProductId(l.getProductId());
            processInspecion.setTechnologicalProcessId(l.getTechnologicalProcessId());
            processInspecion.setOutsourceOrderId(detail != null ? detail.getMasterId() : 0L);
            processInspecion.setReviewerId(SecurityUtils.getLoginUser().getUserId());
            processInspecion.setProcessStepNumber(detailList.get(0).getProcessStepNumber());
            processInspecion.setType("outsourcedInspector");
            processInspecion.setProcessAlias(detailList.get(0).getProcessAlias());
            processInspecion.setReturnReceiptDetailId(detailList.get(0).getId());
            processInspecion.setUserId(getUserId());
            processInspecion.setUserName(getLoginUser().getUser().getUserName());
            if (detail != null) {
                processInspecion.setOutsourceOrderDetailId(detail.getId());
            }
            processInspecion.setProcessNames(lot.getProcessList());
            processInspecion.setAllCarriers(itemCarrier.stream().map(BizDayworkCarrier::getCarrierCode).collect(Collectors.joining(",")));
//
//            if (processInspecion.getStartTime() == null) {
//                processInspecion.setStartTime(new Date());
//            }
//            if (processInspecion.getStatus() != 0) {
//                processInspecion.setEndTime(new Date());
//                processInspecion.setFirstUpdaterId(processInspecion.getFirstUpdaterId() == null || processInspecion.getFirstUpdaterId().equals(0L) ? user != null ? user.getUserId() : 0L : processInspecion.getFirstUpdaterId());
//            }

            if (previousItem == null) {
                processInspecions.add(processInspecion);
            }
        });
        return processInspecions.isEmpty() ? error("载具中不存在未质检批次") : success(processInspecions);
    }

    @Log(title = "保存外协检查单与关联检查单")
    @PostMapping("/saveOutsourceInspectionWithRelate")
    @Transactional
    public AjaxResult saveOutsourceInspectionWithRelate(@RequestBody BizProcessInspecion processInspecion) {
        Map<String, Object> map = getRelationInspection(processInspecion);
        BizProcessInspecion bizProcessInspecion = (BizProcessInspecion) map.get("master");
        List<BizProcessInspecion> relateList = (List<BizProcessInspecion>) map.get("detail");
        // 保存主检测单信息
        saveMasterInspection(bizProcessInspecion);
        // 保存从检测单信息
        relateList.forEach(l -> {
            l = saveRelateInspection(bizProcessInspecion, l);
        });
        // 保存咨询信息
        List<BizDayworkItemConsult> consults = new ArrayList<>(processInspecion.getDayworkItemConsults() == null ? new ArrayList<>() : processInspecion.getDayworkItemConsults());
        SysUser user = sysUserService.selectUserById(processInspecion.getUser().getUserId());
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());
        if (consults.size() > 0) {
            for (BizDayworkItemConsult consult : consults) {
                if (bizProcessInspecion.getId().equals(processInspecion.getId())) {
                    consult.setProductionPlanId(bizProcessInspecion.getProductionPlanId());
                    consult.setProductionPlanDetailId(bizProcessInspecion.getProductionPlanDetailId());
                    consult.setDayworkItemId(bizProcessInspecion.getDayworkItemId());
                    consult.setDayworkId(bizProcessInspecion.getDayworkId());
                } else {
                    BizProcessInspecion item = relateList.stream().filter(t -> t.getId().equals(processInspecion.getId())).findFirst().orElse(null);
                    consult.setProductionPlanId(item == null ? 0 : item.getProductionPlanId());
                    consult.setProductionPlanDetailId(item == null ? 0 : item.getProductionPlanDetailId());
                    consult.setDayworkItemId(item == null ? 0 : item.getDayworkItemId());
                    consult.setDayworkId(item == null ? 0 : item.getDayworkId());
                }
                consult.setUserId(user == null ? 0L : user.getUserId());
                consult.setTenantId(processInspecion.getTenantId());
                consult.setNickName(user == null ? "" : user.getNickName());
                consult.setProcessInspectionId(processInspecion.getId());
//                consult.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                consult.setLotId(processInspecion.getLot().getId());
                consult.setLotCode(processInspecion.getLot().getLotCode());
                consult.setProductId(processInspecion.getLot().getProductId());
                consult.setProductDescription(processInspecion.getLot().getProductDescription());
                consult.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                consult.setTechnologicalProcessDetailId(processInspecion.getDayworkItem().getTechnologicalProcessDetailId());
                consult.setProcessId(processInspecion.getDayworkItem().getProcessId());
                consult.setProcessAlias(processInspecion.getLot().getProcessAlias());
                consult.setIsOutsourcedInspection(1);
                //consult.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
            }
        }
        bizDayworkItemConsultService.saveOrUpdateBatch(consults);
        List<BizDayworkItemConsultPicture> pictures = new ArrayList<>();
        consults.forEach(ne -> {
            if(ne.getPictures() != null) {
            ne.getPictures().forEach(info -> {
                BizDayworkItemConsultPicture consultPicture = new BizDayworkItemConsultPicture();
                consultPicture.setDayworkItemConsultId(ne.getId());
                consultPicture.setPictureName(info.getPictureName());
                consultPicture.setUrl(info.getUrl());
                consultPicture.setTenantId(processInspecion.getTenantId());
                pictures.add(consultPicture);
            });
            }
    });
        bizDayworkItemConsultPictureService.saveBatch(pictures);
        // 保存废品信息
        // 保存图片信息
        if (processInspecion.getId() != null) {
            //保存废品信息
            List<BizProcessInspectionDetail> rejects = processInspecion.getRejects();
            //保存前先删除原先废品信息
            BizDayworkItemReject rej = new BizDayworkItemReject();
            rej.setProcessInspectionId(processInspecion.getId());
            bizDayworkItemRejectService.removeReject(rej);
            BizProcessInspectionDetail inspectionDetail = new BizProcessInspectionDetail();
            inspectionDetail.setProcessInspectionId(processInspecion.getId());
            processInspectionDetailService.physicalDelete(inspectionDetail);
            BizDayworkItemReject newRej = new BizDayworkItemReject();
            newRej.setRejectNum(0L);

            if (rejects != null && rejects.size() > 0) {
                for (BizProcessInspectionDetail reject : rejects) {
                    reject.setId(null);
                    reject.setUserId(user == null ? 0L : user.getUserId());
                    reject.setTenantId(processInspecion.getTenantId());
                    reject.setProcessInspectionId(processInspecion.getId());
                    reject.setDayworkId(0L);
                    reject.setLotId(processInspecion.getLot().getId());
                    Long rejectNum = reject.getRejectNum() == null ? 0L : reject.getRejectNum();
                    newRej.setRejectNum(rejectNum);
                    newRej.setUserId(user == null ? 0L : user.getUserId());
                    newRej.setProcessInspectionId(processInspecion.getId());
                    newRej.setTenantId(processInspecion.getTenantId());
                    newRej.setDayworkId(0L);
                    newRej.setLotId(processInspecion.getLot().getId());
                    newRej.setReason("外协检查废品");
                    newRej.setDeptId(processInspecion.getDeptId());
                    if (bizProcessInspecion.getId().equals(processInspecion.getId())) {
                        reject.setProductionPlanId(bizProcessInspecion.getProductionPlanId());
                        reject.setProductionPlanDetailId(bizProcessInspecion.getProductionPlanDetailId());
                        reject.setProductionPlanDetailSubDetailId(bizProcessInspecion.getProductionPlanDetailSubDetailId());
                        reject.setDayworkItemId(bizProcessInspecion.getDayworkItem().getId());

                        newRej.setProductionPlanId(bizProcessInspecion.getProductionPlanId());
                        newRej.setProductionPlanDetailId(bizProcessInspecion.getProductionPlanDetailId());
                        newRej.setProductionPlanDetailSubDetailId(bizProcessInspecion.getProductionPlanDetailSubDetailId());
                        newRej.setDayworkItemId(bizProcessInspecion.getId());
                    } else {
                        BizProcessInspecion item = relateList.stream().filter(t -> t.getId().equals(processInspecion.getId())).findFirst().orElse(null);
                        reject.setProductionPlanId(item == null ? 0L : item.getProductionPlanId());
                        reject.setProductionPlanDetailId(item == null ? 0L : item.getProductionPlanDetailId());
                        reject.setProductionPlanDetailSubDetailId(item == null ? 0L : item.getProductionPlanDetailSubDetailId());
                        reject.setDayworkItemId(item == null ? 0L : item.getDayworkItem().getId());
                        newRej.setProductionPlanId(item == null ? 0L : item.getProductionPlanId());
                        newRej.setProductionPlanDetailId(item == null ? 0L : item.getProductionPlanDetailId());
                        newRej.setProductionPlanDetailSubDetailId(item == null ? 0L : item.getProductionPlanDetailSubDetailId());
                        newRej.setDayworkItemId(item == null ? 0L : item.getId());
                    }
                }
            }
            processInspectionDetailService.saveBatch(rejects);
            bizDayworkItemRejectService.save(newRej);
            //保存上传图片信息
            List<BizProcessInspectionPicture> oldProcessInspectionList = bizProcessInspectionPictureService.query().eq("process_inspection_id", processInspecion.getId()).list();
            if (processInspecion.getProcessInspectionPictureList() != null && !processInspecion.getProcessInspectionPictureList().isEmpty()) {
                processInspecion.getProcessInspectionPictureList().forEach(item -> {
                    item.setProcessInspectionId(processInspecion.getId());
                });
            }
            saveOrUpdateBatch(bizProcessInspectionPictureService, processInspecion.getProcessInspectionPictureList() == null || processInspecion.getProcessInspectionPictureList().isEmpty() ? new ArrayList<>() : processInspecion.getProcessInspectionPictureList(), oldProcessInspectionList);
        }
        // 保存关联信息

        if (processInspecion.getRelatedId() == null || processInspecion.getRelatedId().equals(0L)) {
            // 判断关联信息是否为空
            if (relateList != null && !relateList.isEmpty()) {
                BizProcessInspectionRelation relation = new BizProcessInspectionRelation();
                relation.setLotId(processInspecion.getLotId());
                relation.setProcessInspectionId(processInspecion.getId());
                relation.setTenantId(processInspecion.getTenantId());
                bizProcessInspectionRelationService.save(relation);
                List<BizProcessInspectionRelationDetail> relatedDetails = new ArrayList<>();
                relateList.forEach(l -> {
                    BizProcessInspectionRelationDetail item = new BizProcessInspectionRelationDetail();
                    item.setProcessInspectionId(l.getId());
                    item.setLotId(l.getLotId());
                    item.setTenantId(processInspecion.getTenantId());
                    item.setProcessInspectionRelationId(relation.getId());
                    relatedDetails.add(item);
                    l.setRelatedId(relation.getId());
                });
                BizProcessInspectionRelationDetail master = new BizProcessInspectionRelationDetail();
                master.setProcessInspectionId(processInspecion.getId());
                master.setLotId(processInspecion.getLotId());
                master.setProcessInspectionRelationId(relation.getId());
                master.setTenantId(processInspecion.getTenantId());
                master.setIsMaster(1);
                relatedDetails.add(master);
                bizProcessInspectionRelationDetailService.saveBatch(relatedDetails);
                bizProcessInspecionService.updateById(processInspecion);
                processInspecion.setRelatedId(relation.getId());
                bizProcessInspecionService.updateBatchById(relateList);
                bizProcessInspecionService.updateById(processInspecion);
            }
        } else {
            List<BizProcessInspectionRelationDetail> details = bizProcessInspectionRelationDetailService.query().eq("process_inspection_relation_id", processInspecion.getRelatedId()).list();
            List<BizProcessInspectionRelationDetail> add = new ArrayList<>();
            List<BizProcessInspectionRelationDetail> remove = details.stream().filter(v -> relateList.stream().noneMatch(t -> t.getId().equals(v.getProcessInspectionId())) && !v.getProcessInspectionId().equals(bizProcessInspecion.getId())).collect(Collectors.toList());
            if(!remove.isEmpty()){
                List<BizProcessInspecion> processInspectionRemove = bizProcessInspecionService.query().in("id", remove.stream().map(BizProcessInspectionRelationDetail::getProcessInspectionId).collect(Collectors.toList())).list();
                bizProcessInspecionService.removeBatchByIds(processInspectionRemove);
            }
            relateList.forEach(l -> {
                if (details.stream().noneMatch(t -> t.getProcessInspectionId().equals(l.getId()))) {
                    BizProcessInspectionRelationDetail item = new BizProcessInspectionRelationDetail();
                    item.setProcessInspectionId(l.getId());
                    item.setLotId(l.getLotId());
                    item.setProcessInspectionRelationId(processInspecion.getRelatedId());
                    add.add(item);
                }
                l.setRelatedId(processInspecion.getRelatedId());
            });
            List<BizProcessInspecion> disconnection = bizProcessInspecionService.query().in("id", remove.isEmpty() ? Collections.singletonList(0L) : remove.stream().map(BizProcessInspectionRelationDetail::getProcessInspectionId).collect(Collectors.toList())).list();
            disconnection.forEach(l -> l.setRelatedId(0L));
            bizProcessInspecionService.updateBatchById(disconnection);

            bizProcessInspectionRelationDetailService.removeBatchByIds(remove);
            bizProcessInspectionRelationDetailService.saveBatch(add);
            bizProcessInspecionService.updateBatchById(relateList);
        }
        return success();

    }

    public Map<String, Object> getRelationInspection(BizProcessInspecion bizProcessInspecion) {
        List<BizProcessInspecion> bizProcessInspecions = new ArrayList<>();
        Map<String, Object> map = new HashMap<>();
        if (bizProcessInspecion.getRelatedId() == null || bizProcessInspecion.getRelatedId().equals(0L)) {
            map.put("master", bizProcessInspecion);
            map.put("detail", bizProcessInspecion.getRelateList());
        } else {
            BizProcessInspectionRelation relation = bizProcessInspectionRelationService.query().eq("id", bizProcessInspecion.getRelatedId()).one();
            if (relation == null || relation.getProcessInspectionId().equals(bizProcessInspecion.getId())) {
                map.put("master", bizProcessInspecion);
                map.put("detail", bizProcessInspecion.getRelateList());
            } else {
                BizProcessInspecion master = bizProcessInspecion.getRelateList().stream().filter(t -> relation.getProcessInspectionId().equals(t.getId())).findFirst().orElse(null);
                if (master != null) {
                    master.setLot(bizLotService.getById(master.getLotId()));
//                    master.setRejects(bizProcessInspecion.getRejects());
//                    master.setDayworkItemConsults(bizProcessInspecion.getDayworkItemConsults());
                    master.setFlag(bizProcessInspecion.getFlag());
                    master.setStatus(bizProcessInspecion.getStatus());
                }
                map.put("master", master);
                bizProcessInspecions.add(bizProcessInspecion);
                bizProcessInspecions.addAll(bizProcessInspecion.getRelateList().stream().filter(t -> !relation.getProcessInspectionId().equals(t.getId())).collect(Collectors.toList()));
                map.put("detail", bizProcessInspecions);
            }
        }
        return map;
    }

    public boolean saveMasterInspection(BizProcessInspecion processInspecion) {
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getDayworkItemId());
//        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getDayworkItemId());
        BizReturnReceiptDetail bizReturnReceiptDetail = bizReturnReceiptDetailService.query().eq("daywork_id", dayworkItem.getDayworkId()).eq("technological_process_detail_id", dayworkItem.getTechnologicalProcessDetailId()).eq("deleted", 0).one();
        SysUser user = sysUserService.selectUserById(getUserId());

//        SysUser user = sysUserService.selectUserById(processInspecion.getUser().getUserId());

        // 判断是否已经进行过质检
        BizProcessInspecion previousItem = bizProcessInspecionService.query().eq("outsource_order_detail_id", bizReturnReceiptDetail.getOutsourceDetailId()).isNull("recheck_date").ne("id", processInspecion.getId() == null ? 0L : processInspecion.getId()).one();
        BizOutsourcedOrderDetail outsourceOrderDetail = bizOutsourcedOrderDetailService.query().eq("id", bizReturnReceiptDetail.getOutsourceDetailId()).one();
        String beginCode = "WDD" + DateTime.now().toString("yyMM");
        String previous = redisCache.hasKey(processInspectionNoKey) ? redisCache.getCacheObject(processInspectionNoKey) : "";
        String codeValue = "";
        if (previous.isEmpty()) {
            List<BizProcessInspecion> processInspectionList = bizProcessInspecionService.getAllList();
            if(!processInspectionList.isEmpty()) {
                codeValue = processInspectionList.get(0).getOutsourcedInspectionNo();
                int lastFourDigits = Integer.parseInt(codeValue.substring(7, 13));
                lastFourDigits ++; // 加1
                codeValue = codeValue.substring(0, 7) + String.format("%06d", lastFourDigits);
                previous = codeValue;
            }else{
                previous = (beginCode + "000001");
            }
        } else {
            codeValue = previous.startsWith(beginCode) ? previous : beginCode + "000000";
            int lastFourDigits = Integer.parseInt(codeValue.substring(7, 13));
            lastFourDigits ++; // 加1
            codeValue = codeValue.substring(0, 7) + String.format("%06d", lastFourDigits);
            previous = codeValue;
        }
        processInspecion.setOutsourcedInspectionNo(previous);
        redisCache.setCacheObject(processInspectionNoKey, previous);
//        }
        processInspecion.setReturnReceiptDetailId(bizReturnReceiptDetail.getId());
//        processInspecion.setUserId(user == null ? 0L : user.getUserId());
//        processInspecion.setNickName(user == null ? "" : user.getNickName());
        if (processInspecion.getId() == null) {
            processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
            processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        }
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setIsOutsourcedInspection(1);
        processInspecion.setProcessStepNumber(dayworkItem.getProcessStepNumber());
        processInspecion.setProductDescription(processInspecion.getLot().getProductDescription());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        processInspecion.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
        processInspecion.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        processInspecion.setOutsourceOrderId(outsourceOrderDetail.getMasterId());
        processInspecion.setReviewerId(SecurityUtils.getLoginUser().getUserId());
        processInspecion.setProcessStepNumber(processInspecion.getLot().getProcessStepNumber());
        processInspecion.setType("outsourcedInspector");
        processInspecion.setDayworkItem(dayworkItem);

        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }
        if (processInspecion.getInspectionStatus() != 0) {
            processInspecion.setEndTime(new Date());
            processInspecion.setFirstUpdaterId(processInspecion.getFirstUpdaterId() == null || processInspecion.getFirstUpdaterId().equals(0L) ? user != null ? user.getUserId() : 0L : processInspecion.getFirstUpdaterId());

        }
        if (!checkOutsourceDayworkItem(processInspecion.getOutsourceOrderDetailId())) {
            throw new RuntimeException("编辑失败，下序已经生产加工");
        }

        BizCarrier carrier = bizCarrierService.query().eq("id", processInspecion.getInspectionCarrierId()).one();
        List<BizDayworkCarrier> dayworkCarriers = new ArrayList<>();
        if (carrier != null) {
            dayworkCarriers = bizDayworkCarrierService.query().eq("carrier_id", carrier.getId()).eq("is_changed", 0).list();
            if (dayworkCarriers.stream().anyMatch(v -> !v.getDayworkId().equals(daywork.getId()))) {
                throw new RuntimeException("检测载具已绑定其他批次");
            }
        }
        if (previousItem != null && previousItem.getAuditStatus() == 1) {
            throw new RuntimeException("已通过结算审核，请取消后再进行质检。");
        }
        Boolean isSave = false;
        if (processInspecion.getId() == null) {
            isSave = true;
        }
        //保存
        if (previousItem != null) {
            previousItem.setRecheckDate(DateTime.now().toJdkDate());
            bizProcessInspecionService.updateById(previousItem);
        }
        bizProcessInspecionService.saveOrUpdate(processInspecion);
        //移交仪器室
        if (processInspecion.getFlag() != null && processInspecion.getFlag()) {
            BizProcessInspecion inspection = new BizProcessInspecion();
            inspection.setUserId(processInspecion.getLot().getUserId());
            inspection.setTenantId(processInspecion.getTenantId());
            inspection.setNickName(processInspecion.getLot().getNickName());
            inspection.setCarrierId(processInspecion.getCarrierId());
            inspection.setCarrierCode(processInspecion.getCarrierCode());
            inspection.setDayworkId(daywork.getId());
            inspection.setProcessStepNumber(processInspecion.getLot().getProcessStepNumber());
            inspection.setProductionPlanId(daywork.getProductionPlanId());
            inspection.setDayworkItemId(dayworkItem.getId());
            inspection.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
            inspection.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
            inspection.setProductDescription(processInspecion.getLot().getProductDescription());
            inspection.setLotId(processInspecion.getLot().getId());
            inspection.setLotCode(processInspecion.getLot().getLotCode());
            inspection.setDeptId(daywork.getDeptId());
            inspection.setProductId(processInspecion.getLot().getProductId());
            inspection.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
            inspection.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
            inspection.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
            inspection.setProcessStepNumber(processInspecion.getProcessStepNumber());
            inspection.setEquipmentDetailCode(processInspecion.getLot().getEquipmentDetailCode());
            inspection.setEquipmentDetailId(processInspecion.getLot().getEquipmentDetailId());
            inspection.setInspectionChamberId(processInspecion.getInspectionChamberId());
            //inspection.setReviewerId(SecurityUtils.getLoginUser().getUserId());
            inspection.setProcessStepNumber(processInspecion.getProcessStepNumber());
            inspection.setFromId(processInspecion.getId());
            inspection.setType("instrumentRoomInspection");
            if (inspection.getStartTime() == null) {
                inspection.setStartTime(new Date());
            }
            bizProcessInspecionService.save(inspection);
            //绑定检测载具
            BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
            newDayworkCarrier.setDayworkId(inspection.getDayworkId());
            newDayworkCarrier.setTenantId(processInspecion.getTenantId());
            newDayworkCarrier.setCarrierCode(inspection.getInspectionCarrierCode());
            newDayworkCarrier.setCarrierId(inspection.getInspectionCarrierId());
            newDayworkCarrier.setLotCode(inspection.getLotCode());
            newDayworkCarrier.setLotId(inspection.getLotId());
            newDayworkCarrier.setProductionPlanDetailId(inspection.getProductionPlanDetailId());
            newDayworkCarrier.setProcessInspectionId(inspection.getId());
            bizDayworkCarrierService.save(newDayworkCarrier);
        }
        if (isSave) {
            if (dayworkCarriers.stream().noneMatch(v -> v.getDayworkId().equals(daywork.getId())) && (processInspecion.getIsInspectionCarrierChanged() == null || processInspecion.getIsInspectionCarrierChanged().equals(0))) {
                // 该检测载具需要绑定到当前批次
                BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
                newDayworkCarrier.setDayworkId(processInspecion.getDayworkId());
                newDayworkCarrier.setCarrierCode(carrier.getCode());
                newDayworkCarrier.setCarrierId(carrier.getId());
                newDayworkCarrier.setTenantId(processInspecion.getTenantId());
                newDayworkCarrier.setLotCode(processInspecion.getLotCode());
                newDayworkCarrier.setLotId(processInspecion.getLotId());
                newDayworkCarrier.setProductionPlanDetailId(processInspecion.getProductionPlanDetailId());
                newDayworkCarrier.setProcessInspectionId(processInspecion.getId());
                bizDayworkCarrierService.save(newDayworkCarrier);
            }
        }
        //修改daywaork信息
        dayworkItem.setTenantId(processInspecion.getTenantId());
        dayworkItem.setIsProcessInspection(1);
        if (dayworkItem.getDeleted() == 1) {
            return true;
        }
        return bizDayworkItemService.saveOrUpdate(dayworkItem);
    }

    public BizProcessInspecion saveRelateInspection(BizProcessInspecion processInspecion, BizProcessInspecion related) {
        BizDayworkItem dayworkItem = bizDayworkItemService.selById(related.getDayworkItemId());
//        BizDayworkItem dayworkItem = bizDayworkItemService.getById(related.getDayworkItemId());
        dayworkItem.setIsProcessInspection(1);
        dayworkItem.setTenantId(processInspecion.getTenantId());
        related.setDeptId(processInspecion.getDeptId());
        related.setStartTime(processInspecion.getStartTime());
        related.setEndTime(processInspecion.getEndTime());
        related.setStatus(processInspecion.getStatus());
        related.setFirstUpdaterId(processInspecion.getFirstUpdaterId());
        related.setTenantId(processInspecion.getTenantId());
        bizProcessInspecionService.saveOrUpdate(related);
        related.setDayworkItem(dayworkItem);
        if (dayworkItem.getDeleted() == 0) {
            bizDayworkItemService.saveOrUpdate(dayworkItem);
        }

        return related;
    }
}
